package datavisualizer;

import javax.swing.JTable;
import javax.swing.JPanel;
import java.awt.Dimension;
import java.awt.Color;
import javawrapper.ModelException;

/**
 * Produces a table of harvest or mortality episode results.  A harvest table
 * requires the detailed output file to have data saved from the "Harvest
 * Results" grid; for mortality episodes, the "Mortality Episode Results" grid.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 * <br>June 29, 2004:  Added disturbance results (LEM)
 */

public class HarvestDataRequest
    extends DataRequest {

  /**The density cut for this timestep for each species*/
  private float[] mp_fDensity;
  /**The basal area cut for this timestep for each species*/
  private float[] mp_fBasalArea;

  /**Translates a data member code to an index in the density array.  The code
   * matches the location in these arrays; that location's value is the index
   * to one of the above arrays.*/
  private int[] mp_iDensityIndexes;

  /**Translates a data member code to an index in the basal area array.  The
   * code matches the location in these arrays; that location's value is the
   * index to one of the above arrays.*/
  private int[] mp_iBasalAreaIndexes;

  /**Column headers, to be consistent between displays*/
  private String[] mp_sColumnNames = {
        "Species", "Density (#/ha)", "Basal Area (sq. m/ha)"};

  /**Keep our species names ready to display, as well*/
  private String[] mp_sSpeciesNames;

  /**Name of the grid from which the data will be collected.*/
  private String m_sThisGrid;

  /**Plot area, in hectares.*/
  private float m_fPlotAreaInHectares;

  /**The number of species*/
  private int m_iNumSpecies,
      /**Number of allowed cut ranges*/
      m_iNumCutRanges = 4,
      /**We want to throw away harvest type - capture its data member code
             so we'll recognize it*/
      m_iHarvestTypeCode = -1;

  /**
   * Constructor
   * @param sChartName Name of the chart
   * @param oManager Detailed output file manager
   * @throws ModelException not really, but I have to declare this.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>June 29, 2004: Dynamically deciding grid name (LEM)
   */
  public HarvestDataRequest(String sChartName, DetailedOutputFileManager oManager) throws
      ModelException {
    super(sChartName, oManager);

    if (sChartName.indexOf("Harvest") > -1) {
      m_sThisGrid = "Harvest Results";
    }
    else {
      m_sThisGrid = "Mortality Episode Results";

    }
    m_iNumSpecies = oManager.GetNumberOfSpecies();
    m_fPlotAreaInHectares = oManager.GetPlotArea();

    //Get our species display names
    mp_sSpeciesNames = new String[oManager.GetNumberOfSpecies()];
    Legend oLegend = oManager.GetLegend();
    for (int i = 0; i < mp_sSpeciesNames.length; i++) {
      mp_sSpeciesNames[i] = oLegend.GetSpeciesDisplayName(i);
    }

    ClearData();
  }

  /**
   * Writes the table's data to tab-delimited text.
   * @param jOut java.io.FileWriter The file to write to.
   * @throws java.io.IOException if there's a problem writing the file.
   */
  protected void WriteChartDataToFile(java.io.FileWriter jOut) throws java.io.
      IOException {

      float fBasalAreaTotal = 0, fDensityTotal = 0;
      int i;

      //Write column headers
      jOut.write(mp_sColumnNames[0] + "\t" +
                 mp_sColumnNames[1] + "\t" +
                 mp_sColumnNames[2] + "\n");

      //Write data
      for (i = 0; i < m_iNumSpecies; i++) {
        jOut.write(mp_sSpeciesNames[i] + "\t" +
                   String.valueOf(mp_fDensity[i] / m_fPlotAreaInHectares) + "\t" +
                   String.valueOf(mp_fBasalArea[i] / m_fPlotAreaInHectares) + "\n");
        fBasalAreaTotal += mp_fBasalArea[i];
        fDensityTotal += mp_fDensity[i];
      }

      //Write totals
      jOut.write("Total:\t" +
                 String.valueOf(fDensityTotal / m_fPlotAreaInHectares) + "\t" +
                 String.valueOf(fBasalAreaTotal / m_fPlotAreaInHectares) + "\n\n");
  }

  /**
   * If this is from a Results grid, it is assumed to be a basal area
   * value.  The index vector is consulted and the values added to the
   * appropriate array location.
   * @param sGridName Name of the grid for this data
   * @param iX X number of the cell from which this value came
   * @param iY Y number of the cell from which this value came
   * @param iCode Data member code of this value.
   * @param fVal Value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddGridFloatData(String sGridName, int iX, int iY, int iCode,
                               float fVal) {
    if (sGridName.equals(m_sThisGrid)) {
      mp_fBasalArea[mp_iBasalAreaIndexes[iCode]] += fVal;
    }
  }

  /**
   * If this is from a Results grid, it is assumed to be a density value.  The
   * index vector is consulted and the values added to the appropriate array
   * location.
   * @param sGridName Name of the grid for this data
   * @param iX X number of the cell from which this value came
   * @param iY Y number of the cell from which this value came
   * @param iCode Data member code of this value.
   * @param iVal Value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddGridIntData(String sGridName, int iX, int iY, int iCode,
                             int iVal) {
    if (sGridName.equals(m_sThisGrid)) {
      if (iCode != m_iHarvestTypeCode) {
        mp_fDensity[mp_iDensityIndexes[iCode]] += iVal;
      }
    }
  }

  /**
   * This wants floats.
   * @return True.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public boolean WantAnyGridFloats() {
    return true;
  }

  /**
   * This wants ints.
   * @return True.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public boolean WantAnyGridInts() {
    return true;
  }

  /**
   * Accepts a grid float data member code for future reference when passed
   * float data members.
   * @param sGridName Name of the grid
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddGridFloatDataMemberCode(String sGridName, String sLabel,
                                         int iCode) {
    if (sGridName.equals(m_sThisGrid)) {

      //Get the species value
      int iSpecies = GetSpeciesFromDataMemberLabel(sLabel);
      if (iSpecies == -1) {
        return;
      }

      mp_iBasalAreaIndexes[iCode] = iSpecies;
    }
  }

  /**
   * Accepts a grid int data member code for future reference when passed int
   * data members.
   * @param sGridName Name of the grid
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddGridIntDataMemberCode(String sGridName, String sLabel,
                                       int iCode) {
    if (sGridName.equals(m_sThisGrid)) {

      //Get the species value
      int iSpecies = GetSpeciesFromDataMemberLabel(sLabel);
      if (iSpecies == -1) {
        if (sLabel.equals("Harvest Type")) {
          m_iHarvestTypeCode = iCode;
          return;
        }
        return;
      }

      mp_iDensityIndexes[iCode] = iSpecies;
    }
  }

  /**
   * Gets the species from a data member label which ends in "_x", where x is
   * the species number.
   * @param sLabel The data member label.
   * @return The species number, or -1 if none exists.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  private int GetSpeciesFromDataMemberLabel(String sLabel) {
    int iPos = sLabel.lastIndexOf("_");
    if (iPos == -1) {
      return -1;
    }
    return new Integer(sLabel.substring(iPos + 1)).intValue();
  }

  /**
   * Creates a table of the results that have been collected.
   * @param oLegend The species legend.
   * @param sChartTitle The chart title.
   * @return A JInternalFrame with the table present in it.
   * @throws datavisualizer.ModelException
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  ModelInternalFrame DrawChart(Legend oLegend, String sChartTitle) throws
      ModelException {

    m_oChartFrame = new ModelInternalFrame(sChartTitle, oLegend);
    m_oChartFrame.setContentPane(CreateTable(oLegend));
    m_oChartFrame = DataGrapher.AddFileMenu(m_oChartFrame, this, m_bShowOneTimestep);
    return m_oChartFrame;
  }

  void UpdateChart(Legend oLegend) throws ModelException {

    m_oChartFrame.setContentPane(CreateTable(oLegend));
  }

  /**
   * Creates the results table.
   * @param oLegend Legend for this chart.
   * @return Panel containing the table.
   * @throws ModelException If anything goes wrong with the drawing.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  private JPanel CreateTable(Legend oLegend) throws ModelException {
    java.text.NumberFormat jFormat = java.text.NumberFormat.getInstance();
    jFormat.setMaximumFractionDigits(3);
    jFormat.setGroupingUsed(false);

    float fBasalAreaTotal = 0, fDensityTotal = 0;
    int i;
    for (i = 0; i < m_iNumSpecies; i++) {
      fBasalAreaTotal += mp_fBasalArea[i];
      fDensityTotal += mp_fDensity[i];
    }

    //Create our table data array
    Object[][] p_oData = new Object[m_iNumSpecies + 1][];
    for (i = 0; i < m_iNumSpecies; i++) {
      p_oData[i] = new Object[3];

      p_oData[i][0] = oLegend.GetSpeciesDisplayName(i);
      //Make the density and basal area values in per hectare units
      p_oData[i][1] = jFormat.format(mp_fDensity[i] / m_fPlotAreaInHectares);
      p_oData[i][2] = jFormat.format(mp_fBasalArea[i] / m_fPlotAreaInHectares);
    }
    p_oData[m_iNumSpecies] = new Object[3];
    p_oData[m_iNumSpecies][0] = "Total";
    p_oData[m_iNumSpecies][1] = jFormat.format(fDensityTotal / m_fPlotAreaInHectares);
    p_oData[m_iNumSpecies][2] = jFormat.format(fBasalAreaTotal /
                                          m_fPlotAreaInHectares);

    JTable jTable = new JTable(p_oData, mp_sColumnNames);
    jTable.setName("ResultsTable");
    jTable.setGridColor(Color.WHITE);
    jTable.setBackground(Color.WHITE);
    jTable.setFont(new javawrapper.ModelFont());
    jTable.getTableHeader().setFont(new javawrapper.ModelFont());
    jTable.setPreferredSize(new Dimension(500,
                                          (int) jTable.getPreferredSize().height));

    //Create the panel and add the table to it.
    JPanel jPanel = new JPanel(new java.awt.BorderLayout());
    jPanel.add(jTable.getTableHeader(), java.awt.BorderLayout.NORTH);
    jPanel.add(jTable, java.awt.BorderLayout.CENTER);
    jPanel.setBackground(Color.WHITE);
    jPanel.add(jTable);
    return jPanel;
  }

  /**
   * Clears out existing data.
   * @throws ModelException Doesn't throw an exception.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  void ClearData() throws ModelException {
    int i, iIndexSize = m_iNumSpecies * m_iNumCutRanges + 1; //+1 for harvest type
    mp_iDensityIndexes = null;
    mp_iBasalAreaIndexes = null;

    mp_iDensityIndexes = new int[iIndexSize];
    mp_iBasalAreaIndexes = new int[iIndexSize];

    for (i = 0; i < iIndexSize; i++) {
      mp_iDensityIndexes[i] = -1;
      mp_iBasalAreaIndexes[i] = -1;
    }

    mp_fDensity = null;
    mp_fBasalArea = null;
    mp_fDensity = new float[m_iNumSpecies];
    mp_fBasalArea = new float[m_iNumSpecies];

    for (i = 0; i < m_iNumSpecies; i++) {
      mp_fDensity[i] = 0;
      mp_fBasalArea[i] = 0;
    }
  }
}
