package datavisualizer;

import org.jfree.data.xy.DefaultXYDataset;
import javax.swing.*;

/**
* Creates different types of line graphs for detailed output files.  It can
* produce absolute and relative graphs of density, basal area, or volume for
* saplings, adults, and snags; and absolute and relative graphs of density
* alone for seedlings.
*
* This chart is different from other charts in that it displays data from all
* timesteps at once, instead of one timestep at a time.  Thus, it tends to
* ignore normal chart drawing requests and has its own code to force parsing
* of all timestep files in a detailed output package at once.
*
* Technically, density data could be extracted from any tree data member,
* since counting up the total is all that is required.  I didn't do it that
* way because I'm a little lazy, and I'll wait for a hue and cry from the
* users before unnecessarily complicating my code.
*
* Copyright: Copyright (c) Charles D. Canham 2003</p>
* Company: Institute of Ecosystem Studies</p>
* @author Lora E. Murphy
* @version 1.1
*
* <br>Edit history:
* <br>------------------
* <br>November 15, 2004:  Created (LEM)
* <br>November 18, 2004:  Updated for real-time data visualization (LEM)
* <br>March 9, 2005:  Added line graph support for volume and made version
* 1.1 (LEM)
* <br>December 20, 2006: Added a checkbox to turn off totals (LEM)
* <br>January 5, 2007: Updated for upgrade to JFreeChart 1.0.3 (LEM)
*/
public class LineGraphDataRequest
    extends DataRequest {

  public final static int RELATIVE_BASAL_AREA = 1; /**<Chart type is relative basal area*/
  public final static int RELATIVE_DENSITY = 2; /**<Chart type is relative density*/
  public final static int ABSOLUTE_BASAL_AREA = 3; /**<Chart type is absolute basal area*/
  public final static int ABSOLUTE_DENSITY = 4; /**<Chart type is absolute density*/
  public final static int ABSOLUTE_VOLUME = 5; /**<Chart type is absolute volume*/
  public final static int RELATIVE_VOLUME = 6; /**<Chart type is relative volume*/

  /**The dataset with all species*/
  private DefaultXYDataset m_oDataset = null;

  /**Data array.  Array indexes are #1 - species, and #2 - timestep.*/
  private float[][] mp_fData;

  /**Data totals for absolute chart types.  Array index is number of timestep.*/
  private float[] mp_fTotal;

  /**Holds the data codes for the data being graphed.  For density for
   * seedlings, this is diam10; for density or basal area, it's DBH; for
   * volume, it's volume.  Array index is # of species.*/
  private int[] mp_iDataCode;

  /**A copy of which species are being displayed, so we know if the user
   * changed it*/
  private boolean[] mp_bWhichSpeciesShown;

  /** Checkbox for displaying the total. Might be null if not needed.*/
  private JCheckBox m_jUseTotals = null;

  /**Area of the plot, in hectares*/
  private float m_fPlotAreaInHectares;

  /**The current timestep.  This helps process an entire detailed output file.
   * It also lets this class know when to ignore offered data - whenever this
   * value is set to -1, some other chart event triggered the parse.*/
  private int m_iCurrentTimestep = -1;

  /**The total number of timesteps.  We keep track of this so we know if
   * something has changed (as in real-time data visualization) so we can
   * update appropriately.*/
  private int m_iNumTimesteps = -1;

  /**The type of chart - absolute or relative / basal area or density.*/
  private int m_iChartType;

  /**Tree type of chart - seedling - snag.*/
  private int m_iTreeType;

  /**
   * Constructor.  Declares all the arrays.
   * @param oManager Parent detailed output file manager
   * @param sChartName Name of the table being drawn.
   * @param iChartType Type of graph, absolute or relative / density or basal
   * area.
   * @param iTreeType Type of tree for which to draw this chart.
   */
  public LineGraphDataRequest(DetailedOutputFileManager oManager,
                              String sChartName, int iChartType, int iTreeType) {
    super(sChartName, oManager);

    if (iChartType == ABSOLUTE_BASAL_AREA) {
      m_iChartType = ABSOLUTE_BASAL_AREA;
      m_jUseTotals = new JCheckBox("Show Totals", true);
      m_jUseTotals.setFont(new javawrapper.ModelFont());
      m_jUseTotals.setActionCommand("total_checkbox");
      m_jUseTotals.addActionListener(this);
    }
    else if (iChartType == RELATIVE_BASAL_AREA) {
      m_iChartType = RELATIVE_BASAL_AREA;
    }
    else if (iChartType == RELATIVE_DENSITY) {
      m_iChartType = RELATIVE_DENSITY;
    }
    else if (iChartType == ABSOLUTE_VOLUME) {
      m_iChartType = ABSOLUTE_VOLUME;
      m_jUseTotals = new JCheckBox("Show Totals", true);
      m_jUseTotals.setFont(new javawrapper.ModelFont());
      m_jUseTotals.setActionCommand("total_checkbox");
      m_jUseTotals.addActionListener(this);
    }
    else if (iChartType == RELATIVE_VOLUME) {
      m_iChartType = RELATIVE_VOLUME;
    }
    else {
      m_iChartType = ABSOLUTE_DENSITY;
      m_jUseTotals = new JCheckBox("Show Totals", true);
      m_jUseTotals.setFont(new javawrapper.ModelFont());
      m_jUseTotals.setActionCommand("total_checkbox");
      m_jUseTotals.addActionListener(this);
    }

    if (iTreeType >= javawrapper.TreePopulation.SEEDLING &&
        iTreeType <= javawrapper.TreePopulation.SNAG) {
      m_iTreeType = iTreeType;
    }

    mp_iDataCode = new int[oManager.GetNumberOfSpecies()];
    int i;
    for (i = 0; i < mp_iDataCode.length; i++) {
      mp_iDataCode[i] = -1;
    }

    mp_bWhichSpeciesShown = new boolean[oManager.GetNumberOfSpecies()];
    for (i = 0; i < mp_bWhichSpeciesShown.length; i++) {
      mp_bWhichSpeciesShown[i] = false;
    }

    m_fPlotAreaInHectares = oManager.GetPlotArea();

    //Get the number of timesteps
    m_iNumTimesteps = oManager.GetNumberOfActualTimesteps();

    //Say that this is a chart that doesn't show only one timestep at a time
    m_bShowOneTimestep = false;
  }

  /**
   * Writes the line graph's data to tab-delimited text.
   * @param jOut java.io.FileWriter The file to write to.
   * @throws java.io.IOException if there is a problem writing the file.
   * @throws ModelException passed through from called methods if the
   * dataset doesn't make sense.
   */
  protected void WriteChartDataToFile(java.io.FileWriter jOut) throws java.io.
      IOException, javawrapper.ModelException {

    int i, j;
    boolean bUseTotals = (mp_fTotal != null &&
                          (m_iChartType == ABSOLUTE_BASAL_AREA ||
                           m_iChartType == ABSOLUTE_DENSITY ||
                           m_iChartType == ABSOLUTE_VOLUME));

    //Write the chart header
    jOut.write(m_oChartFrame.getTitle() + "\n");
    if (m_iChartType == ABSOLUTE_BASAL_AREA) {
      jOut.write("Values in square meters of basal area per hectare.\n");
    }
    else if (m_iChartType == ABSOLUTE_DENSITY) {
      jOut.write("Values in number of trees per hectare.\n");
    }
    else if (m_iChartType == ABSOLUTE_VOLUME) {
      jOut.write("Values in cubic meters per hectare.\n");
    }

    //Make sure there's data
    if (mp_fData == null) {
      jOut.write("This chart has no data.");
      return;
    }

    //Write column headers
    Legend oLegend = m_oManager.GetLegend();
    jOut.write("Timestep");
    for (i = 0; i < mp_fData.length; i++) {
      jOut.write("\t" + oLegend.GetSpeciesDisplayName(i));
    }
    if (bUseTotals) {
      jOut.write("\tTotal");
    }
    jOut.write("\n");

    //Write data - #1 = species, #2 = timestep
    for (i = 0; i < mp_fData[0].length; i++) {
      jOut.write(String.valueOf(i));
      for (j = 0; j < mp_fData.length; j++) {
        jOut.write("\t" + String.valueOf(mp_fData[j][i]));
      }
      if (bUseTotals) {
        jOut.write("\t" + mp_fTotal[i]);
      }
      jOut.write("\n");
    }
  }

  /**
   * This will update the chart only if the user has changed which species are
   * visible or unless the number of timesteps is different.
   * @param oLegend Legend The legend for this chart.
   * @throws ModelException Won't be thrown.
   */
  void UpdateChart(Legend oLegend) throws javawrapper.ModelException {

    //Check to see if we need to respond for real-time data visualization
    boolean bRealTime = false;
    if ( ( (DetailedOutputLegend) oLegend).GetNumberOfTimesteps() != m_iNumTimesteps) {
      //Set this right away to make sure that, when this gets called again in
      //MakeDataset, we don't enter a loop
      m_iNumTimesteps = ( (DetailedOutputLegend) oLegend).GetNumberOfTimesteps();
      bRealTime = true;
    }

    //Check to see if the current set of visible species is different
    boolean[] p_bCurrentlyVisible = new boolean[oLegend.GetNumberOfSpecies()];
    int i;
    boolean bDifferent = false;

    //Get the currently visible species
    for (i = 0; i < p_bCurrentlyVisible.length; i++) {
      p_bCurrentlyVisible[i] = oLegend.GetIsSpeciesSelected(i);
    }

    //Check to see if they're the same
    for (i = 0; i < p_bCurrentlyVisible.length; i++) {
      if (p_bCurrentlyVisible[i] != mp_bWhichSpeciesShown[i]) {
        bDifferent = true;
        break;
      }
    }

    if (bRealTime == false && bDifferent == false) {
      return; //nothing's different
    }

    //If there's realtime visualization, make a new dataset
    if (bRealTime || m_oDataset == null) {
      m_oDataset = MakeDataset( (DetailedOutputLegend) oLegend);
    }

    //If the species list is different, save the current set of visible species
    if (bDifferent == true) {

      //Save the current set of visible species
      for (i = 0; i < p_bCurrentlyVisible.length; i++) {
        mp_bWhichSpeciesShown[i] = p_bCurrentlyVisible[i];
      }
    }

    //Draw the new chart
    DefaultXYDataset oAdjustedDataset = AdjustVisibleSpecies(m_oDataset,
        oLegend);

    DataGrapher.UpdateLineChart(oAdjustedDataset, m_oChartFrame, oLegend, m_jUseTotals,
                                p_bCurrentlyVisible.length);
  }

  /**
   * Creates the chart dataset.
   * @param oLegend Legend The legend for this chart.
   * @return DefaultCategoryDataset The dataset for the chart, NOT adjusted for
   * which species are visible in the legend.
   * @throws ModelException if there is a problem creating the dataset.
   */
  private DefaultXYDataset MakeDataset(DetailedOutputLegend oLegend) throws
      javawrapper.ModelException {
    DetailedOutputFileManager oManager = oLegend.GetDetailedOutputFileManager();
    float fVal;
    int iNumTimesteps = oLegend.GetNumberOfTimesteps(),
        iTimestepToReturnTo = oLegend.GetCurrentTimestep(),
        iNumSpecies = oLegend.GetNumberOfSpecies(),
        i, j;

    //Declare the data arrays
    mp_fData = new float[iNumSpecies][iNumTimesteps + 1];
    mp_fTotal = new float[iNumTimesteps + 1];
    for (i = 0; i < mp_fData.length; i++) {
      for (j = 0; j < mp_fData[i].length; j++) {
        mp_fData[i][j] = 0;
      }
    }

    //Force the processing of each timestep
    for (i = 0; i <= iNumTimesteps; i++) {

      //Stash the timestep we're on for the sake of AddTreeFloatData
      m_iCurrentTimestep = i;

      //Make the file manager parse this file
      oManager.ReadFile(i);
    }

    //Cause this chart to ignore data from other sources
    m_iCurrentTimestep = -1;

    //Refresh all the existing charts back to the way they were
    oManager.ReadFile(iTimestepToReturnTo);
    oManager.UpdateCharts();

    //Normalize all data to per-hectare values if absolute
    if (m_iChartType == ABSOLUTE_BASAL_AREA ||
        m_iChartType == ABSOLUTE_DENSITY ||
        m_iChartType == ABSOLUTE_VOLUME) {
      for (i = 0; i < mp_fData.length; i++) {
        for (j = 0; j < mp_fData[i].length; j++) {
          fVal = mp_fData[i][j];
          mp_fData[i][j] = fVal / m_fPlotAreaInHectares;
        }
      }
    }

    //Calculate totals
    for (i = 0; i < mp_fTotal.length; i++) {
      float fTotal = 0;
      for (j = 0; j < mp_fData.length; j++) {
        fTotal += mp_fData[j][i];
      }
      mp_fTotal[i] = fTotal;
    }

    //If density, calculate proportions
    if (m_iChartType == RELATIVE_BASAL_AREA ||
        m_iChartType == RELATIVE_DENSITY ||
        m_iChartType == RELATIVE_VOLUME) {
      for (i = 0; i < mp_fTotal.length; i++) {
        if (mp_fTotal[i] > 0) {
          for (j = 0; j < mp_fData.length; j++) {
            mp_fData[j][i] = mp_fData[j][i] / mp_fTotal[i];
          }
        }
      }
    }

    // create the oDataset...
    DefaultXYDataset oDataset = new DefaultXYDataset();
    for (i = 0; i < mp_fData.length; i++) {
      double[][] p_fData = new double[2][mp_fData[i].length];
      for (j = 0; j < mp_fData[i].length; j++) {
        p_fData[0][j] = j;
        p_fData[1][j] = mp_fData[i][j];
      }
      oDataset.addSeries(oLegend.GetSpeciesCodeName(i), p_fData);
    }
    //Last one for total, if applicable
    if ((m_iChartType == ABSOLUTE_BASAL_AREA ||
        m_iChartType == ABSOLUTE_DENSITY ||
        m_iChartType == ABSOLUTE_VOLUME) &&
        m_jUseTotals.isSelected()) {
      double[][] p_fData = new double[2][mp_fTotal.length];
      for (j = 0; j < mp_fTotal.length; j++) {
        p_fData[0][j] = j;
        p_fData[1][j] = mp_fTotal[j];
      }
      oDataset.addSeries("Total", p_fData);
    }

    //Adjust the dataset to make sure only desired species are visible
    return oDataset;
  }

  /**
   * Draws the chart.
   * @param oLegend Legend Legend for this chart.
   * @param sChartTitle String Chart name for the window title.
   * @throws ModelException Passed through from other called methods - this
   * method doesn't throw it.
   * @return JInternalFrame The finished chart window.
   */
  ModelInternalFrame DrawChart(Legend oLegend, String sChartTitle) throws
      javawrapper.ModelException {

    //Get a copy of which species will be displayed
    for (int i = 0; i < oLegend.GetNumberOfSpecies(); i++) {
      mp_bWhichSpeciesShown[i] = oLegend.GetIsSpeciesSelected(i);
    }
    String sXAxisLabel = "Timesteps", sYAxisLabel = "";

    if (m_iChartType == ABSOLUTE_BASAL_AREA) {
      sYAxisLabel = "sq. m/ha";
    }
    else if (m_iChartType == RELATIVE_BASAL_AREA ||
             m_iChartType == RELATIVE_DENSITY ||
             m_iChartType == RELATIVE_VOLUME) {
      sYAxisLabel = "Proportion";
    }
    else if (m_iChartType == ABSOLUTE_DENSITY) {
      sYAxisLabel = "#/ha";
    }
    else if (m_iChartType == ABSOLUTE_VOLUME) {
      sYAxisLabel = "m3/ha";
    }
    else {
      return null;
    }

    m_oDataset = MakeDataset( (DetailedOutputLegend) oLegend);
    DefaultXYDataset oAdjustedDataset = AdjustVisibleSpecies(m_oDataset,
        oLegend);

    m_oChartFrame = DataGrapher.DrawLineChart(oAdjustedDataset, this,
                                              sXAxisLabel,
                                              sYAxisLabel,
                                              sChartTitle,
                                              oLegend,
                                              m_jUseTotals,
                                              oLegend.GetNumberOfSpecies());

    return m_oChartFrame;
  }

  /**
   * Does nothing, because the charts don't operate on the detailed output
   * file update protocol.
   * @throws ModelException Won't throw it.
    */
  void ClearData() throws javawrapper.ModelException {
    ;
  }

  /**
   * This wants diameter values for all trees.
   * @return boolean Set to true.
   */
  public boolean WantAnyTreeFloats() {
    return true;
  }

  /**
   * Accepts a tree float data member code.  We're looking for DBH and diam10.
   * @param iSpecies The species for which this is a data member.
   * @param iType The tree type for which this is a data member.
   * @param sLabel The label of the data member.
   * @param iCode The data member code.
   */
  public void AddTreeFloatDataMemberCode(int iSpecies, int iType, String sLabel,
                                         int iCode) {

    if (iType == m_iTreeType) {
      //03/09/05 LEM - check for the appropriate chart type
      if ((m_iChartType == ABSOLUTE_VOLUME ||
          m_iChartType == RELATIVE_VOLUME) &&
         sLabel.equalsIgnoreCase("volume")) {
       mp_iDataCode[iSpecies] = iCode;
     }
     else if (sLabel.equalsIgnoreCase("dbh")) {
        mp_iDataCode[iSpecies] = iCode;
      }
      else if (sLabel.equalsIgnoreCase("diam10") &&
               javawrapper.TreePopulation.SEEDLING == iType) {
        mp_iDataCode[iSpecies] = iCode;
      }
    }
  }

  /**
   * Accepts a tree float data member value.  If it matches our code, we'll
   * increment either the density count or the basal area total, depending
   * on the chart type.
   * @param iSpecies Species of the tree from which this value came.
   * @param iType Type of the tree from which this value came.
   * @param iCode Data member code of this value.
   * @param fVal Value.
   */
  public void AddTreeFloatData(int iSpecies, int iType, int iCode, float fVal) {

    //If this was triggered by some other event, ignore
    if ( -1 == m_iCurrentTimestep) {
      return;
    }

    //Make sure it matches our type
    if (iType != m_iTreeType) {
      return;
    }

    if (iCode == mp_iDataCode[iSpecies]) {

      float fCurrVal = mp_fData[iSpecies][m_iCurrentTimestep];

      if (m_iChartType == ABSOLUTE_DENSITY || m_iChartType == RELATIVE_DENSITY) {
        //Increment the density count
        fCurrVal = fCurrVal + 1;
      }
      else if (m_iChartType == ABSOLUTE_VOLUME || m_iChartType == RELATIVE_VOLUME) {
        //03/09/05 LEM - increment volume
        fCurrVal += fVal;
      }
      else {

        //Increment the basal area
        fCurrVal += (java.lang.Math.pow(fVal * 0.5, 2) *
                     java.lang.Math.PI * 0.0001);
      }

      //Set the value back into the table
      mp_fData[iSpecies][m_iCurrentTimestep] = fCurrVal;
    }
  }

  /**
   * This will take a dataset and make a copy with only series for species
   * which are supposed to be visible.
   * @param oDataset The dataset.
   * @param oLegend The legend which controls which species are visible.
   * @return The dataset copy.
   * @throws ModelException wrapping another exception.
   */
  protected DefaultXYDataset AdjustVisibleSpecies(DefaultXYDataset
      oDataset, Legend oLegend) throws javawrapper.ModelException {

    DefaultXYDataset oNewDataset = null;
   int iSp, iNumSpecies = oLegend.GetNumberOfSpecies(), i;

    //Make a copy of the new dataset - cloning doesn't work
    try {
      oNewDataset = (DefaultXYDataset) oDataset.clone();
    } catch (java.lang.CloneNotSupportedException oErr) {
      throw( new javawrapper.ModelException(0, "JAVA", "Can't clone dataset."));
    }

    for (iSp = 0; iSp < iNumSpecies; iSp++) {
      if (!oLegend.GetIsSpeciesSelected(iSp)) {
        for (i = 0; i < oNewDataset.getSeriesCount(); i++) {
          if (oNewDataset.getSeriesKey(i).compareTo(oLegend.GetSpeciesCodeName(iSp)) ==
              0) {
            oNewDataset.removeSeries(oNewDataset.getSeriesKey(i));
          }
        }
      }
    }

    return oNewDataset;
  }

  /**
   * Performs actions for the controls in the Histogram window.
   * @param oEvent Event triggered.
   */
  public void actionPerformed(java.awt.event.ActionEvent oEvent) {
    try {
      if (oEvent.getActionCommand().equals("total_checkbox")) {
        if (false == m_jUseTotals.isSelected()) {
          //Take away the total
          for (int i = 0; i < m_oDataset.getSeriesCount(); i++) {
            if (((String)m_oDataset.getSeriesKey(i)).equalsIgnoreCase("total")) {
              m_oDataset.removeSeries(m_oDataset.getSeriesKey(i));
              break;
            }
          }
        } else {
          //Add the total row
          if (m_oDataset != null && m_oDataset.getSeriesCount() > 0) {
            double[][] p_fData = new double[2][mp_fTotal.length];
            for (int j = 0; j < mp_fTotal.length; j++) {
              p_fData[0][j] = j;
              p_fData[1][j] = mp_fTotal[j];
            }
            m_oDataset.addSeries("Total", p_fData);
          }
        }
        DefaultXYDataset oAdjustedDataset = AdjustVisibleSpecies(m_oDataset,
          m_oManager.GetLegend());

        DataGrapher.UpdateLineChart(oAdjustedDataset, m_oChartFrame,
                                m_oManager.GetLegend(), m_jUseTotals,
                                m_oManager.GetLegend().GetNumberOfSpecies());
      }
      super.actionPerformed(oEvent);
    }
    catch (javawrapper.ModelException oErr) {
      javawrapper.ErrorGUI oHandler = new javawrapper.ErrorGUI(m_oChartFrame);
      oHandler.WriteErrorMessage(oErr);
    }
  }
}
