package javawrapper;

/**
* This is the organizer class for all disperse behaviors.
* <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
* <p>Company: Institute of Ecosystem Studies</p>
* @author Lora E. Murphy
* @version 1.0
*
* <br>Edit history:
* <br>------------------
* <br>April 28, 2004: Submitted in beta version (LEM)
* <br>April 24, 2007: Added masting spatial disperse (LEM)
*/

public class DisperseBehaviors
    extends BehaviorTypeBase {

  /**Weibull disperse function*/
  public final static int WEIBULL = 0;
  /**Lognormal disperse function*/
  public final static int LOGNORMAL = 1;
  /**Canopy forest cover status for cells*/
  public final static int CANOPY = 0;
  /**Gap forest cover status for cells*/
  public final static int GAP = 1;

  /**Total number of disperse functions*/
  public final static int NUMBER_OF_DISPERSE_FUNCTIONS = 2;
  /**Total number of forest cover statuses*/
  public final static int NUMBER_OF_FOREST_COVERS = 2;

  /**
   * STR for disperse function.  Array is 3D - first index is which disperse
   * function is used - weibull or lognormal.  The second index is cover -
   * canopy or gap.  The third index is species.
   */
  protected ModelVector[][] mp_fSTR,
      /**
       * Beta for disperse function.  Array is 3D - first index is which disperse
       * function is used - weibull or lognormal.  The second index is cover -
       * canopy or gap.  The third index is species.
       */
      mp_fBeta,
      /**
       * Theta (if weibull) or Xb (if lognormal) for disperse function.
       * Array is 3D - first index is which disperse
       * function is used - weibull or lognormal.  The second index is cover -
       * canopy or gap.  The third index is species.
       */
      mp_fThetaOrXb,
      /**
       * Dispersal (if weibull) or X0 (if lognormal) for disperse function.
       * Array is 3D - first index is which disperse
       * function is used - weibull or lognormal.  The second index is cover -
       * canopy or gap.  The third index is species.
       */
      mp_fDispOrX0;

  protected ModelVector[]
      /**Which disperse function to use under each forest cover - valid values
       * are WEIBULL and LOGNORMAL - this is a vector of ModelEnums*/
      mp_iWhichFunctionUsed;

  protected ModelVector
      /**Slope of lambda for non spatial dispersal for each species*/
      mp_fSlopeOfLambda = new ModelVector(
      "Slope Mean Non-Spatial Seed Rain, seeds/m2/ha of BA/yr",
      "di_nonSpatialSlopeOfLambda",
      "di_nssolVal", 0,
      ModelVector.FLOAT);

  /**Intercept of lambda for non spatial dispersal for each species*/
  protected ModelVector mp_fInterceptOfLambda = new ModelVector(
      "Intercept of Mean Non-Spatial Seed Rain, seeds/m2/yr",
      "di_nonSpatialInterceptOfLambda",
      "di_nsiolVal", 0, ModelVector.FLOAT);

  /**Minimum DBH for reproduction for each species*/
  protected ModelVector mp_fMinDbhForReproduction = new ModelVector(
        "Minimum DBH for Reproduction, in cm", "di_minDbhForReproduction",
        "di_mdfrVal", 0, ModelVector.FLOAT);

  /**STR for stump dispersal for each species*/
  protected ModelVector mp_fStumpSTR = new ModelVector("STR/n for Stumps",
      "di_suckerSTR", "di_ssVal", 0, ModelVector.FLOAT);

  /**Beta for stump dispersal for each species*/
  protected ModelVector mp_fStumpBeta = new ModelVector("Beta for Stumps",
      "di_suckerBeta", "di_sbVal", 0, ModelVector.FLOAT);

  /**Azimuth direction of maximum dispersal distance, in radians*/
  protected ModelVector mp_fDirectionMaxDispersal = new ModelVector(
      "Azimuth Direction of Max Dispersal Distance, in rad",
      "di_directionMaxDispersal", "di_dmdVal", 0, ModelVector.FLOAT);

  /**Amplitude of anisotropic effect*/
  protected ModelVector mp_fAnisotropicAmplitude = new ModelVector(
      "Amplitude of Anisotropic Effect", "di_anisotropicAmplitude", "di_aaVal",
      0, ModelVector.FLOAT);

  /**Standard deviation if seed distribution method is normal or lognormal*/
  protected ModelVector mp_fStandardDeviation = new ModelVector(
      "Seed Dist. Std. Deviation (Normal or Lognormal)",
      "di_standardDeviation", "di_sdVal", 0, ModelVector.FLOAT);

  /**Clumping parameter if seed distribution is negative binomial*/
  protected ModelVector mp_fClumpingParameter = new ModelVector(
      "Seed Dist. Clumping Parameter (Neg. Binomial)",
      "di_clumpingParameter", "di_cpVal", 0, ModelVector.FLOAT);

  /**Masting spatial disperse - "a" for masting CDF*/
  protected ModelVector mp_fMastingA = new ModelVector(
      "Masting Disperse - Masting CDF \"a\"",
      "di_mastCDFA", "di_mcdfaVal", 0, ModelVector.FLOAT);

  /**Masting spatial disperse - "b" for masting CDF*/
  protected ModelVector mp_fMastingB = new ModelVector(
      "Masting Disperse - Masting CDF \"b\"",
      "di_mastCDFB", "di_mcdfbVal", 0, ModelVector.FLOAT);

  /**Masting spatial disperse - Probability distribution for STR draw*/
  protected ModelVector mp_iMastSTRDrawPDF = new ModelVector(
      "Masting Disperse - STR Draw PDF",
      "di_mastSTRPDF", "di_mstrpdfVal", 0, ModelVector.MODEL_ENUM);

  /**Masting spatial disperse - Non-mast STR mean*/
  protected ModelVector mp_fMastNonMastSTRMean = new ModelVector(
	      "Masting Disperse - Non-Masting STR/n Mean",
	      "di_spatialSTR", "di_sstrVal", 0, ModelVector.FLOAT);

  /**Masting spatial disperse - Non-mast STR draw standard deviation, if PDF = normal or lognormal*/
  protected ModelVector mp_fMastNonMastSTRStdDev = new ModelVector(
	      "Masting Disperse - Non-Masting STR/n Standard Deviation",
	      "di_spatialSTRStdDev", "di_sstrsdVal", 0, ModelVector.FLOAT);

  /**Masting spatial disperse - Masting STR mean*/
  protected ModelVector mp_fMastMastSTRMean = new ModelVector(
	      "Masting Disperse - Masting STR/n Mean",
	      "di_mastingSTR", "di_mstrVal", 0, ModelVector.FLOAT);

  /**Masting spatial disperse - Masting STR draw standard deviation, if PDF = normal or lognormal*/
  protected ModelVector mp_fMastMastSTRStdDev = new ModelVector(
	      "Masting Disperse - Masting STR/n Standard Deviation",
	      "di_mastingSTRStdDev", "di_mstrsdVal", 0, ModelVector.FLOAT);

  /**Masting spatial disperse - Non-masting beta*/
  protected ModelVector mp_fMastNonMastBeta = new ModelVector(
	      "Masting Disperse - Non-Masting Beta",
	      "di_spatialBeta", "di_sbVal", 0, ModelVector.FLOAT);

  /**Masting spatial disperse - Masting beta*/
  protected ModelVector mp_fMastMastBeta = new ModelVector(
	      "Masting Disperse - Masting Beta",
	      "di_mastingBeta", "di_mbVal", 0, ModelVector.FLOAT);

  /**Masting spatial disperse - Weibull masting dispersal*/
  protected ModelVector mp_fMastMastWeibDisp = new ModelVector(
	      "Masting Disperse - Masting Weibull Dispersal",
	      "di_weibullMastingDispersal", "di_wmdVal", 0, ModelVector.FLOAT);

  /**Masting spatial disperse - Weibull masting theta*/
  protected ModelVector mp_fMastMastWeibTheta = new ModelVector(
	      "Masting Disperse - Masting Weibull Theta",
	      "di_weibullMastingTheta", "di_wmtVal", 0, ModelVector.FLOAT);

  /**Masting spatial disperse - Lognormal masting X0*/
  protected ModelVector mp_fMastMastLognormalX0 = new ModelVector(
	      "Masting Disperse - Masting Lognormal X0",
	      "di_lognormalMastingX0", "di_lmx0Val", 0, ModelVector.FLOAT);

  /**Masting spatial disperse - Lognormal masting Xb*/
  protected ModelVector mp_fMastMastLognormalXb = new ModelVector(
	      "Masting Disperse - Masting Lognormal Xb",
	      "di_lognormalMastingXb", "di_lmxbVal", 0, ModelVector.FLOAT);

  /**Masting spatial disperse - Group identification for each species*/
  protected ModelVector mp_iMastGroupID = new ModelVector(
	      "Masting Disperse - Masting Group",
	      "di_mastGroup", "di_mgVal", 0, ModelVector.MODEL_ENUM);

  /**Masting spatial disperse - Whether to draw STR once per species (1) or once per tree (0)*/
  protected ModelVector mp_iMastDrawPerSpecies = new ModelVector(
      "Masting Disperse - Stochastic STR Draw Frequency",
      "di_mastDrawPerSpecies", "di_mdpsVal", 0, ModelVector.MODEL_ENUM);

  /**Masting spatial disperse - Proportion trees participating in disperse for mast event*/
  protected ModelVector mp_fMastMastPropParticipating = new ModelVector(
      "Masting Disperse - Mast Proportion Participating (0-1)",
      "di_mastPropParticipating", "di_mppVal", 0, ModelVector.FLOAT);

  /**Masting spatial disperse - Proporton trees participating in disperse for non-mast event*/
  protected ModelVector mp_fMastNonMastPropParticipating = new ModelVector(
      "Masting Disperse - Non-Mast Proportion Participating (0-1)",
      "di_spatialPropParticipating", "di_sppVal", 0, ModelVector.FLOAT);

  /**Seed distribution*/
  protected ModelEnum m_iSeedDistributionMethod =
      new ModelEnum(new int[] {0, 1, 2, 3, 4}
                    ,
                    new String[] {"Deterministic", "Poisson", "Lognormal",
                    "Normal", "Negative binomial"}
                    , "Seed Distribution", "di_seedDistributionMethod");

  protected ModelFloat
      /**Maximum search radius, in meters, for neighbors for isotropic and
       * anisotropic disperse*/
      m_fMaxSearchRadius = new ModelFloat(0,
                                          "Maximum Search Distance for Neighbor Parents, in m",
                                          "di_maxSearchRadius");

  protected ModelInt
      /**Max number of parent trees that can be in a grid cell for it to still be
       * marked as gap */
      m_iMaxGapDensity = new ModelInt(0,
                                      "Maximum Parent Trees Allowed in Gap Cell",
                                      "di_maxGapDensity");

  /**
   * Constructor
   * @param oManager GUIManager object.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public DisperseBehaviors(GUIManager oManager) {
    super(oManager, "Disperse", "disperse");

    m_sHelpID = "data.disperse_behaviors";

    int i, j;

    //Initialize all our arrays and prep our vectors

    mp_fSTR = new ModelVector[NUMBER_OF_DISPERSE_FUNCTIONS][];
    mp_fBeta = new ModelVector[NUMBER_OF_DISPERSE_FUNCTIONS][];
    mp_fThetaOrXb = new ModelVector[NUMBER_OF_DISPERSE_FUNCTIONS][];
    mp_fDispOrX0 = new ModelVector[NUMBER_OF_DISPERSE_FUNCTIONS][];

    for (i = 0; i < NUMBER_OF_DISPERSE_FUNCTIONS; i++) {
      mp_fSTR[i] = new ModelVector[NUMBER_OF_FOREST_COVERS];
      mp_fBeta[i] = new ModelVector[NUMBER_OF_FOREST_COVERS];
      mp_fThetaOrXb[i] = new ModelVector[NUMBER_OF_FOREST_COVERS];
      mp_fDispOrX0[i] = new ModelVector[NUMBER_OF_FOREST_COVERS];

      for (j = 0; j < NUMBER_OF_FOREST_COVERS; j++) {
        if (WEIBULL == i && GAP == j) {
          mp_fSTR[i][j] = new ModelVector("Weibull Gap Annual STR/n",
                                          "di_weibullGapSTR",
                                          "di_wgsVal", 0, ModelVector.FLOAT);
          mp_fBeta[i][j] = new ModelVector("Weibull Gap Beta",
                                           "di_weibullGapBeta", "di_wgbVal", 0,
                                           ModelVector.FLOAT);
          mp_fThetaOrXb[i][j] = new ModelVector("Weibull Gap Theta",
                                                "di_weibullGapTheta",
                                                "di_wgtVal", 0,
                                                ModelVector.FLOAT);
          mp_fDispOrX0[i][j] = new ModelVector("Weibull Gap Dispersal",
                                               "di_weibullGapDispersal",
                                               "di_wgdVal", 0,
                                               ModelVector.FLOAT);
        }
        else if (WEIBULL == i && CANOPY == j) {
          mp_fSTR[i][j] = new ModelVector("Weibull Canopy Annual STR/n",
                                          "di_weibullCanopySTR", "di_wcsVal", 0,
                                          ModelVector.FLOAT);
          mp_fBeta[i][j] = new ModelVector("Weibull Canopy Beta",
                                           "di_weibullCanopyBeta", "di_wcbVal",
                                           0, ModelVector.FLOAT);
          mp_fThetaOrXb[i][j] = new ModelVector("Weibull Canopy Theta",
                                                "di_weibullCanopyTheta",
                                                "di_wctVal", 0,
                                                ModelVector.FLOAT);
          mp_fDispOrX0[i][j] = new ModelVector("Weibull Canopy Dispersal",
                                               "di_weibullCanopyDispersal",
                                               "di_wcdVal", 0,
                                               ModelVector.FLOAT);
        }
        else if (LOGNORMAL == i && GAP == j) {
          mp_fSTR[i][j] = new ModelVector("Lognormal Gap Annual STR/n",
                                          "di_lognormalGapSTR", "di_lgsVal", 0,
                                          ModelVector.FLOAT);
          mp_fBeta[i][j] = new ModelVector("Lognormal Gap Beta",
                                           "di_lognormalGapBeta", "di_lgbVal",
                                           0, ModelVector.FLOAT);
          mp_fThetaOrXb[i][j] = new ModelVector("Lognormal Gap Xb",
                                                "di_lognormalGapXb",
                                                "di_lgxbVal", 0,
                                                ModelVector.FLOAT);
          mp_fDispOrX0[i][j] = new ModelVector("Lognormal Gap X0",
                                               "di_lognormalGapX0",
                                               "di_lgx0Val", 0,
                                               ModelVector.FLOAT);
        }
        else {
          mp_fSTR[i][j] = new ModelVector("Lognormal Canopy Annual STR/n",
                                          "di_lognormalCanopySTR", "di_lcsVal",
                                          0, ModelVector.FLOAT);
          mp_fBeta[i][j] = new ModelVector("Lognormal Canopy Beta",
                                           "di_lognormalCanopyBeta",
                                           "di_lcbVal", 0, ModelVector.FLOAT);
          mp_fThetaOrXb[i][j] = new ModelVector("Lognormal Canopy Xb",
                                                "di_lognormalCanopyXb",
                                                "di_lcxbVal", 0,
                                                ModelVector.FLOAT);
          mp_fDispOrX0[i][j] = new ModelVector("Lognormal Canopy X0",
                                               "di_lognormalCanopyX0",
                                               "di_lcx0Val", 0,
                                               ModelVector.FLOAT);
        }
      }
    }

    mp_iWhichFunctionUsed = new ModelVector[NUMBER_OF_FOREST_COVERS];

    for (i = 0; i < NUMBER_OF_FOREST_COVERS; i++) {
      if (CANOPY == i) {
        mp_iWhichFunctionUsed[i] = new ModelVector("Canopy Function Used",
            "di_canopyFunction", "di_cfVal", 0, ModelVector.MODEL_ENUM);
      }
      else {
        mp_iWhichFunctionUsed[i] = new ModelVector("Gap Function Used",
            "di_gapFunction", "di_gfVal", 0, ModelVector.MODEL_ENUM);
      }
    }

    //Set up behavior list
    mp_oChildBehaviors = new Behavior[5];
    int iIndex = 0;

    //Add a behavior for non spatial dispersal
    mp_oChildBehaviors[iIndex] = new Behavior("non spatial disperse",
                                         "Non-spatial disperse",
                                         "non spatial disperse", (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMinDbhForReproduction);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSlopeOfLambda);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fInterceptOfLambda);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_iSeedDistributionMethod);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStandardDeviation);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fClumpingParameter);
    //Disallow for seedlings
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
    iIndex++;

    //Add a behavior for non-gap disperse
    mp_oChildBehaviors[iIndex] = new Behavior("non-gap disperse",
                                         "Non-gap spatial disperse",
                                         "non-gap disperse",
                                         (float) 1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMinDbhForReproduction);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_iWhichFunctionUsed[CANOPY]);
    for (i = 0; i < NUMBER_OF_DISPERSE_FUNCTIONS; i++) {
      mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSTR[i][CANOPY]);
      mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBeta[i][CANOPY]);
      mp_oChildBehaviors[iIndex].AddRequiredData(mp_fThetaOrXb[i][CANOPY]);
      mp_oChildBehaviors[iIndex].AddRequiredData(mp_fDispOrX0[i][CANOPY]);
    }
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStumpSTR);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStumpBeta);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_iSeedDistributionMethod);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStandardDeviation);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fClumpingParameter);
    //Disallow for seedlings
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
    //Allow for stumps
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.STUMP, true);
    iIndex++;

    //Add a behavior for gap disperse
    mp_oChildBehaviors[iIndex] = new Behavior("gap disperse", "Gap spatial disperse",
                                         "gap disperse", (float) 1);
    //Disallow for seedlings
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
    //Allow for stumps
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.STUMP, true);

    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMinDbhForReproduction);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_iMaxGapDensity);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_iSeedDistributionMethod);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStumpSTR);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStumpBeta);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStandardDeviation);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fClumpingParameter);
    for (i = 0; i < NUMBER_OF_FOREST_COVERS; i++) {
      mp_oChildBehaviors[iIndex].AddRequiredData(mp_iWhichFunctionUsed[i]);
    }
    for (i = 0; i < NUMBER_OF_DISPERSE_FUNCTIONS; i++) {
      for (j = 0; j < NUMBER_OF_FOREST_COVERS; j++) {
        mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSTR[i][j]);
        mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBeta[i][j]);
        mp_oChildBehaviors[iIndex].AddRequiredData(mp_fThetaOrXb[i][j]);
        mp_oChildBehaviors[iIndex].AddRequiredData(mp_fDispOrX0[i][j]);
      }
    }
    iIndex++;
    
    //Add a behavior for masting spatial disperse
    mp_oChildBehaviors[iIndex] = new Behavior("masting spatial disperse",
                                         "Masting spatial disperse",
                                         "masting spatial disperse", (float) 1);

    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMastingA);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMastingB);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMinDbhForReproduction);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_iMastSTRDrawPDF);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_iWhichFunctionUsed[CANOPY]);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMastNonMastSTRMean);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMastNonMastSTRStdDev);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_iMastDrawPerSpecies);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMastNonMastBeta);
    for (i = 0; i < NUMBER_OF_DISPERSE_FUNCTIONS; i++) {
      mp_oChildBehaviors[iIndex].AddRequiredData(mp_fThetaOrXb[i][CANOPY]);
      mp_oChildBehaviors[iIndex].AddRequiredData(mp_fDispOrX0[i][CANOPY]);
    }
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMastNonMastPropParticipating);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMastMastSTRMean);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMastMastSTRStdDev);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMastMastBeta);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMastMastWeibTheta);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMastMastWeibDisp);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMastMastLognormalXb);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMastMastLognormalX0);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMastMastPropParticipating);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_iMastGroupID);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_iSeedDistributionMethod);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStandardDeviation);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fClumpingParameter);
    //Disallow for seedlings
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
    iIndex++;

    //Add a behavior for anisotropic disperse
    mp_oChildBehaviors[iIndex] = new Behavior("anisotropic disperse",
                                         "Anisotropic spatial disperse",
                                         "anisotropic disperse", (float) 1);
    //Disallow for seedlings
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fMaxSearchRadius);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMinDbhForReproduction);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_iWhichFunctionUsed[CANOPY]);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_iSeedDistributionMethod);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStandardDeviation);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fClumpingParameter);
    for (i = 0; i < NUMBER_OF_DISPERSE_FUNCTIONS; i++) {
      mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSTR[i][CANOPY]);
      mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBeta[i][CANOPY]);
      mp_oChildBehaviors[iIndex].AddRequiredData(mp_fThetaOrXb[i][CANOPY]);
      mp_oChildBehaviors[iIndex].AddRequiredData(mp_fDispOrX0[i][CANOPY]);
    }
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fDirectionMaxDispersal);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAnisotropicAmplitude);
    iIndex++;

    //All data - in the order it will be written in XML
    mp_oAllData.add(mp_fMinDbhForReproduction);
    //Weibull canopy parameters
    mp_oAllData.add(mp_fSTR[WEIBULL][CANOPY]);
    mp_oAllData.add(mp_fBeta[WEIBULL][CANOPY]);
    mp_oAllData.add(mp_fDispOrX0[WEIBULL][CANOPY]);
    mp_oAllData.add(mp_fThetaOrXb[WEIBULL][CANOPY]);
    //Weibull gap parameters
    mp_oAllData.add(mp_fSTR[WEIBULL][GAP]);
    mp_oAllData.add(mp_fBeta[WEIBULL][GAP]);
    mp_oAllData.add(mp_fDispOrX0[WEIBULL][GAP]);
    mp_oAllData.add(mp_fThetaOrXb[WEIBULL][GAP]);
    //Lognormal canopy parameters
    mp_oAllData.add(mp_fSTR[LOGNORMAL][CANOPY]);
    mp_oAllData.add(mp_fBeta[LOGNORMAL][CANOPY]);
    mp_oAllData.add(mp_fDispOrX0[LOGNORMAL][CANOPY]);
    mp_oAllData.add(mp_fThetaOrXb[LOGNORMAL][CANOPY]);
    //Lognormal gap parameters
    mp_oAllData.add(mp_fSTR[LOGNORMAL][GAP]);
    mp_oAllData.add(mp_fBeta[LOGNORMAL][GAP]);
    mp_oAllData.add(mp_fDispOrX0[LOGNORMAL][GAP]);
    mp_oAllData.add(mp_fThetaOrXb[LOGNORMAL][GAP]);
    //Stump parameters
    mp_oAllData.add(mp_fStumpSTR);
    mp_oAllData.add(mp_fStumpBeta);

    mp_oAllData.add(m_iMaxGapDensity);

    mp_oAllData.add(mp_iWhichFunctionUsed[GAP]);
    mp_oAllData.add(mp_iWhichFunctionUsed[CANOPY]);
    //Bath recruitment parameters
    mp_oAllData.add(mp_fSlopeOfLambda);
    mp_oAllData.add(mp_fInterceptOfLambda);
    //Anisotropic disperse parameters
    mp_oAllData.add(m_fMaxSearchRadius);
    mp_oAllData.add(mp_fDirectionMaxDispersal);
    mp_oAllData.add(mp_fAnisotropicAmplitude);
    //Masting spatial disperse parameters
    mp_oAllData.add(mp_fMastingA);
    mp_oAllData.add(mp_fMastingB);
    mp_oAllData.add(mp_iMastSTRDrawPDF);
    mp_oAllData.add(mp_fMastNonMastSTRMean);
    mp_oAllData.add(mp_fMastNonMastSTRStdDev);
    mp_oAllData.add(mp_iMastDrawPerSpecies);
    mp_oAllData.add(mp_fMastNonMastBeta);
    mp_oAllData.add(mp_fMastNonMastPropParticipating);
    mp_oAllData.add(mp_fMastMastSTRMean);
    mp_oAllData.add(mp_fMastMastSTRStdDev);
    mp_oAllData.add(mp_fMastMastBeta);
    mp_oAllData.add(mp_fMastMastWeibTheta);
    mp_oAllData.add(mp_fMastMastWeibDisp);
    mp_oAllData.add(mp_fMastMastLognormalXb);
    mp_oAllData.add(mp_fMastMastLognormalX0);
    mp_oAllData.add(mp_fMastMastPropParticipating);
    mp_oAllData.add(mp_iMastGroupID);
    //Seed distribution parameters
    mp_oAllData.add(m_iSeedDistributionMethod);
    mp_oAllData.add(mp_fStandardDeviation);
    mp_oAllData.add(mp_fClumpingParameter);
  }

  /**
   * Does setup.  Sets up the substrate favorability grid.
   * @param oPop TreePopulation object.
   * @throws ModelException if there's a problem setting behavior use data.
   * @todo Something for stump removal?
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void DoSetup(TreePopulation oPop) throws ModelException {
    int iNumSpecies = oPop.GetNumberOfSpecies(), i, j, k;

    //Set up the "which function" vector
    for (i = 0; i < NUMBER_OF_FOREST_COVERS; i++) {
      if (mp_iWhichFunctionUsed[i].GetValue().size() == 0) {
        for (j = 0; j < iNumSpecies; j++) {
          mp_iWhichFunctionUsed[i].GetValue().add(new ModelEnum(new int[] {
              WEIBULL, LOGNORMAL}, new String[] {"Weibull", "Lognormal"}
              , "Function used", ""));
        }
      } else {
        //There were already species - verify that any nulls get their enum
        //added
        for (j = 0; j < iNumSpecies; j++) {
          if (null == mp_iWhichFunctionUsed[i].GetValue().elementAt(j)) {
            mp_iWhichFunctionUsed[i].GetValue().remove(j);
            mp_iWhichFunctionUsed[i].GetValue().add(j, new ModelEnum(new int[] {
                WEIBULL, LOGNORMAL}, new String[] {"Weibull", "Lognormal"},
                "Function used", ""));
          }
        }
      }
    }

    //Set up the "which masting STR draw" vector
    if (mp_iMastSTRDrawPDF.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_iMastSTRDrawPDF.GetValue().add(new ModelEnum(new int[] {
              0, 3, 2}, new String[] {"Deterministic", "Normal", "Lognormal"}
              , "Function used", ""));
      }
    } else {
      //There were already species - verify that any nulls get their enum
      //added
      for (i = 0; i < iNumSpecies; i++) {
        if (null == mp_iMastSTRDrawPDF.GetValue().elementAt(i)) {
          mp_iMastSTRDrawPDF.GetValue().remove(i);
          mp_iMastSTRDrawPDF.GetValue().add(i, new ModelEnum(new int[] {
            0, 3, 2}, new String[] {"Deterministic", "Normal", "Lognormal"}
            , "Function used", ""));
        }
      }
    }
    
    //Set up the masting group ID vector
    if (mp_iMastGroupID.GetValue().size() == 0) {
      //Create an enum with one group for each species
      String[] p_sGroupNames = new String[iNumSpecies];
      int[] p_iGroupNums = new int[iNumSpecies];
      j = iNumSpecies;
      for (i = 0; i < iNumSpecies; i++) {
    	  p_sGroupNames[i] = String.valueOf(j);
        p_iGroupNums[i] = j;
        j--;
      }
      ModelEnum oEnum;
      for (i = 0; i < iNumSpecies; i++) {
        oEnum = new ModelEnum(p_iGroupNums, p_sGroupNames, "Masting group", "");
        oEnum.SetValue(i+1);
        mp_iMastGroupID.GetValue().add(oEnum);
      }
    } else {
      //There were already species - replace the enums so that there are
      //still the same number of groups as species
      //Create an enum with one group for each species
      String[] p_sGroupNames = new String[iNumSpecies];
      int[] p_iGroupNums = new int[iNumSpecies];
      j = iNumSpecies;
      for (i = 0; i < iNumSpecies; i++) {
        p_sGroupNames[i] = String.valueOf(j);
        p_iGroupNums[i] = j;
        j--;
      }
      ModelEnum oEnum;
      int iGroup;
      for (i = 0; i < iNumSpecies; i++) {
        if (null == mp_iMastGroupID.GetValue().elementAt(i)) {
          iGroup = i + 1;
        } else {
          oEnum = (ModelEnum) mp_iMastGroupID.GetValue().elementAt(i);
          iGroup = oEnum.GetValue();
          if (iGroup > iNumSpecies) iGroup = i+1;
        }
        oEnum = new ModelEnum(p_iGroupNums, p_sGroupNames, "Masting group", "");
        oEnum.SetValue(iGroup);
        mp_iMastGroupID.GetValue().remove(i);
        mp_iMastGroupID.GetValue().add(i, oEnum);
      }
    }
    
    //Set up the draw frequency vector
    if (mp_iMastDrawPerSpecies.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_iMastDrawPerSpecies.GetValue().add(new ModelEnum(new int[] {
              1, 0}, new String[] {"Per Species", "Per Tree"}
              , "Draw frequency", ""));
      }
    } else {
      //There were already species - verify that any nulls get their enum
      //added
      for (i = 0; i < iNumSpecies; i++) {
        if (null == mp_iMastDrawPerSpecies.GetValue().elementAt(i)) {
          mp_iMastDrawPerSpecies.GetValue().remove(i);
          mp_iMastDrawPerSpecies.GetValue().add(i, new ModelEnum(new int[] {
              1, 0}, new String[] {"Per Species", "Per Tree"}
          , "Draw frequency", ""));
        }
      }
    }
    
    //Set up the proportion participating vectors to default to values of 1
    if (mp_fMastMastPropParticipating.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fMastMastPropParticipating.GetValue().add(new Float(1.0));
      }
    }
    if (mp_fMastNonMastPropParticipating.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fMastNonMastPropParticipating.GetValue().add(new Float(1.0));
      }
    }

    //Set up the seeds grid
    DataMember[] p_oDataMembers = new DataMember[iNumSpecies + 2];
    //mp_oGrids = new Grid[1];
    Grid oNewGrid;

    String sGridName = "Dispersed Seeds";

    //The accessible data members are a seed count for each species and
    //gap status
    for (i = 0; i < iNumSpecies; i++) {

      p_oDataMembers[i] = new DataMember("Number of seeds for " +
                                         oPop.GetSpeciesNameFromCode(i).
                                         replace('_', ' '),
                                         "seeds_" + i,
                                         DataMember.FLOAT);
    }

    p_oDataMembers[iNumSpecies] = new DataMember("Gap status", "Is Gap",
                                                 DataMember.BOOLEAN);

    p_oDataMembers[iNumSpecies +
        1] = new DataMember("Adult tree count", "count",
                            DataMember.INTEGER);

    //Create our grid
    oNewGrid = new Grid(sGridName, p_oDataMembers, null, 8, 8);
    oNewGrid = m_oManager.AddGrid(oNewGrid);

    //Assign the grid to all behaviors
    for (i = 0; i < mp_oChildBehaviors.length; i++) {
      mp_oChildBehaviors[i].AddGrid(oNewGrid);
    }

    //Initialize our arrays with zeroes if they haven't been set
    for (i = 0; i < NUMBER_OF_DISPERSE_FUNCTIONS; i++) {
      for (j = 0; j < NUMBER_OF_FOREST_COVERS; j++) {
        if (mp_fSTR[i][j].GetValue().size() == 0) {
          for (k = 0; k < iNumSpecies; k++) {
            mp_fSTR[i][j].GetValue().add(new Float(0));
          }
        }
        if (mp_fBeta[i][j].GetValue().size() == 0) {
          for (k = 0; k < iNumSpecies; k++) {
            mp_fBeta[i][j].GetValue().add(new Float(0));
          }
        }
        if (mp_fThetaOrXb[i][j].GetValue().size() == 0) {
          for (k = 0; k < iNumSpecies; k++) {
            mp_fThetaOrXb[i][j].GetValue().add(new Float(0));
          }
        }
        if (mp_fDispOrX0[i][j].GetValue().size() == 0) {
          for (k = 0; k < iNumSpecies; k++) {
            mp_fDispOrX0[i][j].GetValue().add(new Float(0));
          }
        }
      }
    }
    if (mp_fStumpSTR.GetValue().size() == 0) {
      for (k = 0; k < iNumSpecies; k++) {
        mp_fStumpSTR.GetValue().add(new Float(0));
      }
    }
    if (mp_fStumpBeta.GetValue().size() == 0) {
      for (k = 0; k < iNumSpecies; k++) {
        mp_fStumpBeta.GetValue().add(new Float(0));
      }
    }

  }

  /**
   * Validates the data in preparation for parameter file writing or some such.
   * @param oPop TreePopulation object.
   * @throws ModelException if:
   * <ul>
   * <li>Any value in mp_fMinDbhForReproduction is 0</li>
   * <li>There aren't values for all species for any disperse function used</li>
   * <li>Max search radius for anisotropic disperse is not a positive number</li>
   * and less than half the shortest plot distance</li>
   * <li>The direction of max dispersal is not between 0 and 2PI</li>
   * <li>Spatial disperse is used and a value for weibull theta (masting or 
   * non-masting) is greater than or equal to 50</li>
   * <li>Spatial disperse is used and a value for beta (masting or non-masting)
   * is greater than 25</li>
   * <li>Fraction participating, either masting or non-masting, is not a 
   * proportion</li>
   * <li>All values for X0 and Xb are not 0</li>
   * </ul>
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void ValidateData(TreePopulation oPop) throws ModelException {
    Behavior oNoGap = GetBehaviorByKey("non-gap disperse"),
        oGap = GetBehaviorByKey("gap disperse"),
        oMast = GetBehaviorByKey("masting spatial disperse");
    int iNumSpecies = oPop.GetNumberOfSpecies(),
        i, j, k;

    //If there are no behaviors enabled, exit
    if (!AnyBehaviorsEnabled()) {
      return;
    }

    boolean[] p_bApplies = null;
    for (i = 0; i < mp_oChildBehaviors.length; i++) {
      if (mp_oChildBehaviors[i].m_bIsEnabled) {
        if (p_bApplies == null) {
          p_bApplies = mp_oChildBehaviors[i].GetWhichSpeciesUsed(oPop);
        }
        else {
          p_bApplies = MergeBooleans(p_bApplies,
                                     mp_oChildBehaviors[i].
                                     GetWhichSpeciesUsed(oPop));
        }
      }
    }
    MakeSureAllPositive(mp_fMinDbhForReproduction, p_bApplies);

    if (oGap.m_bIsEnabled) {
      for (i = 0; i < NUMBER_OF_DISPERSE_FUNCTIONS; i++) {
        for (j = 0; j < NUMBER_OF_FOREST_COVERS; j++) {
          MakeSureRightSize(mp_fSTR[i][j], iNumSpecies);
          MakeSureRightSize(mp_fBeta[i][j], iNumSpecies);
          MakeSureRightSize(mp_fThetaOrXb[i][j], iNumSpecies);
          MakeSureRightSize(mp_fDispOrX0[i][j], iNumSpecies);
        }
      }
    }
    else if (oNoGap.m_bIsEnabled) {
      for (i = 0; i < NUMBER_OF_DISPERSE_FUNCTIONS; i++) {
        MakeSureRightSize(mp_fSTR[i][CANOPY], iNumSpecies);
        MakeSureRightSize(mp_fBeta[i][CANOPY], iNumSpecies);
        MakeSureRightSize(mp_fThetaOrXb[i][CANOPY], iNumSpecies);
        MakeSureRightSize(mp_fDispOrX0[i][CANOPY], iNumSpecies);
      }
    }

    //Make sure all values for weibull theta are less than 50
    if (oGap.m_bIsEnabled) {
      p_bApplies = oGap.GetWhichSpeciesUsed(oPop);
      for (i = 0; i < mp_fThetaOrXb[WEIBULL][GAP].GetValue().size(); i++) {
        ModelEnum oEnum = (ModelEnum) mp_iWhichFunctionUsed[GAP].GetValue().elementAt(i);
        if (oEnum.GetValue() == WEIBULL && p_bApplies[i]) {
          float fNumber = ( (Float) mp_fThetaOrXb[WEIBULL][GAP].GetValue().
                                 elementAt(i)).floatValue();
          if (fNumber >= 50) {
            throw (new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                                    "The values in " +
                                    mp_fThetaOrXb[WEIBULL][GAP].GetDescriptor() +
                                   " must be less than 50 to avoid math errors."));
          }
        }  
      }
    }
    if (oGap.m_bIsEnabled || oNoGap.m_bIsEnabled || oMast.m_bIsEnabled) {
      p_bApplies = MergeBooleans(oNoGap.GetWhichSpeciesUsed(oPop),
                                 oMast.GetWhichSpeciesUsed(oPop));
      p_bApplies = MergeBooleans(p_bApplies, oGap.GetWhichSpeciesUsed(oPop));
      for (i = 0; i < mp_fThetaOrXb[WEIBULL][CANOPY].GetValue().size(); i++) {
        ModelEnum oEnum = (ModelEnum) mp_iWhichFunctionUsed[GAP].GetValue().elementAt(i);
        if (oEnum.GetValue() == WEIBULL && p_bApplies[i]) {
          float fNumber = ( (Float) mp_fThetaOrXb[WEIBULL][CANOPY].GetValue().
                                 elementAt(i)).floatValue();
          if (fNumber >= 50) {
            throw (new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                                    "The values in " +
                                    mp_fThetaOrXb[WEIBULL][CANOPY].GetDescriptor() +
                                   " must be less than 50 to avoid math errors."));
          }
        }
      }
    }
    
    //Make sure all values for lognormal Xb and X0 don't equal 0
    if (oGap.m_bIsEnabled) {
      p_bApplies = oGap.GetWhichSpeciesUsed(oPop);
      for (i = 0; i < mp_fThetaOrXb[LOGNORMAL][GAP].GetValue().size(); i++) {
        ModelEnum oEnum = (ModelEnum) mp_iWhichFunctionUsed[GAP].GetValue().elementAt(i);
        if (oEnum.GetValue() != LOGNORMAL) p_bApplies[i] = false;
      }  
      MakeSureAllNonZero(mp_fThetaOrXb[LOGNORMAL][GAP], p_bApplies); 
      MakeSureAllNonZero(mp_fDispOrX0[LOGNORMAL][GAP], p_bApplies);
    }
    if (oGap.m_bIsEnabled || oNoGap.m_bIsEnabled || oMast.m_bIsEnabled) {
      p_bApplies = MergeBooleans(oNoGap.GetWhichSpeciesUsed(oPop),
                                 oMast.GetWhichSpeciesUsed(oPop));
      p_bApplies = MergeBooleans(p_bApplies, oGap.GetWhichSpeciesUsed(oPop));
      for (i = 0; i < mp_fThetaOrXb[LOGNORMAL][CANOPY].GetValue().size(); i++) {
        ModelEnum oEnum = (ModelEnum) mp_iWhichFunctionUsed[CANOPY].GetValue().elementAt(i);
        if (oEnum.GetValue() != LOGNORMAL) p_bApplies[i] = false;
      }
      MakeSureAllNonZero(mp_fThetaOrXb[LOGNORMAL][CANOPY], p_bApplies);
      MakeSureAllNonZero(mp_fDispOrX0[LOGNORMAL][CANOPY], p_bApplies);
    }

    //Make sure all values for beta are less than 25
    if (oGap.m_bIsEnabled || oNoGap.m_bIsEnabled || oMast.m_bIsEnabled) {
      p_bApplies = MergeBooleans(oGap.GetWhichSpeciesUsed(oPop), 
                                 oNoGap.GetWhichSpeciesUsed(oPop));
      p_bApplies = MergeBooleans(p_bApplies, 
                                 oMast.GetWhichSpeciesUsed(oPop));
      for (i = 0; i < NUMBER_OF_DISPERSE_FUNCTIONS; i++) {
        for (k = 0; k < mp_fBeta[i][CANOPY].GetValue().size(); k++) {
          if (p_bApplies[k]) {
            float fNumber = ( (Float) mp_fBeta[i][CANOPY].GetValue().
                             elementAt(k)).floatValue();
            if (fNumber > 25) {
              throw (new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                                        "The values in " +
                                        mp_fBeta[i][CANOPY].
                                        GetDescriptor() +
                                        " must be less than 25 to avoid math errors."));
            }
          }
        }
      }
    }
    if (oGap.m_bIsEnabled) {
      p_bApplies = oGap.GetWhichSpeciesUsed(oPop);
      for (i = 0; i < NUMBER_OF_DISPERSE_FUNCTIONS; i++) {
        for (k = 0; k < mp_fBeta[i][GAP].GetValue().size(); k++) {
          if (p_bApplies[k]) {
            float fNumber = ( (Float) mp_fBeta[i][GAP].GetValue().
                             elementAt(k)).floatValue();
            if (fNumber > 25) {
              throw (new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                                        "The values in " +
                                        mp_fBeta[i][GAP].
                                        GetDescriptor() +
                                        " must be less than 25 to avoid math errors."));
            }
          }
        }
      }
    }

    if ( //GetBehaviorByKey("isotropic disperse").m_bIsEnabled ||
        GetBehaviorByKey("anisotropic disperse").m_bIsEnabled) {
    	MakeSureGreaterThan(m_fMaxSearchRadius, 0);
      //Find out what half the shortest plot length is
      Plot oPlot = m_oManager.GetPlot();
      float fMax = java.lang.Math.min(oPlot.GetPlotXLength(),
                                      oPlot.GetPlotYLength()) / 2;
      if (m_fMaxSearchRadius.GetValue() > fMax) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", m_sName + " - " +
                                 m_fMaxSearchRadius.GetDescriptor() +
                  " must not be greater than half the shortest plot length."));
      }
    }

    if (GetBehaviorByKey("anisotropic disperse").m_bIsEnabled) {
      for (i = 0; i < mp_fDirectionMaxDispersal.GetValue().size(); i++) {
        if (mp_fDirectionMaxDispersal.GetValue().elementAt(i) != null) {
          Float fAngle = (Float) mp_fDirectionMaxDispersal.GetValue().elementAt(
              i);
          if (fAngle.floatValue() < 0 ||
              fAngle.floatValue() >= (2 * java.lang.Math.PI)) {
            throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", m_sName + " - "
                                     + mp_fDirectionMaxDispersal.GetDescriptor()
                                     + " values must be between 0 and 2PI."));
          }
        }
      }
    }
    
    //Masting spatial disperse specific validation
    if (oMast.m_bIsEnabled) {
      p_bApplies = oMast.GetWhichSpeciesUsed(oPop);
      
      //Make sure masting thetas are not greater than 50
      for (i = 0; i < mp_fMastMastWeibTheta.GetValue().size(); i++) {
        if (p_bApplies[i]) {
          float fNumber = ( (Float) mp_fMastMastWeibTheta.GetValue().
                                 elementAt(i)).floatValue();
          if (fNumber >= 50) {
            throw (new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                              "The values in " +
                               mp_fMastMastWeibTheta.GetDescriptor() +
                               " must be less than 50 to avoid math errors."));
          }
        }
      }
      
      //Make sure masting betas are not greater than 25
      for (i = 0; i < mp_fMastMastBeta.GetValue().size(); i++) {
        if (p_bApplies[i]) {
          float fNumber = ( (Float) mp_fMastMastBeta.GetValue().
                                 elementAt(i)).floatValue();
          if (fNumber >= 25) {
            throw (new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                              "The values in " +
                              mp_fMastMastBeta.GetDescriptor() +
                              " must be less than 25 to avoid math errors."));
          }
        }
      }
      
      //Make sure all lognormal X0 and Xb != 0
      for (i = 0; i < mp_fMastMastLognormalXb.GetValue().size(); i++) {
        ModelEnum oEnum = (ModelEnum) mp_iWhichFunctionUsed[CANOPY].GetValue().elementAt(i);
        if (oEnum.GetValue() != LOGNORMAL) p_bApplies[i] = false;
      } 
      MakeSureAllNonZero(mp_fMastMastLognormalX0, p_bApplies);
      MakeSureAllNonZero(mp_fMastMastLognormalXb, p_bApplies);
      
      //Make sure fraction participating are proportions
      p_bApplies = oMast.GetWhichSpeciesUsed(oPop);
      MakeSureAllAreProportions(mp_fMastMastPropParticipating, p_bApplies);
      MakeSureAllAreProportions(mp_fMastNonMastPropParticipating, p_bApplies);
    }
  }
}
