package javawrapper;

import java.io.*;
import java.util.Vector;
import org.xml.sax.Attributes;

/**
 * This class manages data for disturbance behaviors. Planting is not included.
 * Copyright: Copyright (c) Charles D. Canham 2003
 * Company: Institute of Ecosystem Studies
 * 
 * @author Lora E. Murphy
 * @version 1.0
 * 
 * Edit history: <br>
 * ------------------ <br>
 * August 15, 2007: Cleared edit history up to 6.08 (LEM) <br>
 * August 15, 2007: Added the competition harvest behavior (LEM) <br>
 * December 11, 2007: Added storm scheduling (LEM)<br>
 * January 17, 2008: Added the storm direct killer (LEM)<br>
 */

public class DisturbanceBehaviors extends BehaviorTypeBase {

	/** Partial cut. Matches the old code's enum value - that's important!. */
	public static final int PARTIAL_CUT = 1;

	/** Gap cut. Matches the old code's enum value - that's important!. */
	public static final int GAP_CUT = 2;

	/** Clear cut. Matches the old code's enum value - that's important!. */
	public static final int CLEAR_CUT = 3;

	/**
	 * Percentage basal area. Matches the old code's enum value - that's
	 * important!.
	 */
	public static final int PERCENTAGE_BASAL_AREA = 2;

	/**
	 * Absolute basal area. Matches the old code's enum value - that's important!.
	 */
	public static final int ABSOLUTE_BASAL_AREA = 3;

	/**
	 * Percentage density. Matches the old code's enum value - that's important!.
	 */
	public static final int PERCENTAGE_DENSITY = 0;

	/**
	 * Absolute density. Matches the old code's enum value - that's important!.
	 */
	public static final int ABSOLUTE_DENSITY = 1;

	/** Our list of harvest cuts, held as HarvestData objects */
	protected Vector mp_oHarvestCuts = new Vector(0);

	/** List of episodic mortality events, held as HarvestData objects (ignoring
	 * the cut type)*/
	protected Vector mp_oMortEpisodes = new Vector(0);
	
	/** List of scheduled storm events (objects of class ScheduledStormInfo)*/
	protected Vector mp_oScheduledStorms = new Vector(0);

	/** Minimum DBH to which storm damage applies for each species */
	protected ModelVector mp_fMinStormDamageDBH = new ModelVector(
			"Minimum DBH for Storm Damage, in cm", "st_minDBH", "st_mdVal", 0,
			ModelVector.FLOAT);

	/** Storm damage intercept for medium damage (a) for each species */
	protected ModelVector mp_fStmDmgInterceptMed = new ModelVector(
			"Storm Damage Intercept (a) for Medium Damage", "st_stmDmgInterceptMed",
			"st_sdimVal", 0, ModelVector.FLOAT);

	/** Storm damage intercept for complete damage (a) for each species */
	protected ModelVector mp_fStmDmgInterceptFull = new ModelVector(
			"Storm Damage Intercept (a) for Heavy Damage", "st_stmDmgInterceptFull",
			"st_sdifVal", 0, ModelVector.FLOAT);

	/** Storm intensity coefficient (b) for each species */
	protected ModelVector mp_fStmIntensityCoeff = new ModelVector(
			"Storm Intensity Coefficient (b)", "st_stmIntensityCoeff", "st_sicVal",
			0, ModelVector.FLOAT);

	/** Storm DBH coefficient (d) for each species */
	protected ModelVector mp_fStmDBHCoeff = new ModelVector(
			"Storm DBH Coefficient (d)", "st_stmDBHCoeff", "st_sdcVal", 0,
			ModelVector.FLOAT);

	/** Storm - Medium damage probability of survival "a" - species-specific */
	protected ModelVector mp_fStmMedDmgSurvProbA = new ModelVector(
			"Storm Medium Damage Survival Prob Intercept (a)",
			"st_stmMedDmgSurvProbA", "st_smdspaVal", 0, ModelVector.FLOAT);

	/** Storm - Medium damage probability of survival "b" - species-specific */
	protected ModelVector mp_fStmMedDmgSurvProbB = new ModelVector(
			"Storm Medium Damage Survival Prob DBH Coeff. (b)",
			"st_stmMedDmgSurvProbB", "st_smdspbVal", 0, ModelVector.FLOAT);

	/** Storm - Heavy damage probability of survival "a" - species-specific */
	protected ModelVector mp_fStmFullDmgSurvProbA = new ModelVector(
			"Storm Heavy Damage Survival Prob Intercept (a)",
			"st_stmFullDmgSurvProbA", "st_sfdspaVal", 0, ModelVector.FLOAT);

	/** Storm - Heavy damage probability of survival "b" - species-specific */
	protected ModelVector mp_fStmFullDmgSurvProbB = new ModelVector(
			"Storm Heavy Damage Survival Prob DBH Coeff. (b)",
			"st_stmFullDmgSurvProbB", "st_sfdspbVal", 0, ModelVector.FLOAT);

	/** Windstorm - Minimum DBH for windstorm mortality - species-specific */
	protected ModelVector mp_fWindstormMinDBH = new ModelVector(
			"Windstorm - Minimum DBH for Windstorm Mortality", "ws_minDBH",
			"ws_mdVal", 0, ModelVector.FLOAT);

	/** Windstorm - Mortality probability intercept "a" - species-specific */
	protected ModelVector mp_fWindstormMortInterceptA = new ModelVector(
			"Windstorm - Mortality Intercept (a)", "ws_stmInterceptA", "ws_siaVal",
			0, ModelVector.FLOAT);

	/**
	 * Windstorm - Mortality probability storm intensity coefficient "c" -
	 * species-specific
	 */
	protected ModelVector mp_fWindstormStmIntensCoeffC = new ModelVector(
			"Windstorm - Storm Intensity Coefficient (c)", "ws_stmIntensCoeffC",
			"ws_sicVal", 0, ModelVector.FLOAT);

	/** Windstorm - DBH exponent "b" - species-specific */
	protected ModelVector mp_fWindstormDBHExpB = new ModelVector(
			"Windstorm - DBH Exponent (b)", "ws_stmDBHExpB", "ws_sdebVal", 0,
			ModelVector.FLOAT);

	/** Storm - proportion of trees with total damage that die that tip up */
	protected ModelVector mp_fStmPropFullTipUp = new ModelVector(
			"Storm - Prop. Heavy Damage Dead Trees that Tip Up",
			"st_stmPropTipUpFullDmg", "st_sptufdVal", 0, ModelVector.FLOAT);

	/** Random browse - browse probability */
	protected ModelVector mp_fRandomBrowseProb = new ModelVector(
			"Random Browse - Annual Browse Probability (0-1)", "di_randBrowseProb",
			"di_rbpVal", 0, ModelVector.FLOAT);
	
	/** Random browse - browse probability standard deviation */
	protected ModelVector mp_fRandomBrowseStdDev = new ModelVector(
			"Random Browse - Browse Probability Standard Deviation", 
			"di_randBrowseStdDev", "di_rbsdVal", 0, ModelVector.FLOAT);
	
	/*****************************************************************************
	 * Storm direct killer parameters
	 ****************************************************************************/
	/** Storm direct killer - "a" parameter */
	protected ModelVector mp_fStormDirectKillerA = new ModelVector(
			"Storm Direct Killer - \"a\"", "st_stormDirectKillerA",
			"st_sdkaVal", 0, ModelVector.FLOAT);
	
	/** Storm direct killer - "b" parameter */
	protected ModelVector mp_fStormDirectKillerB = new ModelVector(
			"Storm Direct Killer - \"b\"", "st_stormDirectKillerB",
			"st_sdkbVal", 0, ModelVector.FLOAT);

	/*****************************************************************************
	 * Competition Harvest Parameters
	 ****************************************************************************/

	/** Competition harvest - maximum crowding radius - species specific */
	protected ModelVector mp_fCompHarvMaxRadius = new ModelVector(
			"Competition Harvest: Max Radius of Competitive Effects (m)",
			"di_compHarvMaxCrowdingRadius", "di_chmcrVal", 0, ModelVector.FLOAT);

	/** Competition harvest - proportion of each species to cut as a value 
	 * between 0 and 1 - species specific. All 1s mean treat the species as a 
	 * common pool.*/
	protected ModelVector mp_fCompHarvProportion = new ModelVector(
			"Competition Harvest: Amount of Harvest Per Species (0 - 1)",
			"di_compHarvProportion", "di_chpVal", 0, ModelVector.FLOAT);

	/** Competition harvest - neighbor DBH effect (alpha) parameter for 
	 * competition calculation - species specific - required for all species*/
	protected ModelVector mp_fCompHarvAlpha = new ModelVector(
			"Competition Harvest: DBH Effect of Targets (alpha)", "di_compHarvAlpha",
			"di_chaVal", 0, ModelVector.FLOAT, true);

	/** Competition harvest - neighbor distance effect (beta) parameter for
	 * competition calculation - species specific - required for all species*/
	protected ModelVector mp_fCompHarvBeta = new ModelVector(
			"Competition Harvest: Distance Effect of Targets (beta)",
			"di_compHarvBeta", "di_chbVal", 0, ModelVector.FLOAT, true);

	/** Competition harvest - size sensitivity to competition parameter (gamma) 
	 * for competition calculations - species specific - required for all 
	 * species */
	protected ModelVector mp_fCompHarvGamma = new ModelVector(
			"Competition Harvest: Size Sensitivity (gamma)", "di_compHarvGamma",
			"di_chgVal", 0, ModelVector.FLOAT, true);

	/** Competition harvest - crowding effect slope (C) - species specific -
	 * required for all species*/
	protected ModelVector mp_fCompHarvC = new ModelVector(
			"Competition Harvest: C", "di_compHarvCrowdingSlope", "di_chcsVal", 0,
			ModelVector.FLOAT, true);

	/** Competition harvest - crowding effect steepness (D) - species specific -
	 * required for all species*/
	protected ModelVector mp_fCompHarvD = new ModelVector(
			"Competition Harvest: D", "di_compHarvCrowdingSteepness", "di_chcstVal",
			0, ModelVector.FLOAT, true);

	/** Competition harvest - NCI DBH divisor */
	protected ModelFloat m_fCompHarvQ = new ModelFloat(1,
			"Competition Harvest: Target DBH Divisor", "di_compHarvDbhDivisor");

	/** Competition harvest - minimum DBH for harvesting */
	protected ModelFloat m_fCompHarvMinDBH = new ModelFloat(0,
			"Competition Harvest: Minimum DBH to Harvest", "di_compHarvMinHarvDBH");

	/** Competition harvest - maximum DBH for harvesting */
	protected ModelFloat m_fCompHarvMaxDBH = new ModelFloat(1000,
			"Competition Harvest: Maximum DBH to Harvest", "di_compHarvMaxHarvDBH");

	/** Competition harvest - type of harvest - 0 = fixed interval, 1 = fixed BA
	 * threshold with fixed amount to cut, 2 = fixed BA threshold with percentage
	 * to cut */
	protected ModelEnum m_iCompHarvHarvType = new ModelEnum(
			new int[] { 2, 1, 0 }, new String[] { "Fixed BA %", "Fixed BA Amt",
					"Fixed Interval" }, "Competition Harvest: Harvest Type",
			"di_compHarvTypeHarvest");

	/** Competition harvest - cut amount - if this is a fixed interval harvest,
	 * this is the amount to which the plot is cut back, in m2/ha of basal area;
	 * if this is a fixed BA threshold harvest with fixed amount to cut, this is
	 * that amount to cut; if this is a fixed BA threshold harvest with percentage
	 * to cut, this is the proportion to cut between 0 and 1 */
	protected ModelFloat m_fCompHarvCutAmount = new ModelFloat(0,
			"Competition Harvest: Amount to Harvest", "di_compHarvCutAmount");

	/** Competition harvest - for fixed BA threshold harvests, the minimum 
	 * interval between harvests */
	protected ModelInt m_iCompHarvMinInterval = new ModelInt(1,
			"Competition Harvest: Min Years Between Fixed BA Harvests",
			"di_compHarvMinInterval");

	/** Competition harvest - for fixed interval threshold harvests, the 
	 * threshold */
	protected ModelInt m_iCompHarvInterval = new ModelInt(1,
			"Competition Harvest: Fixed Interval Harvest Interval (yr)",
			"di_compHarvInterval");

	/** Competition harvest - for fixed basal area threshold harvests, the
	 * threshold */
	protected ModelFloat m_fCompHarvBAThreshold = new ModelFloat(0,
			"Competition Harvest: Fixed BA Harvest Threshold (m2/ha)",
			"di_compHarvBAThreshold");

	/** Competition harvest - filename for list of trees harvested - optional */
	protected ModelString m_sCompHarvFilename = new ModelString("",
			"Competition Harvest: Filename for List of Harvested Trees",
			"di_compHarvHarvestedListFile");

	/** Storm return interval for 0-0.1 severity storm */
	protected ModelFloat m_fClass1RtrnInt = new ModelFloat(0,
			"Return Interval for Severity Storm Class 0 - 0.1", "st_s1ReturnInterval");

	/** Storm return interval for 0.1-0.2 severity storm */
	protected ModelFloat m_fClass2RtrnInt = new ModelFloat(0,
			"Return Interval for Severity Storm Class 0.1 - 0.2",
			"st_s2ReturnInterval");

	/** Storm return interval for 0.2-0.3 severity storm */
	protected ModelFloat m_fClass3RtrnInt = new ModelFloat(0,
			"Return Interval for Severity Storm Class 0.2 - 0.3",
			"st_s3ReturnInterval");

	/** Storm return interval for 0.3-0.4 severity storm */
	protected ModelFloat m_fClass4RtrnInt = new ModelFloat(0,
			"Return Interval for Severity Storm Class 0.3 - 0.4",
			"st_s4ReturnInterval");

	/** Storm return interval for 0.4-0.5 severity storm */
	protected ModelFloat m_fClass5RtrnInt = new ModelFloat(0,
			"Return Interval for Severity Storm Class 0.4 - 0.5",
			"st_s5ReturnInterval");

	/** Storm return interval for 0.5-0.6 severity storm */
	protected ModelFloat m_fClass6RtrnInt = new ModelFloat(0,
			"Return Interval for Severity Storm Class 0.5 - 0.6",
			"st_s6ReturnInterval");

	/** Storm return interval for 0.6-0.7 severity storm */
	protected ModelFloat m_fClass7RtrnInt = new ModelFloat(0,
			"Return Interval for Severity Storm Class 0.6 - 0.7",
			"st_s7ReturnInterval");

	/** Storm return interval for 0.7-0.8 severity storm */
	protected ModelFloat m_fClass8RtrnInt = new ModelFloat(0,
			"Return Interval for Severity Storm Class 0.7 - 0.8",
			"st_s8ReturnInterval");

	/** Storm return interval for 0.8-0.9 severity storm */
	protected ModelFloat m_fClass9RtrnInt = new ModelFloat(0,
			"Return Interval for Severity Storm Class 0.8 - 0.9",
			"st_s9ReturnInterval");

	/** Storm return interval for 0.9-1.0 severity storm */
	protected ModelFloat m_fClass10RtrnInt = new ModelFloat(0,
			"Return Interval for Severity Storm Class 0.9 - 1.0",
			"st_s10ReturnInterval");

	/** Storm - SST periodicity (Sr) */
	protected ModelFloat m_fStormSSTPeriod = new ModelFloat(1,
			"Storm - Sea Surface Temperature Cyclicity Period (Years)",
			"st_stmSSTPeriod");

	/** Storm - Sine function d */
	protected ModelFloat m_fStormSineD = new ModelFloat(0,
			"Storm - Storm Cyclicity Sine Curve d", "st_stmSineD");

	/** Storm - Sine function f */
	protected ModelFloat m_fStormSineF = new ModelFloat(1,
			"Storm - Storm Cyclicity Sine Curve f", "st_stmSineF");

	/** Storm - Sine function g */
	protected ModelFloat m_fStormSineG = new ModelFloat(1,
			"Storm - Storm Cyclicity Sine Curve g", "st_stmSineG");

	/** Storm - Trend function slope (m) */
	protected ModelFloat m_fStormTrendSlopeM = new ModelFloat(0,
			"Storm - Storm Cyclicity Trend Function Slope (m)", "st_stmTrendSlopeM");

	/** Storm - Trend function intercept (i) */
	protected ModelFloat m_fStormTrendInterceptI = new ModelFloat(1,
			"Storm - Storm Cyclicity Trend Function Intercept (i)",
			"st_stmTrendInterceptI");

	/** Windstorm - Storm intensity for 1 year return interval - LEM 03/19/06 */
	protected ModelFloat m_fWindstormReturnInt1Severity = new ModelFloat(0,
			"Windstorm - Severity for 1 Year Return Interval Storm",
			"ws_severityReturnInterval1");

	/** Windstorm - Storm intensity for 5 year return interval - LEM 03/19/06 */
	protected ModelFloat m_fWindstormReturnInt5Severity = new ModelFloat(0,
			"Windstorm - Severity for 5 Year Return Interval Storm",
			"ws_severityReturnInterval5");

	/** Windstorm - Storm intensity for 10 year return interval - LEM 03/19/06 */
	protected ModelFloat m_fWindstormReturnInt10Severity = new ModelFloat(0,
			"Windstorm - Severity for 10 Year Return Interval Storm",
			"ws_severityReturnInterval10");

	/** Windstorm - Storm intensity for 20 year return interval - LEM 03/19/06 */
	protected ModelFloat m_fWindstormReturnInt20Severity = new ModelFloat(0,
			"Windstorm - Severity for 20 Year Return Interval Storm",
			"ws_severityReturnInterval20");

	/** Windstorm - Storm intensity for 40 year return interval - LEM 03/19/06 */
	protected ModelFloat m_fWindstormReturnInt40Severity = new ModelFloat(0,
			"Windstorm - Severity for 40 Year Return Interval Storm",
			"ws_severityReturnInterval40");

	/** Windstorm - Storm intensity for 80 year return interval - LEM 03/19/06 */
	protected ModelFloat m_fWindstormReturnInt80Severity = new ModelFloat(0,
			"Windstorm - Severity for 80 Year Return Interval Storm",
			"ws_severityReturnInterval80");

	/** Windstorm - Storm intensity for 160 year return interval - LEM 03/19/06 */
	protected ModelFloat m_fWindstormReturnInt160Severity = new ModelFloat(0,
			"Windstorm - Severity for 160 Year Return Interval Storm",
			"ws_severityReturnInterval160");

	/** Windstorm - Storm intensity for 320 year return interval - LEM 03/19/06 */
	protected ModelFloat m_fWindstormReturnInt320Severity = new ModelFloat(0,
			"Windstorm - Severity for 320 Year Return Interval Storm",
			"ws_severityReturnInterval320");

	/** Windstorm - Storm intensity for 640 year return interval - LEM 03/19/06 */
	protected ModelFloat m_fWindstormReturnInt640Severity = new ModelFloat(0,
			"Windstorm - Severity for 640 Year Return Interval Storm",
			"ws_severityReturnInterval640");

	/** Windstorm - Storm intensity for 1280 year return interval - LEM 03/19/06 */
	protected ModelFloat m_fWindstormReturnInt1280Severity = new ModelFloat(0,
			"Windstorm - Severity for 1280 Year Return Interval Storm",
			"ws_severityReturnInterval1280");

	/** Windstorm - Storm intensity for 2560 year return interval - LEM 03/19/06 */
	protected ModelFloat m_fWindstormReturnInt2560Severity = new ModelFloat(0,
			"Windstorm - Severity for 2560 Year Return Interval Storm",
			"ws_severityReturnInterval2560");

	/** Windstorm - SST periodicity (Sr) */
	protected ModelFloat m_fWindstormSSTPeriod = new ModelFloat(1,
			"Windstorm - Sea Surface Temperature Cyclicity Period (Years)",
			"ws_stmSSTPeriod");

	/** Windstorm - Sine function d */
	protected ModelFloat m_fWindstormSineD = new ModelFloat(0,
			"Windstorm - Storm Cyclicity Sine Curve d", "ws_stmSineD");

	/** Windstorm - Sine function f */
	protected ModelFloat m_fWindstormSineF = new ModelFloat(1,
			"Windstorm - Storm Cyclicity Sine Curve f", "ws_stmSineF");

	/** Windstorm - Sine function g */
	protected ModelFloat m_fWindstormSineG = new ModelFloat(1,
			"Windstorm - Storm Cyclicity Sine Curve g", "ws_stmSineG");

	/** Windstorm - Trend function slope (m) */
	protected ModelFloat m_fWindstormTrendSlopeM = new ModelFloat(0,
			"Windstorm - Storm Cyclicity Trend Function Slope (m)",
			"ws_stmTrendSlopeM");

	/** Windstorm - Trend function intercept (i) */
	protected ModelFloat m_fWindstormTrendInterceptI = new ModelFloat(1,
			"Windstorm - Storm Cyclicity Trend Function Intercept (i)",
			"ws_stmTrendInterceptI");

	/** Standard deviation for normal and lognormal stochastic storm damage
	 * functions */
	protected ModelFloat m_fStdDev = new ModelFloat(0,
			"Standard Deviation (lognormal or normal)", "st_standardDeviation");

	/** Selection harvest cut range 1 lower DBH*/
	protected ModelFloat m_fCut1LoDBH = new ModelFloat(0,
			"Selection Harvest Cut Range 1 Lower DBH (cm)", "sha_loDBH");

	/** Selection harvest cut range 1 upper DBH*/
	protected ModelFloat m_fCut1HiDBH = new ModelFloat(0,
			"Selection Harvest Cut Range 1 Upper DBH (cm)", "sha_hiDBH");

	/** Selection harvest cut range 1 target BA*/
	protected ModelFloat m_fCut1BA = new ModelFloat(0,
			"Selection Harvest Cut Range 1 Target Basal Area (m2/ha)",
			"sha_target_BA");

	/** Selection harvest cut range 2 lower DBH*/
	protected ModelFloat m_fCut2LoDBH = new ModelFloat(0,
			"Selection Harvest Cut Range 2 Lower DBH (cm)", "sha_loDBH");

	/** Selection harvest cut range 2 upper DBH*/
	protected ModelFloat m_fCut2HiDBH = new ModelFloat(0,
			"Selection Harvest Cut Range 2 Upper DBH (cm)", "sha_hiDBH");

	/** Selection harvest cut range 2 target BA*/
	protected ModelFloat m_fCut2BA = new ModelFloat(0,
			"Selection Harvest Cut Range 2 Target Basal Area (m2/ha)",
			"sha_target_BA");

	/** Selection harvest cut range 3 lower DBH*/
	protected ModelFloat m_fCut3LoDBH = new ModelFloat(0,
			"Selection Harvest Cut Range 3 Lower DBH (cm)", "sha_loDBH");

	/** Selection harvest cut range 3 upper DBH*/
	protected ModelFloat m_fCut3HiDBH = new ModelFloat(0,
			"Selection Harvest Cut Range 3 Upper DBH (cm)", "sha_hiDBH");

	/** Selection harvest cut range 3 target BA*/
	protected ModelFloat m_fCut3BA = new ModelFloat(0,
			"Selection Harvest Cut Range 3 Target Basal Area (m2/ha)",
			"sha_target_BA");

	/** Selection harvest cut range 4 lower DBH */
	protected ModelFloat m_fCut4LoDBH = new ModelFloat(0,
			"Selection Harvest Cut Range 4 Lower DBH (cm)", "sha_loDBH");

	/** Selection harvest cut range 4 upper DBH */
	protected ModelFloat m_fCut4HiDBH = new ModelFloat(0,
			"Selection Harvest Cut Range 4 Upper DBH (cm)", "sha_hiDBH");

	/** Selection harvest cut range 4 target BA */
	protected ModelFloat m_fCut4BA = new ModelFloat(0,
			"Selection Harvest Cut Range 4 Target Basal Area (m2/ha)",
			"sha_target_BA");

	/** Selection harvest initial age */
	protected ModelInt m_iInitialAge = new ModelInt(0,
			"Selection Harvest Initial Age", "sha_InitialAge");

	/** Windstorm - Timestep to start storms */
	protected ModelInt m_iWindstormTimestepToStartStorms = new ModelInt(0,
			"Windstorm - Timestep to Start Storms", "ws_stmTSToStartStorms");

	/** Number of years damaged trees stay damaged */
	protected ModelInt m_iNumYearsToHeal = new ModelInt(0,
			"Number of Years Damaged Trees Take to Heal", "st_numYearsToHeal");

	/** Number of years storm-damaged snags last before being removed */
	protected ModelInt m_iNumSnagYears = new ModelInt(0,
			"Number of Years Storm-Damaged Snags Last", "st_numYearsStormSnags");

	/** Storm susceptibility pattern */
	protected ModelEnum m_iSusceptibility = new ModelEnum(new int[] { 0, 1 },
			new String[] { "Mapped", "Uniform" },
			"Plot Storm Susceptibility Pattern", "st_susceptibility");

	/** Storm stochasticity pattern */
	protected ModelEnum m_iStochasticity = new ModelEnum(new int[] { 0, 1 },
			new String[] { "Deterministic", "Stochastic" },
			"Storm Damage Application", "st_stochasticity");

	/** Probability distribution function */
	protected ModelEnum m_iProbDistFunc = new ModelEnum(new int[] { 0, 1 },
			new String[] { "Lognormal", "Normal" },
			"Stochastic Pattern Damage Distribution", "st_probFunction");
	
	/** Random browse - browse PDF */
	protected ModelEnum m_iRandomBrowsePDF = new ModelEnum(new int[] { 3, 0 },
			new String[] { "Normal", "None" },
			"Random Browse - Probability PDF", "di_randBrowsePDF");

	// *********************************
	// Harvest interface parameters
	// ********************************
	/** Path and filename of user executable */
	protected ModelString m_sHarvIntExecutable = new ModelString("", "",
			"hi_executable");

	/** Path and filename of file for SORTIE to write with harvestable trees */
	protected ModelString m_sHarvIntSORTIEOutFile = new ModelString("", "",
			"hi_harvestableTreesFile");

	/** Path and filename of file for executable to write with trees to harvest */
	protected ModelString m_sHarvIntExecHarvestOutFile = new ModelString("", "",
			"hi_treesToHarvestFile");

	/** Path and filename of file for executable to write with trees to update */
	protected ModelString m_sHarvIntExecUpdateOutFile = new ModelString("", "",
			"hi_treesToUpdateFile");

	/** Path and filename of batch parameters */
	protected ModelString m_sHarvIntBatchParamsFile = new ModelString("", "",
			"hi_batchParamsFile");

	/** Path and filename of single-run parameters file to write */
	protected ModelString m_sHarvIntBatchSingleRunParamsFile = new ModelString(
			"", "", "hi_batchSingleRunParamsFile");

	/** Arguments to pass to executable */
	protected ModelString m_sHarvIntExecArgs = new ModelString("", "",
			"hi_executableArguments");

	/** List of new tree data members to add */
	protected Vector mp_sHarvIntNewTreeDataMembers = new Vector(0);

	/** List of file columns */
	protected ModelVector mp_sHarvIntFileColumns = new ModelVector("",
			"hi_dataMembers", "hi_dataMember", 0, ModelVector.STRING);

	/** How often harvests occur, in years */
	protected ModelInt m_iHarvIntHarvestPeriod = new ModelInt(1, "",
			"hi_harvestPeriod");

	/**
	 * Placeholder for reading cut ranges from an XML file. This one is for DBH
	 * range lower bound
	 */
	protected float m_iCutRangeLowerBound = -1;

	/**
	 * Placeholder for reading cut ranges from an XML file. This one is for DBH
	 * range upper bound
	 */
	protected float m_iCutRangeUpperBound = -1;

	/**
	 * Placeholder for reading cut ranges from an XML file. This one is for range
	 * cut amount
	 */
	protected float m_iCutRangeAmount = -1;

	/** Placeholder for reading selection harvest cut ranges from an XML file. */
	protected int m_iSelectionHarvestCutRange = 0;

	/**
	 * Constructor.
	 * 
	 * @param oManager
	 *          GUIManager object.
	 * @throws ModelException
	 *           if there's a problem.
	 */
	public DisturbanceBehaviors(GUIManager oManager) throws ModelException {
		super(oManager, "Disturbance", "");

		mp_sHarvIntFileColumns.SetIsSpeciesSpecific(false);

		// Set up disturbance behaviors
		mp_oChildBehaviors = new Behavior[11];
		int iIndex = 0;

		// Competition Harvest
		mp_oChildBehaviors[iIndex] = new Behavior("Competition Harvest",
				"Competition Harvest", "Competition Harvest", (float) 1.0);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fCompHarvMaxRadius);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fCompHarvProportion);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fCompHarvAlpha);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fCompHarvBeta);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fCompHarvGamma);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fCompHarvC);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fCompHarvD);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_iCompHarvHarvType);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fCompHarvCutAmount);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_iCompHarvMinInterval);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_iCompHarvInterval);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fCompHarvBAThreshold);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fCompHarvMinDBH);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fCompHarvMaxDBH);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fCompHarvQ);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_sCompHarvFilename);
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SNAG, false);
		// This behavior does use a data member but since it has no meaning
		// outside the behavior I am not adding it to the GUI
		iIndex++;

		// Episodic mortality behavior
		mp_oChildBehaviors[iIndex] = new Behavior("Episodic Mortality",
				"Episodic Mortality", "episodic mortality", (float) 1);
		mp_oChildBehaviors[iIndex].m_bCanEditAutomatically = false;
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = false;
		iIndex++;

		// Harvest behavior
		mp_oChildBehaviors[iIndex] = new Behavior("Harvest", "Harvest", "harvest",
				(float) 1);
		mp_oChildBehaviors[iIndex].m_bCanEditAutomatically = false;
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = false;
		iIndex++;

		// Random browse
		mp_oChildBehaviors[iIndex] = new Behavior("Random Browse", "Random Browse",
				"random browse", (float) 1.0);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_iRandomBrowsePDF);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fRandomBrowseProb);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fRandomBrowseStdDev);
		mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
				"Browsed", "Browsed", DataMember.BOOLEAN));
		iIndex++;

		mp_oChildBehaviors[iIndex] = new Behavior("Selection Harvest",
				"Selection Harvest", "SelectionHarvest", (float) 1.0);
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = false;
		mp_oChildBehaviors[iIndex].AddRequiredData(m_iInitialAge);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fCut1LoDBH);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fCut1HiDBH);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fCut1BA);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fCut2LoDBH);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fCut2HiDBH);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fCut2BA);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fCut3LoDBH);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fCut3HiDBH);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fCut3BA);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fCut4LoDBH);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fCut4HiDBH);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fCut4BA);
		iIndex++;

		// Storm behavior
		mp_oChildBehaviors[iIndex] = new Behavior("Storms", "Storms", "storm",
				(float) 1);
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = false;
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fClass1RtrnInt);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fClass2RtrnInt);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fClass3RtrnInt);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fClass4RtrnInt);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fClass5RtrnInt);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fClass6RtrnInt);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fClass7RtrnInt);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fClass8RtrnInt);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fClass9RtrnInt);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fClass10RtrnInt);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_iSusceptibility);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_iStochasticity);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_iProbDistFunc);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fStdDev);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fStormSSTPeriod);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fStormSineD);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fStormSineF);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fStormSineG);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fStormTrendSlopeM);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fStormTrendInterceptI);
		iIndex++;

		// Storm Damage Applier behavior
		mp_oChildBehaviors[iIndex] = new Behavior("Storm Damage Applier",
				"Storm Damage Applier", "storm damage applier", (float) 1.0);
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = true;
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMinStormDamageDBH);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStmDmgInterceptMed);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStmDmgInterceptFull);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStmIntensityCoeff);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStmDBHCoeff);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_iNumYearsToHeal);
		// Storm damage counter data member
		mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
				"Storm Damage Value", "stm_dmg", DataMember.INTEGER));
		iIndex++;

		// Storm Killer behavior
		mp_oChildBehaviors[iIndex] = new Behavior("Storm Killer", "Storm Damage Killer",
				"storm killer", (float) 1.0);
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = true;
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMinStormDamageDBH);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStmMedDmgSurvProbA);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStmMedDmgSurvProbB);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStmFullDmgSurvProbA);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStmFullDmgSurvProbB);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStmPropFullTipUp);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_iNumSnagYears);
		iIndex++;
		
    //Storm direct killer behavior
		mp_oChildBehaviors[iIndex] = new Behavior("Storm Direct Killer",
				"Storm Direct Killer", "storm direct killer", (float) 1.0);
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = true;
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, true);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStormDirectKillerA);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStormDirectKillerB);
		iIndex++;

		// Windstorm behavior
		mp_oChildBehaviors[iIndex] = new Behavior("Windstorm", "Windstorm",
				"Windstorm", (float) 2.0);
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = true;
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWindstormMortInterceptA);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWindstormStmIntensCoeffC);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWindstormDBHExpB);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWindstormMinDBH);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fWindstormReturnInt1Severity);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fWindstormReturnInt5Severity);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fWindstormReturnInt10Severity);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fWindstormReturnInt20Severity);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fWindstormReturnInt40Severity);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fWindstormReturnInt80Severity);
		mp_oChildBehaviors[iIndex]
				.AddRequiredData(m_fWindstormReturnInt160Severity);
		mp_oChildBehaviors[iIndex]
				.AddRequiredData(m_fWindstormReturnInt320Severity);
		mp_oChildBehaviors[iIndex]
				.AddRequiredData(m_fWindstormReturnInt640Severity);
		mp_oChildBehaviors[iIndex]
				.AddRequiredData(m_fWindstormReturnInt1280Severity);
		mp_oChildBehaviors[iIndex]
				.AddRequiredData(m_fWindstormReturnInt2560Severity);
		mp_oChildBehaviors[iIndex]
				.AddRequiredData(m_iWindstormTimestepToStartStorms);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fWindstormSSTPeriod);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fWindstormSineD);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fWindstormSineF);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fWindstormSineG);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fWindstormTrendSlopeM);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fWindstormTrendInterceptI);
		iIndex++;

		// Harvest interface
		mp_oChildBehaviors[iIndex] = new Behavior("Harvest Interface",
				"Harvest Interface", "Harvest Interface", (float) 1);
		mp_oChildBehaviors[iIndex].m_bCanEditAutomatically = false;
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = false;
		/*
		 * mp_oChildBehaviors[7].AddRequiredData(m_sHarvIntExecutable);
		 * mp_oChildBehaviors[7].AddRequiredData(m_sHarvIntSORTIEOutFile);
		 * mp_oChildBehaviors[7].AddRequiredData(m_sHarvIntExecHarvestOutFile);
		 * mp_oChildBehaviors[7].AddRequiredData(m_sHarvIntExecUpdateOutFile);
		 * mp_oChildBehaviors[7].AddRequiredData(m_sHarvIntBatchParamsFile);
		 * mp_oChildBehaviors[7].AddRequiredData(m_sHarvIntBatchSingleRunParamsFile);
		 * mp_oChildBehaviors[7].AddRequiredData(m_sHarvIntExecArgs);
		 * mp_oChildBehaviors[7].AddRequiredData(mp_sHarvIntFileColumns);
		 * mp_oChildBehaviors[7].AddRequiredData(m_iHarvIntHarvestPeriod);
		 */
		iIndex++;

		mp_oAllData.add(m_fClass1RtrnInt);
		mp_oAllData.add(m_fClass2RtrnInt);
		mp_oAllData.add(m_fClass3RtrnInt);
		mp_oAllData.add(m_fClass4RtrnInt);
		mp_oAllData.add(m_fClass5RtrnInt);
		mp_oAllData.add(m_fClass6RtrnInt);
		mp_oAllData.add(m_fClass7RtrnInt);
		mp_oAllData.add(m_fClass8RtrnInt);
		mp_oAllData.add(m_fClass9RtrnInt);
		mp_oAllData.add(m_fClass10RtrnInt);
		mp_oAllData.add(m_iSusceptibility);
		mp_oAllData.add(m_iStochasticity);
		mp_oAllData.add(m_iProbDistFunc);
		mp_oAllData.add(m_fStdDev);
		mp_oAllData.add(mp_fMinStormDamageDBH);
		mp_oAllData.add(mp_fStmDmgInterceptMed);
		mp_oAllData.add(mp_fStmDmgInterceptFull);
		mp_oAllData.add(mp_fStmIntensityCoeff);
		mp_oAllData.add(mp_fStmDBHCoeff);
		mp_oAllData.add(mp_fStmMedDmgSurvProbA);
		mp_oAllData.add(mp_fStmMedDmgSurvProbB);
		mp_oAllData.add(mp_fStmFullDmgSurvProbA);
		mp_oAllData.add(mp_fStmFullDmgSurvProbB);
		mp_oAllData.add(m_iNumYearsToHeal);
		mp_oAllData.add(mp_fStmPropFullTipUp);
		mp_oAllData.add(m_iNumSnagYears);
		mp_oAllData.add(m_iInitialAge);
		mp_oAllData.add(m_fCut1LoDBH);
		mp_oAllData.add(m_fCut1HiDBH);
		mp_oAllData.add(m_fCut1BA);
		mp_oAllData.add(m_fCut2LoDBH);
		mp_oAllData.add(m_fCut2HiDBH);
		mp_oAllData.add(m_fCut2BA);
		mp_oAllData.add(m_fCut3LoDBH);
		mp_oAllData.add(m_fCut3HiDBH);
		mp_oAllData.add(m_fCut3BA);
		mp_oAllData.add(m_fCut4LoDBH);
		mp_oAllData.add(m_fCut4HiDBH);
		mp_oAllData.add(m_fCut4BA);
		mp_oAllData.add(mp_fWindstormMortInterceptA);
		mp_oAllData.add(mp_fWindstormStmIntensCoeffC);
		mp_oAllData.add(mp_fWindstormDBHExpB);
		mp_oAllData.add(mp_fWindstormMinDBH);
		mp_oAllData.add(m_fWindstormReturnInt1Severity);
		mp_oAllData.add(m_fWindstormReturnInt5Severity);
		mp_oAllData.add(m_fWindstormReturnInt10Severity);
		mp_oAllData.add(m_fWindstormReturnInt20Severity);
		mp_oAllData.add(m_fWindstormReturnInt40Severity);
		mp_oAllData.add(m_fWindstormReturnInt80Severity);
		mp_oAllData.add(m_fWindstormReturnInt160Severity);
		mp_oAllData.add(m_fWindstormReturnInt320Severity);
		mp_oAllData.add(m_fWindstormReturnInt640Severity);
		mp_oAllData.add(m_fWindstormReturnInt1280Severity);
		mp_oAllData.add(m_fWindstormReturnInt2560Severity);
		mp_oAllData.add(m_iWindstormTimestepToStartStorms);
		mp_oAllData.add(m_fWindstormSSTPeriod);
		mp_oAllData.add(m_fWindstormSineD);
		mp_oAllData.add(m_fWindstormSineF);
		mp_oAllData.add(m_fWindstormSineG);
		mp_oAllData.add(m_fWindstormTrendSlopeM);
		mp_oAllData.add(m_fWindstormTrendInterceptI);
		mp_oAllData.add(m_sHarvIntExecutable);
		mp_oAllData.add(m_sHarvIntSORTIEOutFile);
		mp_oAllData.add(m_sHarvIntExecHarvestOutFile);
		mp_oAllData.add(m_sHarvIntExecUpdateOutFile);
		mp_oAllData.add(m_sHarvIntBatchParamsFile);
		mp_oAllData.add(m_sHarvIntBatchSingleRunParamsFile);
		mp_oAllData.add(m_sHarvIntExecArgs);
		mp_oAllData.add(mp_sHarvIntFileColumns);
		mp_oAllData.add(m_iHarvIntHarvestPeriod);
		mp_oAllData.add(m_fStormSSTPeriod);
		mp_oAllData.add(m_fStormSineD);
		mp_oAllData.add(m_fStormSineF);
		mp_oAllData.add(m_fStormSineG);
		mp_oAllData.add(m_fStormTrendSlopeM);
		mp_oAllData.add(m_fStormTrendInterceptI);
		mp_oAllData.add(m_iRandomBrowsePDF);
		mp_oAllData.add(mp_fRandomBrowseProb);
		mp_oAllData.add(mp_fRandomBrowseStdDev);
		mp_oAllData.add(mp_fCompHarvMaxRadius);
		mp_oAllData.add(mp_fCompHarvProportion);
		mp_oAllData.add(mp_fCompHarvAlpha);
		mp_oAllData.add(mp_fCompHarvBeta);
		mp_oAllData.add(mp_fCompHarvGamma);
		mp_oAllData.add(mp_fCompHarvC);
		mp_oAllData.add(mp_fCompHarvD);
		mp_oAllData.add(m_iCompHarvHarvType);
		mp_oAllData.add(m_fCompHarvCutAmount);
		mp_oAllData.add(m_iCompHarvMinInterval);
		mp_oAllData.add(m_iCompHarvInterval);
		mp_oAllData.add(m_fCompHarvBAThreshold);
		mp_oAllData.add(m_fCompHarvMinDBH);
		mp_oAllData.add(m_fCompHarvMaxDBH);
		mp_oAllData.add(m_fCompHarvQ);
		mp_oAllData.add(m_sCompHarvFilename);
		mp_oAllData.add(mp_fStormDirectKillerA);
		mp_oAllData.add(mp_fStormDirectKillerB);

		//Default the random browse PDF to "None"
		m_iRandomBrowsePDF.SetValue(0);
		
		// We have to wait to set up the grids
	}

	/**
	 * This adds the data contained in a HarvestData object to a list of harvest
	 * data. A HarvestData object is first checked to make sure there are any cut
	 * ranges defined; if not, nothing happens. If there is data to be added, it
	 * is compared to existing HarvestData objects on the list. If they match
	 * except for the grid cell, then the new object's grid cell is added to the
	 * existing object and the new object thrown away. If it is truly new data, it
	 * is added to the list.
	 * 
	 * IMPORTANT: It is assumed that the harvest data list is all single-species
	 * cuts. This makes this method not very generalized.
	 * 
	 * @param oList
	 *          The list of HarvestData objects to add to.
	 * @param oNewHarvest
	 *          The HarvestData object to add.
	 * @return The new list with the data added.
	 * @throws ModelException
	 *           Passing through underlying exceptions.
	 */
	protected Vector AddHarvestData(Vector oList, HarvestData oNewHarvest)
			throws ModelException {
		HarvestData oExistingHarvest;
		int i, j, k;
		boolean bMatch, bAllMatch;

		if (null == oNewHarvest) {
			return oList;
		}

		// If there are no cut ranges, don't add
		if (oNewHarvest.GetNumberOfCutRanges() == 0) {
			return null;
		}

		// If the list is currently null, create it
		if (null == oList) {
			oList = new Vector(0);
			oList.add(oNewHarvest);
			return oList;
		}

		for (i = 0; i < oList.size(); i++) {

			oExistingHarvest = (HarvestData) oList.elementAt(i);

			// Compare all the data
			if (oExistingHarvest.GetCutAmountType() == oNewHarvest.GetCutAmountType()
					&& oExistingHarvest.GetCutType() == oNewHarvest.GetCutType()
					&& oExistingHarvest.GetTimestep() == oNewHarvest.GetTimestep()
					&& oExistingHarvest.GetNumberOfCutRanges() == oNewHarvest
							.GetNumberOfCutRanges()
					&& oExistingHarvest.GetSpecies(0) == oNewHarvest.GetSpecies(0)) {

				// Make sure the cut ranges are the same
				bAllMatch = true;
				for (j = 0; j < oNewHarvest.GetNumberOfCutRanges(); j++) {
					bMatch = false;
					for (k = 0; k < oExistingHarvest.GetNumberOfCutRanges(); k++) {
						if (oNewHarvest.GetLowerBound(j) == oExistingHarvest
								.GetLowerBound(k)
								&& oNewHarvest.GetUpperBound(j) == oExistingHarvest
										.GetUpperBound(k)
								&& oNewHarvest.GetCutAmount(j) == oExistingHarvest
										.GetCutAmount(k)) {
							bMatch = true;
							break;
						}
					}
					if (!bMatch) {
						bAllMatch = false;
						break;
					}
				}

				if (bAllMatch) {
					// Add this one's grid cell to the existing list
					for (j = 0; j < oNewHarvest.GetNumberOfCells(); j++) {
						oExistingHarvest.AddCell(oNewHarvest.GetCell(j));
					}
					return oList;
				}
			}
		}

		// If we've gotten to here, then the new object's data is unique and should
		// be added.
		oList.add(oNewHarvest);
		return oList;
	}

	/**
	 * Takes a list of HarvestData objects where each is one single species, and
	 * replaces it with a list of HarvestData objects for multiple species for any
	 * whose data match.
	 * 
	 * @param oList
	 *          The list to compact.
	 * @return The compacted list.
	 * @throws ModelException
	 *           Passing through underlying exceptions
	 */
	protected Vector CompactHarvestList(Vector oList) throws ModelException {

		Vector oTempList = new Vector(0);
		HarvestData oExistingHarvest, oNewHarvest;
		Cell oNewCell, oExistingCell;
		int i, j, k, m;
		boolean bUnique = true, bMatch, bAllMatch;

		// If the list is null, return
		if (null == oList) {
			return null;
		}

		for (i = 0; i < oList.size(); i++) {

			oNewHarvest = (HarvestData) oList.elementAt(i);
			bUnique = true;
			// See if this object's data matches any existing object
			for (j = 0; j < oTempList.size(); j++) {

				oExistingHarvest = (HarvestData) oTempList.elementAt(j);

				if (oExistingHarvest.GetCutAmountType() == oNewHarvest
						.GetCutAmountType()
						&& oExistingHarvest.GetCutType() == oNewHarvest.GetCutType()
						&& oExistingHarvest.GetTimestep() == oNewHarvest.GetTimestep()
						&& oExistingHarvest.GetNumberOfCutRanges() == oNewHarvest
								.GetNumberOfCutRanges()
						&& oExistingHarvest.GetNumberOfCells() == oNewHarvest
								.GetNumberOfCells()) {

					// Test cut ranges
					bAllMatch = true;
					for (k = 0; k < oNewHarvest.GetNumberOfCutRanges(); k++) {
						bMatch = false;
						for (m = 0; m < oExistingHarvest.GetNumberOfCutRanges(); m++) {
							if (oNewHarvest.GetLowerBound(k) == oExistingHarvest
									.GetLowerBound(m)
									&& oNewHarvest.GetUpperBound(k) == oExistingHarvest
											.GetUpperBound(m)
									&& oNewHarvest.GetCutAmount(k) == oExistingHarvest
											.GetCutAmount(m)) {
								bMatch = true;
								break;
							}
						}
						if (!bMatch) {
							bAllMatch = false;
							break;
						}
					}

					// If cut ranges checked out, test the cell list
					if (bAllMatch) {

						bAllMatch = true;
						for (k = 0; k < oNewHarvest.GetNumberOfCells(); k++) {
							oNewCell = (Cell) oNewHarvest.GetCell(k);
							bMatch = false;
							for (m = 0; m < oExistingHarvest.GetNumberOfCells(); m++) {
								oExistingCell = (Cell) oExistingHarvest.GetCell(m);
								if (oNewCell.GetX() == oExistingCell.GetX()
										&& oNewCell.GetY() == oExistingCell.GetY()) {
									bMatch = true;
									break;
								}
							}
							if (!bMatch) {
								bAllMatch = false;
								break;
							}
						}
						if (bAllMatch) {
							bUnique = false;
							oExistingHarvest.AddSpecies(oNewHarvest.GetSpecies(0));
						}
					}
				}
			}

			if (bUnique) {
				// This object has unique data - add it to the temp list
				oTempList.add(oNewHarvest);
			}
		}

		// Return the temp list to the existing list
		return oTempList;
	}

	/**
	 * Writes the storm data to the parameter file. Does nothing if the storm
	 * behaviors are not enabled.
	 * 
	 * @param jOut
	 *          FileWriter File to write to.
	 * @throws IOException
	 *           if there is a problem writing the file.
	 * @throws ModelException
	 *           passed through from called functions.
	 */
	private void WriteStorm(BufferedWriter jOut) throws java.io.IOException,
			ModelException {

		if (GetBehaviorByKey("Storms").m_bIsEnabled) {
			jOut.write("<storm>");

			WriteData(jOut, m_fClass1RtrnInt);
			WriteData(jOut, m_fClass2RtrnInt);
			WriteData(jOut, m_fClass3RtrnInt);
			WriteData(jOut, m_fClass4RtrnInt);
			WriteData(jOut, m_fClass5RtrnInt);
			WriteData(jOut, m_fClass6RtrnInt);
			WriteData(jOut, m_fClass7RtrnInt);
			WriteData(jOut, m_fClass8RtrnInt);
			WriteData(jOut, m_fClass9RtrnInt);
			WriteData(jOut, m_fClass10RtrnInt);
			WriteData(jOut, m_iSusceptibility);
			WriteData(jOut, m_iStochasticity);
			WriteData(jOut, m_fStormSSTPeriod);
			WriteData(jOut, m_fStormSineD);
			WriteData(jOut, m_fStormSineF);
			WriteData(jOut, m_fStormSineG);
			WriteData(jOut, m_fStormTrendSlopeM);
			WriteData(jOut, m_fStormTrendInterceptI);
			if (m_iStochasticity.GetStringValue().equals("Stochastic")) {
				WriteData(jOut, m_iProbDistFunc);

				if (m_iProbDistFunc.GetStringValue().equalsIgnoreCase("Lognormal")
						|| m_iProbDistFunc.GetStringValue().equalsIgnoreCase("Normal")) {
					WriteData(jOut, m_fStdDev);
				}
			}

			if (GetBehaviorByKey("Storm Damage Applier").m_bIsEnabled) {
				TreePopulation oPop = m_oManager.GetTreePopulation();
				boolean[] p_bAppliesTo = GetBehaviorByKey("Storm Damage Applier")
						.GetWhichSpeciesUsed(oPop);
				WriteSpeciesSpecificValue(jOut, mp_fMinStormDamageDBH, oPop,
						p_bAppliesTo);
				WriteSpeciesSpecificValue(jOut, mp_fStmDmgInterceptMed, oPop,
						p_bAppliesTo);
				WriteSpeciesSpecificValue(jOut, mp_fStmDmgInterceptFull, oPop,
						p_bAppliesTo);
				WriteSpeciesSpecificValue(jOut, mp_fStmIntensityCoeff, oPop,
						p_bAppliesTo);
				WriteSpeciesSpecificValue(jOut, mp_fStmDBHCoeff, oPop, p_bAppliesTo);
				WriteData(jOut, m_iNumYearsToHeal);
			}

			if (GetBehaviorByKey("Storm Killer").m_bIsEnabled) {
				TreePopulation oPop = m_oManager.GetTreePopulation();
				boolean[] p_bAppliesTo = GetBehaviorByKey("Storm Killer")
						.GetWhichSpeciesUsed(oPop);
				WriteSpeciesSpecificValue(jOut, mp_fStmMedDmgSurvProbA, oPop,
						p_bAppliesTo);
				WriteSpeciesSpecificValue(jOut, mp_fStmMedDmgSurvProbB, oPop,
						p_bAppliesTo);
				WriteSpeciesSpecificValue(jOut, mp_fStmFullDmgSurvProbA, oPop,
						p_bAppliesTo);
				WriteSpeciesSpecificValue(jOut, mp_fStmFullDmgSurvProbB, oPop,
						p_bAppliesTo);
				WriteSpeciesSpecificValue(jOut, mp_fStmPropFullTipUp, oPop,
						p_bAppliesTo);
				WriteData(jOut, m_iNumSnagYears);
			}
			
			//Write scheduled storms data
			if (mp_oScheduledStorms.size() > 0) {
				jOut.write("<st_stmScheduledStorms>");
				ScheduledStormInfo oStorm;
				int i;
				for (i = 0; i < mp_oScheduledStorms.size(); i++) {
					oStorm = (ScheduledStormInfo) mp_oScheduledStorms.elementAt(i);
					jOut.write("<st_stmEvent min=\"" + oStorm.fMin + "\" max=\"" +
							oStorm.fMax + "\" yr=\"" + oStorm.iYear + "\"/>");
				}
				jOut.write("</st_stmScheduledStorms>");
			}
			
			//Write storm direct killer info
			if (GetBehaviorByKey("Storm Direct Killer").m_bIsEnabled) {
				TreePopulation oPop = m_oManager.GetTreePopulation();
				boolean[] p_bAppliesTo = GetBehaviorByKey("Storm Direct Killer")
						.GetWhichSpeciesUsed(oPop);
				WriteSpeciesSpecificValue(jOut, mp_fStormDirectKillerA, oPop,
						p_bAppliesTo);
				WriteSpeciesSpecificValue(jOut, mp_fStormDirectKillerB, oPop,
						p_bAppliesTo);
				
			}

			jOut.write("</storm>");
		}
	}

	/**
	 * Writes the windstorm data to the parameter file. Does nothing if the
	 * windstorm behavior is not enabled.
	 * 
	 * @param jOut
	 *          FileWriter File to write to.
	 * @throws IOException
	 *           if there is a problem writing the file.
	 * @throws ModelException
	 *           passed through from called functions.
	 */
	private void WriteWindstorm(BufferedWriter jOut) throws java.io.IOException,
			ModelException {

		Behavior oBeh = GetBehaviorByKey("Windstorm");
		if (oBeh.m_bIsEnabled) {

			TreePopulation oPop = m_oManager.GetTreePopulation();
			boolean[] p_bAppliesTo = oBeh.GetWhichSpeciesUsed(oPop);

			jOut.write("<windstorm>");

			WriteData(jOut, m_fWindstormReturnInt1Severity);
			WriteData(jOut, m_fWindstormReturnInt5Severity);
			WriteData(jOut, m_fWindstormReturnInt10Severity);
			WriteData(jOut, m_fWindstormReturnInt20Severity);
			WriteData(jOut, m_fWindstormReturnInt40Severity);
			WriteData(jOut, m_fWindstormReturnInt80Severity);
			WriteData(jOut, m_fWindstormReturnInt160Severity);
			WriteData(jOut, m_fWindstormReturnInt320Severity);
			WriteData(jOut, m_fWindstormReturnInt640Severity);
			WriteData(jOut, m_fWindstormReturnInt1280Severity);
			WriteData(jOut, m_fWindstormReturnInt2560Severity);
			WriteData(jOut, m_fWindstormSSTPeriod);
			WriteData(jOut, m_fWindstormSineD);
			WriteData(jOut, m_fWindstormSineF);
			WriteData(jOut, m_fWindstormSineG);
			WriteData(jOut, m_fWindstormTrendSlopeM);
			WriteData(jOut, m_fWindstormTrendInterceptI);
			WriteSpeciesSpecificValue(jOut, mp_fWindstormMinDBH, oPop, p_bAppliesTo);
			WriteSpeciesSpecificValue(jOut, mp_fWindstormMortInterceptA, oPop,
					p_bAppliesTo);
			WriteSpeciesSpecificValue(jOut, mp_fWindstormStmIntensCoeffC, oPop,
					p_bAppliesTo);
			WriteSpeciesSpecificValue(jOut, mp_fWindstormDBHExpB, oPop, p_bAppliesTo);

			WriteData(jOut, m_iWindstormTimestepToStartStorms);

			jOut.write("</windstorm>");
		}
	}
	
	/**
	 * Writes the competition harvest data to the parameter file. Does nothing if
	 * the competition harvest behavior is not enabled.
	 * 
	 * @param jOut
	 *          FileWriter File to write to.
	 * @throws IOException
	 *           if there is a problem writing the file.
	 * @throws ModelException
	 *           passed through from called functions.
	 */
	private void WriteCompetitionHarvest(BufferedWriter jOut) throws java.io.IOException,
			ModelException {

		Behavior oBeh = GetBehaviorByKey("Competition Harvest");
		if (oBeh.m_bIsEnabled) {

			TreePopulation oPop = m_oManager.GetTreePopulation();
			boolean[] p_bAppliesTo = oBeh.GetWhichSpeciesUsed(oPop);
			ModelData oData;
			ModelVector oLambda;
			String sKey;
			int i, iIndex;
			
			WriteSpeciesSpecificValue(jOut, mp_fCompHarvMaxRadius, oPop, p_bAppliesTo);
			WriteSpeciesSpecificValue(jOut, mp_fCompHarvProportion, oPop, p_bAppliesTo);
			WriteSpeciesSpecificValue(jOut, mp_fCompHarvAlpha, oPop);
			WriteSpeciesSpecificValue(jOut, mp_fCompHarvBeta, oPop);
			WriteSpeciesSpecificValue(jOut, mp_fCompHarvGamma, oPop);
			WriteSpeciesSpecificValue(jOut, mp_fCompHarvC, oPop);
			WriteSpeciesSpecificValue(jOut, mp_fCompHarvD, oPop);
			
			//Write the lambdas
			iIndex = 0;
			for (i = 0; i < mp_oAllData.size(); i++) {
				oData = (ModelData) mp_oAllData.elementAt(i);
				sKey = oData.GetDescriptor().toLowerCase();
				if (sKey.indexOf("target lambda") > -1) {
					iIndex = i;
					break;	
				}
			}	
			for (i = 0; i < p_bAppliesTo.length; i++) {
				if (p_bAppliesTo[i]) {
					oLambda = (ModelVector) mp_oAllData.elementAt(iIndex + i);
					WriteSpeciesSpecificValue(jOut, oLambda, oPop);
				}
			}
			
			WriteData(jOut, m_fCompHarvQ);
			WriteData(jOut, m_fCompHarvMinDBH);
			WriteData(jOut, m_fCompHarvMaxDBH);
			WriteData(jOut, m_iCompHarvHarvType);
			WriteData(jOut, m_fCompHarvCutAmount);
			if (m_iCompHarvHarvType.GetStringValue().equals("Fixed Interval")) {
				WriteData(jOut, m_iCompHarvInterval);
			} else {
				WriteData(jOut, m_iCompHarvMinInterval);
				WriteData(jOut, m_fCompHarvBAThreshold);
			}
			if (m_sCompHarvFilename.GetValue().trim().length() > 0) {
				WriteData(jOut, m_sCompHarvFilename);
			}	
		}
	}

	/**
	 * Writes the random browse data to the parameter file. Does nothing if the
	 * random browse behavior is not enabled.
	 * 
	 * @param jOut
	 *          FileWriter File to write to.
	 * @throws IOException
	 *           if there is a problem writing the file.
	 * @throws ModelException
	 *           passed through from called functions.
	 */
	private void WriteRandomBrowse(BufferedWriter jOut)
			throws java.io.IOException, ModelException {

		Behavior oBeh = GetBehaviorByKey("Random Browse");
		if (oBeh.m_bIsEnabled) {

			TreePopulation oPop = m_oManager.GetTreePopulation();
			boolean[] p_bAppliesTo = oBeh.GetWhichSpeciesUsed(oPop);

			WriteData(jOut, m_iRandomBrowsePDF);
			WriteSpeciesSpecificValue(jOut, mp_fRandomBrowseProb, oPop, p_bAppliesTo);
			if (m_iRandomBrowsePDF.GetValue() > 0) {
				WriteSpeciesSpecificValue(jOut, mp_fRandomBrowseStdDev, oPop, p_bAppliesTo);
			}
		}
	}

	/**
	 * Writes the selection harvest data to the parameter file. Does nothing if
	 * the selection harvest behavior is not enabled.
	 * 
	 * @param jOut
	 *          FileWriter File to write to.
	 * @throws IOException
	 *           if there is a problem writing the file.
	 * @throws ModelException
	 *           passed through from called functions.
	 */
	private void WriteSelectionHarvest(BufferedWriter jOut)
			throws java.io.IOException, ModelException {

		if (GetBehaviorByKey("Selection Harvest").m_bIsEnabled) {
			jOut.write("<SelectionHarvest>");

			WriteData(jOut, m_iInitialAge);

			if (m_fCut1BA.GetValue() > 0) {
				jOut.write("<sha_CutRange>");
				WriteData(jOut, m_fCut1LoDBH);
				WriteData(jOut, m_fCut1HiDBH);
				WriteData(jOut, m_fCut1BA);
				jOut.write("</sha_CutRange>");
			}
			if (m_fCut2BA.GetValue() > 0) {
				jOut.write("<sha_CutRange>");
				WriteData(jOut, m_fCut2LoDBH);
				WriteData(jOut, m_fCut2HiDBH);
				WriteData(jOut, m_fCut2BA);
				jOut.write("</sha_CutRange>");
			}
			if (m_fCut3BA.GetValue() > 0) {
				jOut.write("<sha_CutRange>");
				WriteData(jOut, m_fCut3LoDBH);
				WriteData(jOut, m_fCut3HiDBH);
				WriteData(jOut, m_fCut3BA);
				jOut.write("</sha_CutRange>");
			}
			if (m_fCut4BA.GetValue() > 0) {
				jOut.write("<sha_CutRange>");
				WriteData(jOut, m_fCut4LoDBH);
				WriteData(jOut, m_fCut4HiDBH);
				WriteData(jOut, m_fCut4BA);
				jOut.write("</sha_CutRange>");
			}

			jOut.write("</SelectionHarvest>");
		}
	}

	/**
	 * Writes the episodic mortality data for the parameter file. Does nothing if
	 * there is no episodic mortality data.
	 * 
	 * @param jOut
	 *          FileWriter File to write to.
	 * @param oPop
	 *          Tree population object.
	 * @throws java.io.IOException
	 *           if there is a problem writing the file
	 * @throws ModelException
	 *           if there are unrecognized disturbance cut types (should never
	 *           happen).
	 */
	private void WriteMortalityEpisode(BufferedWriter jOut, TreePopulation oPop)
			throws java.io.IOException, ModelException {
		HarvestData oData;
		Cell oCell;
		int i, iNumObjects = mp_oMortEpisodes.size(), j, iTemp;

		if (iNumObjects > 0 && GetBehaviorByKey("Episodic Mortality").m_bIsEnabled) {

			jOut.write("<disturbance>");

			// Write each harvest object's data
			for (i = 0; i < iNumObjects; i++) {
				oData = (HarvestData) mp_oMortEpisodes.elementAt(i);

				jOut.write("<ds_deathEvent>");

				// Write each species
				iTemp = oData.GetNumberOfSpecies();
				for (j = 0; j < iTemp; j++) {
					jOut.write("<ds_applyToSpecies species=\""
							+ oPop.GetSpeciesNameFromCode(oData.GetSpecies(j)) + "\"/>");
				}

				// Write the timestep
				jOut.write("<ds_timestep>" + String.valueOf(oData.GetTimestep())
						+ "</ds_timestep>");

				// Write the cut amount type
				jOut.write("<ds_cutAmountType>");
				iTemp = oData.GetCutAmountType();
				if (iTemp == ABSOLUTE_BASAL_AREA) {
					jOut.write("absolute basal area");
				} else if (iTemp == ABSOLUTE_DENSITY) {
					jOut.write("absolute density");
				} else if (iTemp == PERCENTAGE_BASAL_AREA) {
					jOut.write("percent of basal area");
				} else if (iTemp == PERCENTAGE_DENSITY) {
					jOut.write("percent of density");
				} else {
					throw (new ModelException(ErrorGUI.BAD_DATA, "JAVA",
							"When writing parameter file, came across unrecognized "
									+ "mortality episode cut type \"" + String.valueOf(iTemp)
									+ "\"."));
				}
				jOut.write("</ds_cutAmountType>");

				// Write the cut ranges
				iTemp = oData.GetNumberOfCutRanges();
				for (j = 0; j < iTemp; j++) {

					jOut.write("<ds_dbhRange>");
					jOut.write("<ds_low>" + String.valueOf(oData.GetLowerBound(j))
							+ "</ds_low>");
					jOut.write("<ds_high>" + String.valueOf(oData.GetUpperBound(j))
							+ "</ds_high>");
					jOut.write("<ds_amountToCut>" + String.valueOf(oData.GetCutAmount(j))
							+ "</ds_amountToCut>");
					jOut.write("</ds_dbhRange>");
				}

				// Write the list of cells
				iTemp = oData.GetNumberOfCells();
				for (j = 0; j < iTemp; j++) {

					oCell = (Cell) oData.GetCell(j);

					jOut.write("<ds_applyToCell x=\"" + String.valueOf(oCell.GetX())
							+ "\" y=\"" + String.valueOf(oCell.GetY()) + "\"/>");
				}

				jOut.write("</ds_deathEvent>");
			}

			jOut.write("</disturbance>");
		}
	}

	/**
	 * Writes the harvest data for the parameter file. Does nothing if there is no
	 * harvest data.
	 * 
	 * @param jOut
	 *          FileWriter File to write to.
	 * @param oPop
	 *          Tree population object.
	 * @throws java.io.IOException
	 *           if there is a problem writing the file
	 * @throws ModelException
	 *           if there are unrecognized harvest cut types (should never
	 *           happen).
	 */
	private void WriteHarvest(BufferedWriter jOut, TreePopulation oPop)
			throws java.io.IOException, ModelException {
		HarvestData oData;
		Cell oCell;
		int i, iNumObjects = mp_oHarvestCuts.size(), j, iTemp;

		// ************************
		// Harvest
		// ************************

		if (iNumObjects > 0 && GetBehaviorByKey("Harvest").m_bIsEnabled) {

			jOut.write("<harvest>");

			// Write each harvest object's data
			for (i = 0; i < iNumObjects; i++) {
				oData = (HarvestData) mp_oHarvestCuts.elementAt(i);

				jOut.write("<ha_cutEvent>");

				// Write each species
				iTemp = oData.GetNumberOfSpecies();
				for (j = 0; j < iTemp; j++) {
					jOut.write("<ha_applyToSpecies species=\""
							+ oPop.GetSpeciesNameFromCode(oData.GetSpecies(j)) + "\"/>");
				}

				// Write the timestep
				jOut.write("<ha_timestep>" + String.valueOf(oData.GetTimestep())
						+ "</ha_timestep>");

				// Write the cut type
				jOut.write("<ha_cutType>");
				iTemp = oData.GetCutType();
				if (iTemp == GAP_CUT) {
					jOut.write("gap");
				} else if (iTemp == PARTIAL_CUT) {
					jOut.write("partial");
				} else if (iTemp == CLEAR_CUT) {
					jOut.write("clear");
				} else {
					throw (new ModelException(ErrorGUI.BAD_DATA, "JAVA",
							"When writing parameter file, came across unrecognized "
									+ "harvest cut type \"" + String.valueOf(iTemp) + "\"."));
				}
				jOut.write("</ha_cutType>");

				// Write the cut amount type
				jOut.write("<ha_cutAmountType>");
				iTemp = oData.GetCutAmountType();
				if (iTemp == ABSOLUTE_BASAL_AREA) {
					jOut.write("absolute basal area");
				} else if (iTemp == ABSOLUTE_DENSITY) {
					jOut.write("absolute density");
				} else if (iTemp == PERCENTAGE_BASAL_AREA) {
					jOut.write("percent of basal area");
				} else if (iTemp == PERCENTAGE_DENSITY) {
					jOut.write("percent of density");
				} else {
					throw (new ModelException(ErrorGUI.BAD_DATA, "JAVA",
							"When writing parameter file, came across unrecognized "
									+ "harvest cut type \"" + String.valueOf(iTemp) + "\"."));
				}
				jOut.write("</ha_cutAmountType>");

				// Write the cut ranges
				iTemp = oData.GetNumberOfCutRanges();
				for (j = 0; j < iTemp; j++) {

					jOut.write("<ha_dbhRange>");
					jOut.write("<ha_low>" + String.valueOf(oData.GetLowerBound(j))
							+ "</ha_low>");
					jOut.write("<ha_high>" + String.valueOf(oData.GetUpperBound(j))
							+ "</ha_high>");
					jOut.write("<ha_amountToCut>" + String.valueOf(oData.GetCutAmount(j))
							+ "</ha_amountToCut>");
					jOut.write("</ha_dbhRange>");
				}

				// Write the list of cells
				iTemp = oData.GetNumberOfCells();
				for (j = 0; j < iTemp; j++) {

					oCell = (Cell) oData.GetCell(j);

					jOut.write("<ha_applyToCell x=\"" + String.valueOf(oCell.GetX())
							+ "\" y=\"" + String.valueOf(oCell.GetY()) + "\"/>");
				}

				jOut.write("</ha_cutEvent>");
			}

			jOut.write("</harvest>");
		}

	}

	/**
	 * Writes the harvest interface data for the parameter file. Does nothing if
	 * the harvest interface behavior is not being used.
	 * 
	 * @param jOut
	 *          FileWriter File to write to.
	 * @throws java.io.IOException
	 *           if there is a problem writing the file
	 */
	private void WriteHarvestInterface(BufferedWriter jOut)
			throws java.io.IOException {
		if (GetBehaviorByKey("Harvest Interface").m_bIsEnabled == false)
			return;

		jOut.write("<harvestInterface>");

		// Our parameters are not "assigned" to any behavior, so we have to use the
		// WorkerBase static method to write them. Simply using WriteData would
		// prevent them from being written, since it belongs to BehaviorTypeBase and
		// checks to make sure data is applied to an enabled behavior.

		// User executable filename
		WorkerBase.WriteDataToFile(jOut, m_sHarvIntExecutable);

		// Filename of harvestable trees for SORTIE to write
		WorkerBase.WriteDataToFile(jOut, m_sHarvIntSORTIEOutFile);

		// Filename of trees to harvest for the user executable
		WorkerBase.WriteDataToFile(jOut, m_sHarvIntExecHarvestOutFile);

		// How often harvests occur
		WorkerBase.WriteDataToFile(jOut, m_iHarvIntHarvestPeriod);

		// Filename of trees to update for the user executable to write, if being
		// used
		if (m_sHarvIntExecUpdateOutFile.GetValue().length() > 0) {
			WorkerBase.WriteDataToFile(jOut, m_sHarvIntExecUpdateOutFile);
		}

		// Filename of batch parameters, if being used
		if (m_sHarvIntBatchParamsFile.GetValue().length() > 0) {
			WorkerBase.WriteDataToFile(jOut, m_sHarvIntBatchParamsFile);
			WorkerBase.WriteDataToFile(jOut, m_sHarvIntBatchSingleRunParamsFile);
		}

		// Arguments to pass to user executable, if being used
		if (m_sHarvIntExecArgs.GetValue().length() > 0) {
			WorkerBase.WriteDataToFile(jOut, m_sHarvIntExecArgs);
		}

		// List of tree data members to include in the files beyond the required
		// list, if being used
		if (mp_sHarvIntFileColumns.GetValue().size() > 0) {
			WorkerBase.WriteVectorValueToFile(jOut, mp_sHarvIntFileColumns);
		}

		jOut.write("</harvestInterface>");
	}

	/**
	 * Validates the data prior to writing it. What it does:
	 * <ul>
	 * <li>Causes all HarvestData objects to validate themselves.</li>
	 * <li>If the Storms behavior is enabled, this makes sure that all storm
	 * return intervals are greater than 0.</li>
	 * <li>If the Storms behavior is enabled and the susceptibility is set to
	 * "mapped", this makes sure that there are values for the grid object "Storm
	 * Susceptibility" and they are greater than 0.</li>
	 * <li>If the Storms behavior is enabled and the damage pattern is set to
	 * "mapped", this makes sure that the grid resolution for the "Storm
	 * Susceptibility" and "Storm Damage" grids is the same.</li>
	 * <li>If the Storm Damage Applier behavior is enabled, this checks to make
	 * sure the Storms behavior is enabled.</li>
	 * <li>If the Storm Damage Applier behavior is enabled, this checks to make
	 * sure the value in m_iNumYearsToHeal is positive.</li>
	 * <li>If the Storm Killer behavior is enabled, this checks to make sure that
	 * both the Storm and Storm Damage Applier behaviors are enabled.</li>
	 * <li>If the Storm Killer behavior is enabled, this checks to make sure the
	 * value in m_iNumSnagYears is not negative.</li>
	 * <li>If the Storm Damage Killer behavior is enabled, this makes sure that
	 * the proportion of dead trees that tip up are all proportions.</li>
	 * <li>If Selection Harvest is enabled, this makes sure that target basal
	 * area is not less than 0 and cut ranges don't overlap</li>
	 * <li>If Windstorm is enabled, this makes sure that:
	 * <ul>
	 * <li>Minimum DBH not negative for any value</li>
	 * <li>Timestep to start storms not negative</li>
	 * <li>Storm severities are all proportions</li>
	 * <li>Some mortality behavior is applied to all trees</li>
	 * </ul>
	 * <li>If Competition Harvest is enabled, this makes sure that:
	 * <ul>
	 * <li>If harvest type is fixed interval, the interval value is at least 
	 * 1</li>
	 * <li>If harvest type is fixed BA with proportion to cut, the cut amount is
	 * a proportion between 0 and 1</li>
	 * <li>Species proportions to cut either add up to 1 or are all 1</li>
	 * <li>If harvest type is fixed BA, the minimum interval is at least 1</li>
	 * <li>All values in max crowding radius are greater than 0</li>
	 * </ul></li>
	 * </ul>
	 * 
	 * @param oPop TreePopulation object
	 * @throws ModelException if one of the data objects is not valid.
	 */
	public void ValidateData(TreePopulation oPop) throws ModelException {
		HarvestData oData;
		Plot oPlot = m_oManager.GetPlot();
		int i, iNumObjects;

		// Disable the Harvest behavior if there are no harvest events
		// No longer - Selection Harvest may enable it
		// if (iNumObjects == 0) {
		// GetBehaviorByKey("Harvest").m_bIsEnabled = false;
		// }

		// Validate each harvest event
		iNumObjects = mp_oHarvestCuts.size();
		for (i = 0; i < iNumObjects; i++) {
			oData = (HarvestData) mp_oHarvestCuts.elementAt(i);
			oData.ValidateCut(oPop, oPlot);
		}

		// Disable the Episodic Mortality behavior if there are no mortality
		// episodes
		// events
		// if (iNumObjects == 0) {
		// GetBehaviorByKey("Episodic Mortality").m_bIsEnabled = false;
		// }

		// Validate each Episodic Mortality event
		iNumObjects = mp_oMortEpisodes.size();
		for (i = 0; i < iNumObjects; i++) {
			oData = (HarvestData) mp_oMortEpisodes.elementAt(i);
			oData.ValidateCut(oPop, oPlot);
		}

		// Storms behavior validation
		if (GetBehaviorByKey("Storms").m_bIsEnabled) {

			// Make sure all the return intervals are non-negative
			MakeSureGreaterThanEqualTo(m_fClass1RtrnInt, 0);
			MakeSureGreaterThanEqualTo(m_fClass2RtrnInt, 0);
			MakeSureGreaterThanEqualTo(m_fClass3RtrnInt, 0);
			MakeSureGreaterThanEqualTo(m_fClass4RtrnInt, 0);
			MakeSureGreaterThanEqualTo(m_fClass5RtrnInt, 0);
			MakeSureGreaterThanEqualTo(m_fClass6RtrnInt, 0);
			MakeSureGreaterThanEqualTo(m_fClass7RtrnInt, 0);
			MakeSureGreaterThanEqualTo(m_fClass8RtrnInt, 0);
			MakeSureGreaterThanEqualTo(m_fClass9RtrnInt, 0);
			MakeSureGreaterThanEqualTo(m_fClass10RtrnInt, 0);

			// If the damage pattern is mapped, make sure there are values for the
			// "Storm Susceptibility" grid and that they are all > 0
			if (m_iSusceptibility.GetStringValue().equalsIgnoreCase("mapped")) {
				Grid oSusceptibility = m_oManager.GetGridByName("Storm Susceptibility");
				if (oSusceptibility == null) {
					// "Storm Susceptibility" is missing - shouldn't happen, but a
					// good double-check
					ModelException oErr = new ModelException(
							ErrorGUI.DATA_MISSING,
							"JAVA",
							"The grid \"Storm Susceptibility\" is required for mapped storm damage patterns.");
					throw (oErr);
				}

				iNumObjects = oSusceptibility.mp_oGridVals.size();
				if (iNumObjects == 0) {
					// Map is missing for "Storm Susceptibility"
					ModelException oErr = new ModelException(
							ErrorGUI.DATA_MISSING,
							"JAVA",
							"A map of values for the grid \"Storm Susceptibility\" is required for mapped storm damage patterns.");
					throw (oErr);
				}

				for (i = 0; i < iNumObjects; i++) {
					GridValue oVal = (GridValue) oSusceptibility.mp_oGridVals
							.elementAt(i);
					if (oVal.mp_fFloats == null || oVal.mp_fFloats[0] == null
							|| oVal.mp_fFloats[0].floatValue() < 0) {
						ModelException oErr = new ModelException(
								ErrorGUI.DATA_MISSING,
								"JAVA",
								"All values in the map for the grid \"Storm Susceptibility\" must be greater than 0.");
						throw (oErr);
					}
				}

				// Get the grid "Storm Damage" and make sure it has the same resolution
				// as "Storm Susceptibility"
				Grid oDamage = m_oManager.GetGridByName("Storm Damage");
				if (oDamage != null) {
					if (oDamage.m_bEdited) {
						if (java.lang.Math.abs(oDamage.GetXCellLength()
								- oSusceptibility.GetXCellLength()) > 0.01
								|| java.lang.Math.abs(oDamage.GetYCellLength()
										- oSusceptibility.GetYCellLength()) > 0.01) {
							ModelException oErr = new ModelException(ErrorGUI.BAD_DATA,
									"JAVA",
									"The grids \"Storm Susceptibility\" and \"Storm Damage\" "
											+ "must have matching grid cell resolutions.");
							throw (oErr);

						}
					}
				}
			}
		}

		if (GetBehaviorByKey("Storm Damage Applier").m_bIsEnabled) {
			if (GetBehaviorByKey("Storms").m_bIsEnabled == false) {
				ModelException oErr = new ModelException(ErrorGUI.BAD_DATA, "JAVA",
						"When the behavior \"Storm Damage Applier\" is enabled, "
								+ "the behavior \"Storms\" must be enabled as well.");
				throw (oErr);
			}
			MakeSureGreaterThan(m_iNumYearsToHeal, 0);
		}

		if (GetBehaviorByKey("Storm Killer").m_bIsEnabled) { // LEM 01/09/06
			Behavior oKiller = GetBehaviorByKey("Storm Killer"), oDamageApplier = GetBehaviorByKey("Storm Damage Applier");
			if (oDamageApplier.m_bIsEnabled == false) {
				ModelException oErr = new ModelException(
						ErrorGUI.BAD_DATA,
						"JAVA",
						"When the behavior \"Storm Killer\" is enabled, "
								+ "the behavior \"Storm Damage Applier\" must be enabled as well.");
				throw (oErr);
			}

			// Make sure every species/type combo for storm killer has also been
			// assigned to storm damage applier
			SpeciesTypeCombo oKillerCombo, oDamageCombo;
			int j;
			boolean bFound;
			for (i = 0; i < oKiller.GetNumberOfCombos(); i++) {
				bFound = false;
				oKillerCombo = oKiller.GetSpeciesTypeCombo(i);
				for (j = 0; j < oDamageApplier.GetNumberOfCombos(); j++) {
					oDamageCombo = oDamageApplier.GetSpeciesTypeCombo(j);
					if (oKillerCombo.equals(oDamageCombo)) {
						bFound = true;
						break;
					}
				}
				if (bFound == false) {
					ModelException oErr = new ModelException(
							ErrorGUI.BAD_DATA,
							"JAVA",
							"All trees to which the behavior \"Storm Killer\" is applied "
									+ "must also have the behavior \"Storm Damage Applier\" applied as well.");
					throw (oErr);

				}
			}

			boolean[] p_bAppliesTo = GetBehaviorByKey("Storm Killer")
					.GetWhichSpeciesUsed(oPop);
			MakeSureAllAreProportions(mp_fStmPropFullTipUp, p_bAppliesTo);
			if (m_oManager.GetSnagAwareness() == true) {
				MakeSureGreaterThan(m_iNumSnagYears, 0);
			}
		}

		// LEM 05-12-05
		if (GetBehaviorByKey("Selection Harvest").m_bIsEnabled) {

			GetBehaviorByKey("Harvest").m_bIsEnabled = true;

			MakeSureGreaterThan(m_fCut1BA, 0);
			MakeSureGreaterThan(m_fCut2BA, 0);
			MakeSureGreaterThan(m_fCut3BA, 0);
			MakeSureGreaterThan(m_fCut4BA, 0);

			// Make sure the upper bound of each cut range is larger than the lower
			// bound
			if (m_fCut1HiDBH.GetValue() < m_fCut1LoDBH.GetValue()
					|| m_fCut2HiDBH.GetValue() < m_fCut2LoDBH.GetValue()
					|| m_fCut3HiDBH.GetValue() < m_fCut3LoDBH.GetValue()
					|| m_fCut4HiDBH.GetValue() < m_fCut4LoDBH.GetValue()) {
				throw new ModelException(
						ErrorGUI.BAD_DATA,
						"JAVA",
						"Selection Harvest cut range upper DBHs must be greater than corresponding lower DBHs.");
			}
		}

		// LEM 03/19/06
		if (GetBehaviorByKey("Windstorm").m_bIsEnabled) {
			Behavior oWindstorm = GetBehaviorByKey("Windstorm");
			// Make sure all storm severity values are proportions
			MakeSureIsProportion(m_fWindstormReturnInt1Severity);
			MakeSureIsProportion(m_fWindstormReturnInt5Severity);
			MakeSureIsProportion(m_fWindstormReturnInt10Severity);
			MakeSureIsProportion(m_fWindstormReturnInt20Severity);
			MakeSureIsProportion(m_fWindstormReturnInt40Severity);
			MakeSureIsProportion(m_fWindstormReturnInt80Severity);
			MakeSureIsProportion(m_fWindstormReturnInt160Severity);
			MakeSureIsProportion(m_fWindstormReturnInt320Severity);
			MakeSureIsProportion(m_fWindstormReturnInt640Severity);
			MakeSureIsProportion(m_fWindstormReturnInt1280Severity);
			MakeSureIsProportion(m_fWindstormReturnInt2560Severity);

			// Make sure the timestep to start storms is not negative
			MakeSureGreaterThanEqualTo(m_iWindstormTimestepToStartStorms, 0);

			// Make sure the SST periodicity is not 0
			MakeSureNotEqualTo(m_fWindstormSSTPeriod, 0);

			// Make sure minimum DBH values are not negative
			boolean[] p_bApplies = oWindstorm.GetWhichSpeciesUsed(oPop);
			MakeSureAllNonNegative(mp_fWindstormMinDBH, p_bApplies);

			// Make sure there's a mortality behavior applied to every tree to
			// which windstorm is applied
			Behavior[] p_oMortBeh = m_oManager.GetMortalityBehaviors()
					.GetAllBehaviors();
			SpeciesTypeCombo oWindstormCombo, oMortCombo;
			int iBeh, j, iNumWindstormCombos = oWindstorm.GetNumberOfCombos(), iNumMortCombos;
			boolean bFound;
			for (i = 0; i < iNumWindstormCombos; i++) {
				bFound = false;
				oWindstormCombo = oWindstorm.GetSpeciesTypeCombo(i);
				for (iBeh = 0; iBeh < p_oMortBeh.length; iBeh++) {
					iNumMortCombos = p_oMortBeh[iBeh].GetNumberOfCombos();
					for (j = 0; j < iNumMortCombos; j++) {
						oMortCombo = p_oMortBeh[iBeh].GetSpeciesTypeCombo(j);
						if (oWindstormCombo.equals(oMortCombo)) {
							bFound = true;
							break;
						}
					}
					if (bFound)
						break;
				}
				if (bFound == false) {
					ModelException oErr = new ModelException(ErrorGUI.BAD_DATA, "JAVA",
							"All trees to which the behavior \"" + oWindstorm.GetDescriptor()
									+ "\" is applied "
									+ "must also have a mortality behavior applied as well.");
					throw (oErr);

				}
			}
		}

		//Random Browse validation
		if (GetBehaviorByKey("Random Browse").m_bIsEnabled) {
			MakeSureAllAreProportions(mp_fRandomBrowseProb);
		}
		
		//Competition Harvest validation
		if (GetBehaviorByKey("Competition Harvest").m_bIsEnabled) {
			boolean[] p_bAppliesTo = GetBehaviorByKey("Competition Harvest").GetWhichSpeciesUsed(oPop);
			Float fVal;
			float fTotal;
			int iCount;
			
			MakeSureAllPositive(mp_fCompHarvMaxRadius, p_bAppliesTo);
			MakeSureAllAreProportions(mp_fCompHarvProportion, p_bAppliesTo);
			MakeSureNotEqualTo(m_fCompHarvQ, 0);
			MakeSureGreaterThanEqualTo(m_fCompHarvMinDBH, 0);
			MakeSureGreaterThanEqualTo(m_fCompHarvMaxDBH, 0);
			
			if (m_iCompHarvHarvType.GetStringValue().equals("Fixed Interval")) {
				MakeSureGreaterThanEqualTo(m_fCompHarvCutAmount, 0);
				MakeSureGreaterThan(m_iCompHarvInterval, 0);
			} else if (m_iCompHarvHarvType.GetStringValue().equals("Fixed BA Amt")) {
				MakeSureGreaterThanEqualTo(m_fCompHarvCutAmount, 0);
				MakeSureGreaterThan(m_iCompHarvMinInterval, 0);
				MakeSureGreaterThan(m_fCompHarvBAThreshold, 0);
			} else if (m_iCompHarvHarvType.GetStringValue().equals("Fixed BA %")) {
				MakeSureIsProportion(m_fCompHarvCutAmount);
				MakeSureGreaterThan(m_iCompHarvMinInterval, 0);
				MakeSureGreaterThan(m_fCompHarvBAThreshold, 0);
			}
			
			//If any of the values in the proportions are not 1, make sure they
			//add up to 1
			iCount = 0;
			for (i = 0; i < p_bAppliesTo.length; i++)
				if (p_bAppliesTo[i]) iCount++;
			fTotal = 0;
			for (i = 0; i < mp_fCompHarvProportion.GetValue().size(); i++) {
				fVal = (Float) mp_fCompHarvProportion.GetValue().elementAt(i);
				if (null != fVal && p_bAppliesTo[i]) {
					fTotal += fVal.floatValue();
				}
			}
			if (java.lang.Math.abs(fTotal - iCount) > 0.001 && 
					java.lang.Math.abs(fTotal - 1.0) > 0.001) {
				throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
						"The values in \"" + mp_fCompHarvProportion.GetDescriptor()
						+ "\" must either all be 1 or add up to 1."));
			}
		}
	}

	/**
	 * Overridden because disturbance writing is special.
	 * 
	 * @param jOut File to write to.
	 * @param oPop TreePopulation object
	 * @throws javawrapper.ModelException if there's a problem writing the file.
	 */
	public void WriteXML(BufferedWriter jOut, TreePopulation oPop)
			throws ModelException {
		try {

			WriteHarvest(jOut, oPop);
			WriteMortalityEpisode(jOut, oPop);
			WriteStorm(jOut);
			WriteSelectionHarvest(jOut); // LEM 05/12/05
			WriteWindstorm(jOut); // LEM 03/16/06
			WriteHarvestInterface(jOut); // LEM 09-29-06
			
			//Write the stuff in the "disturbance other" category
			if (GetBehaviorByKey("Competition Harvest").m_bIsEnabled ||
					GetBehaviorByKey("Random Browse").m_bIsEnabled) {
				jOut.write("<disturbanceOther>");
			  WriteRandomBrowse(jOut);
			  WriteCompetitionHarvest(jOut);
			  jOut.write("</disturbanceOther>");
			}

		} catch (java.io.IOException e) {
			throw (new ModelException(ErrorGUI.BAD_FILE, "JAVA",
					"There was a problem writing the parameter file."));

		}
	}

	/**
	 * Sets up the grid objects.
	 * 
	 * @param oPop TreePopulation object.
	 * @throws javawrapper.ModelException passthrough only.
	 */
	public void DoSetup(TreePopulation oPop) throws javawrapper.ModelException {

		int iNumSpecies = oPop.GetNumberOfSpecies(), i, j, k, m;

		Grid oNewGrid;

		// ********************************
		// Harvest grid
		// ********************************

		String sGridName = "Harvest Results";

		// The accessible data members are one for each cut range and species
		String[] p_sBaseMembers = { "Cut Density", "Cut Basal Area" };
		DataMember[] p_oDataMembers = new DataMember[p_sBaseMembers.length
				* iNumSpecies * HarvestData.NUMBER_ALLOWED_CUT_RANGES + 1];

		p_oDataMembers[0] = new DataMember("Harvest Type", "Harvest Type",
				DataMember.INTEGER);

		m = 1;
		for (i = 0; i < p_sBaseMembers.length; i++) {
			if (p_sBaseMembers[i].equals("Cut Density")) {
				for (j = 0; j < iNumSpecies; j++) {
					for (k = 0; k < HarvestData.NUMBER_ALLOWED_CUT_RANGES; k++) {
						p_oDataMembers[m] = new DataMember(p_sBaseMembers[i] + ", "
								+ oPop.GetSpeciesNameFromCode(j).replace('_', ' ')
								+ ", cut range " + k, p_sBaseMembers[i] + "_" + k + "_" + j,
								DataMember.INTEGER);
						m++;
					}
				}
			} else {
				for (j = 0; j < iNumSpecies; j++) {
					for (k = 0; k < HarvestData.NUMBER_ALLOWED_CUT_RANGES; k++) {
						p_oDataMembers[m] = new DataMember(p_sBaseMembers[i] + ", "
								+ oPop.GetSpeciesNameFromCode(j).replace('_', ' ')
								+ ", cut range " + k, p_sBaseMembers[i] + "_" + k + "_" + j,
								DataMember.FLOAT);
						m++;
					}
				}
			}
		}

		// Create our grid
		oNewGrid = new Grid(sGridName, p_oDataMembers, null, 8, 8);

		// Add it to the harvest behavior
		oNewGrid = m_oManager.AddGrid(oNewGrid);
		GetBehaviorByKey("Harvest").AddGrid(oNewGrid);

		// ********************************
		// Episodic Mortality grid
		// ********************************

		sGridName = "Mortality Episode Results";

		// The accessible data members are one for each cut range and species
		p_oDataMembers = new DataMember[p_sBaseMembers.length * iNumSpecies
				* HarvestData.NUMBER_ALLOWED_CUT_RANGES];

		m = 0;
		for (i = 0; i < p_sBaseMembers.length; i++) {
			if (p_sBaseMembers[i].equals("Cut Density")) {
				for (j = 0; j < iNumSpecies; j++) {
					for (k = 0; k < HarvestData.NUMBER_ALLOWED_CUT_RANGES; k++) {
						p_oDataMembers[m] = new DataMember(p_sBaseMembers[i] + ", "
								+ oPop.GetSpeciesNameFromCode(j).replace('_', ' ')
								+ ", cut range " + k, p_sBaseMembers[i] + "_" + k + "_" + j,
								DataMember.INTEGER);
						m++;
					}
				}
			} else {
				for (j = 0; j < iNumSpecies; j++) {
					for (k = 0; k < HarvestData.NUMBER_ALLOWED_CUT_RANGES; k++) {
						p_oDataMembers[m] = new DataMember(p_sBaseMembers[i] + ", "
								+ oPop.GetSpeciesNameFromCode(j).replace('_', ' ')
								+ ", cut range " + k, p_sBaseMembers[i] + "_" + k + "_" + j,
								DataMember.FLOAT);
						m++;
					}
				}
			}
		}

		// Create our grid
		oNewGrid = new Grid(sGridName, p_oDataMembers, null, 8, 8);

		// Add it to the episodic mortality behavior
		oNewGrid = m_oManager.AddGrid(oNewGrid);
		GetBehaviorByKey("Episodic Mortality").AddGrid(oNewGrid);

		// ********************************
		// Storm Damage grid
		// ********************************

		sGridName = "Storm Damage";

		p_oDataMembers = new DataMember[2];
		p_oDataMembers[0] = new DataMember("Damage Index", "dmg_index",
				DataMember.FLOAT);
		p_oDataMembers[1] = new DataMember("Time Since Last Storm", "stormtime",
				DataMember.FLOAT);

		DataMember[] p_oPackageDataMembers = new DataMember[1];
		p_oPackageDataMembers[0] = new DataMember("Single Storm Damage Index",
				"1dmg_index", DataMember.FLOAT);
		oNewGrid = new Grid(sGridName, p_oDataMembers, p_oPackageDataMembers, 8, 8);

		// Add to the Storms behavior
		oNewGrid = m_oManager.AddGrid(oNewGrid);
		GetBehaviorByKey("Storms").AddGrid(oNewGrid);

		// ********************************
		// Storm Susceptibility grid
		// ********************************

		sGridName = "Storm Susceptibility";

		// The accessible data members are one for each cut range and species
		p_oDataMembers = new DataMember[1];
		p_oDataMembers[0] = new DataMember("Susceptibility", "index",
				DataMember.FLOAT);

		// Create our grid
		oNewGrid = new Grid(sGridName, p_oDataMembers, null, 8, 8);

		// Add to the Storms behavior
		oNewGrid = m_oManager.AddGrid(oNewGrid);
		GetBehaviorByKey("Storms").AddGrid(oNewGrid);

		// ********************************
		// Windstorm Results grid
		// ********************************
		p_oDataMembers = new DataMember[(2 * iNumSpecies) + 1];
		Plot p_oPlot = m_oManager.GetPlot();

		sGridName = "Windstorm Results";

		// The accessible data members are storm severity, and a basal area and
		// density killed for each species
		for (i = 0; i < iNumSpecies; i++) {

			p_oDataMembers[i] = new DataMember("Basal Area Dead For "
					+ oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "ba_" + i,
					DataMember.FLOAT);
		}

		for (i = 0; i < iNumSpecies; i++) {

			p_oDataMembers[i + iNumSpecies] = new DataMember("Density Dead For "
					+ oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "density_" + i,
					DataMember.FLOAT);
		}

		p_oDataMembers[2 * iNumSpecies] = new DataMember("Storm Severity",
				"severity", DataMember.FLOAT);

		// Create our grid
		oNewGrid = new Grid(sGridName, null, p_oDataMembers, p_oPlot
				.GetPlotXLength(), p_oPlot.GetPlotYLength());

		// Assign the grid to the windstorm behavior
		oNewGrid = m_oManager.AddGrid(oNewGrid);
		GetBehaviorByKey("Windstorm").AddGrid(oNewGrid);

		// ********************************
		// Competition Harvest Results grid
		// ********************************

		sGridName = "Competition Harvest Results";

		// The accessible data members are two for each species
		p_oDataMembers = new DataMember[2 * iNumSpecies];

		for (i = 0; i < iNumSpecies; i++) {
			p_oDataMembers[i] = new DataMember("Cut Density, "
					+ oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "Cut Density_"
					+ i, DataMember.INTEGER);
		}
		for (i = 0; i < iNumSpecies; i++) {
			p_oDataMembers[i + iNumSpecies] = new DataMember("Cut Basal Area, "
					+ oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "Cut Basal Area_"
					+ i, DataMember.FLOAT);
		}

		// Create our grid
		Plot oPlot = m_oManager.GetPlot();
		oNewGrid = new Grid(sGridName, p_oDataMembers, null,
				oPlot.GetPlotXLength(), oPlot.GetPlotYLength());

		// Add it to the episodic mortality behavior
		oNewGrid = m_oManager.AddGrid(oNewGrid);
		GetBehaviorByKey("Competition Harvest").AddGrid(oNewGrid);

		// ********************************
		// Set up lambdas for competition harvest
		// ********************************
		Behavior oBeh = GetBehaviorByKey("Competition Harvest");
		boolean bAlreadyLambdas = false;

		// See if we already had lambdas
		for (i = 0; i < mp_oAllData.size(); i++) {
			ModelData oData = (ModelData) mp_oAllData.elementAt(i);
			String sKey = oData.GetDescriptor().toLowerCase();
			if (sKey.indexOf("target lambda") > -1) {
				bAlreadyLambdas = true;
				break;
			}
		}

		// We can't tell which species are assigned - so assign to all of 'em
		if (bAlreadyLambdas == false) {
			for (i = 0; i < iNumSpecies; i++) {
				String sSpName = oPop.GetSpeciesNameFromCode(i);
				ModelVector oVector = new ModelVector("Competition Harvest: "
						+ sSpName.replace('_', ' ') + " Target Lambda", "di_compHarv"
						+ sSpName + "NeighborLambda", "di_nlVal", 0, ModelVector.FLOAT,
						true);

				mp_oAllData.add(oVector);
				oBeh.AddRequiredData(oVector);
			}
		} else {
			// We already had lambdas. What we want to do is preserve any for species
			// that didn't change, because they may have values.

			// Create an array of the lambda arrays, placed in the new species
			// order
			ModelData[] p_oNCI = new ModelData[iNumSpecies];
			int iCode;

			for (i = 0; i < iNumSpecies; i++) {
				p_oNCI[i] = null;
			}

			// Get the existing lambdas and put them where they go in our arrays -
			// any for species that were removed will be left behind
			for (i = 0; i < mp_oAllData.size(); i++) {
				ModelData oData = (ModelData) mp_oAllData.elementAt(i);
				String sKey = oData.GetDescriptor().toLowerCase();
				if (sKey.indexOf("target lambda") > -1) {
					// get the species
					String sSpecies;
					sSpecies = sKey.substring(sKey.indexOf(": ") + 2,
							sKey.indexOf(" target")).replace(' ', '_');
					iCode = oPop.GetSpeciesCodeFromName(sSpecies);
					if (iCode > -1) {
						p_oNCI[iCode] = oData;
					}
				}
			}

			// Now remove all lambdas from the required data and from all behaviors
			for (i = 0; i < mp_oAllData.size(); i++) {
				ModelData oData = (ModelData) mp_oAllData.elementAt(i);
				String sKey = oData.GetDescriptor().toLowerCase();
				if (sKey.indexOf("target lambda") > -1) {
					mp_oAllData.remove(i);
					i--;
				}
			}
			for (i = 0; i < mp_oChildBehaviors.length; i++) {
				for (j = 0; j < mp_oChildBehaviors[i].mp_oRequiredData.size(); j++) {
					ModelData oData = (ModelData) mp_oChildBehaviors[i].mp_oRequiredData
							.elementAt(j);
					String sKey = oData.GetDescriptor().toLowerCase();
					if (sKey.indexOf("target lambda") > -1) {
						mp_oChildBehaviors[i].mp_oRequiredData.remove(j);
						j--;
					}
				}
			}

			// Now add back the lambdas, creating new ones for any new species
			for (i = 0; i < p_oNCI.length; i++) {
				if (p_oNCI[i] == null) {
					String sSpName = oPop.GetSpeciesNameFromCode(i);
					p_oNCI[i] = new ModelVector("Competition Harvest: "
							+ sSpName.replace('_', ' ') + " Target Lambda", "di_compHarv"
							+ sSpName + "NeighborLambda", "di_nlVal", 0, ModelVector.FLOAT,
							true);
				}
				mp_oAllData.add(p_oNCI[i]);
				oBeh.AddRequiredData(p_oNCI[i]);
			}
		}

		// Competition harvest - if the cut proportions have not already been set
		// up, set all their values to a default of 1. Otherwise leave them alone.
		if (mp_fCompHarvProportion.GetValue().size() == 0) {
			for (i = 0; i < iNumSpecies; i++) {
				mp_fCompHarvProportion.GetValue().add(new Float(1.0));
			}
		}

	}

	/**
	 * Accepts an XML parent tag (empty, no data) from the parser. This method
	 * watches for the following tags:
	 * <ul>
	 * <li>ha_applyToCell</li>
	 * <li>ha_applyToSpecies</li>
	 * <li>ha_dbhRange</li>
	 * <li>ha_cutEvent</li>
	 * <li>ds_applyToCell</li>
	 * <li>ds_applyToSpecies</li>
	 * <li>ds_dbhRange</li>
	 * <li>ds_deathEvent</li>
	 * <li>SelectionHarvest</li>
	 * <li>sha_CutRange</li>
	 * </ul>
	 * 
	 * @param sXMLTag The XML tag.
	 * @param oAttributes The attributes of this object.
	 * @throws ModelException if there is a problem reading this data.
	 */
	public void ReadXMLParentTag(String sXMLTag, Attributes oAttributes)
			throws ModelException {

		if (sXMLTag.equals("ds_applyToCell")) {

			// Get the X and Y values
			int iX = new Integer(oAttributes.getValue("x")).intValue(), iY = new Integer(
					oAttributes.getValue("y")).intValue();

			// Assign them to the newest mortality episode
			HarvestData oMortEpisode = (HarvestData) mp_oMortEpisodes
					.elementAt(mp_oMortEpisodes.size() - 1);
			oMortEpisode.AddCell(iX, iY, m_oManager.GetPlot());

		} else if (sXMLTag.equals("ds_applyToSpecies")) {

			// Extract species
			TreePopulation oPop = m_oManager.GetTreePopulation();
			int iSpecies = oPop.GetSpeciesCodeFromName(oAttributes
					.getValue("species"));

			// Get the last harvest event
			if (mp_oMortEpisodes.size() == 0) {
				throw (new ModelException(ErrorGUI.BAD_DATA, "JAVA",
						"Mortality episode parameters are incorrectly "
								+ "specified in the parameter file.  Found tag"
								+ " \"ds_applyToSpecies\" outside of "
								+ "disturbance event tag."));
			}
			HarvestData oMortEpisode = (HarvestData) mp_oMortEpisodes
					.elementAt(mp_oMortEpisodes.size() - 1);

			// Add this species to that event
			oMortEpisode.AddSpecies(iSpecies);

		} else if (sXMLTag.equals("ds_dbhRange")) {

			// Reset our placeholders for cut range
			m_iCutRangeAmount = -1;
			m_iCutRangeLowerBound = -1;
			m_iCutRangeUpperBound = -1;
		} else if (sXMLTag.equals("ha_applyToCell")) {

			// Get the X and Y values
			int iX = new Integer(oAttributes.getValue("x")).intValue(), iY = new Integer(
					oAttributes.getValue("y")).intValue();

			// Assign them to the newest harvest event
			HarvestData oHarvest = (HarvestData) mp_oHarvestCuts
					.elementAt(mp_oHarvestCuts.size() - 1);
			oHarvest.AddCell(iX, iY, m_oManager.GetPlot());

		} else if (sXMLTag.equals("ha_applyToSpecies")) {

			// Extract species
			TreePopulation oPop = m_oManager.GetTreePopulation();
			int iSpecies = oPop.GetSpeciesCodeFromName(oAttributes
					.getValue("species"));

			// Get the last harvest event
			HarvestData oHarvest = (HarvestData) mp_oHarvestCuts
					.elementAt(mp_oHarvestCuts.size() - 1);

			// Add this species to that event
			oHarvest.AddSpecies(iSpecies);

		} else if (sXMLTag.equals("ha_dbhRange")) {

			// Reset our placeholders for cut range
			m_iCutRangeAmount = -1;
			m_iCutRangeLowerBound = -1;
			m_iCutRangeUpperBound = -1;
		} else if (sXMLTag.equals("ds_deathEvent")) {

			// Create a new mortality episode
			HarvestData oMortEpisode = new HarvestData();
			mp_oMortEpisodes.add(oMortEpisode);

			// Set it to partial cut - what all mortality episodes get
			oMortEpisode.SetCutType(PARTIAL_CUT);
		} else if (sXMLTag.equals("ha_cutEvent")) {

			// Create a new harvest event
			HarvestData oHarvest = new HarvestData();
			mp_oHarvestCuts.add(oHarvest);
		} else if (sXMLTag.equals("sha_CutRange")) { // LEM 05-12-05
			m_iSelectionHarvestCutRange++;
		} else if (sXMLTag.equals("harvest")) {

			// Enable the harvest behavior
			GetBehaviorByKey("Harvest").m_bIsEnabled = true;
		} else if (sXMLTag.equals("disturbance")) {

			// Enable the episodic mortality behavior
			GetBehaviorByKey("Episodic Mortality").m_bIsEnabled = true;
		} else if (sXMLTag.equals("SelectionHarvest")) { // LEM 05-12-05

			// Enable the harvest behavior and selection harvest behavior
			GetBehaviorByKey("Harvest").m_bIsEnabled = true;
			GetBehaviorByKey("Selection Harvest").m_bIsEnabled = true;
			m_iSelectionHarvestCutRange = 0;
		} else if (sXMLTag.equals("st_stmEvent")) {
			  float fMin = Float.parseFloat((String) oAttributes.getValue("min")),
              fMax = Float.parseFloat((String) oAttributes.getValue("max"));
        int iYear = Integer.parseInt((String) oAttributes.getValue("yr"));
        ScheduledStormInfo oStorm = new ScheduledStormInfo();
        oStorm.fMin = fMin;
        oStorm.fMax = fMax;
        oStorm.iYear = iYear;
      mp_oScheduledStorms.add(oStorm);
		}
		super.ReadXMLParentTag(sXMLTag, oAttributes);
	}

	/**
	 * This method looks for the following tags:
	 * <ul>
	 * <li>ds_timestep
	 * <li>ds_cutAmountType
	 * <li>ds_low
	 * <li>ds_high
	 * <li>ds_amountToCut
	 * <li>ha_timestep
	 * <li>ha_cutType
	 * <li>ha_cutAmountType
	 * <li>ha_low
	 * <li>ha_high
	 * <li>ha_amountToCut
	 * </ul>
	 * 
	 * @param sXMLTag
	 *          XML tag of data object whose value is to be set.
	 * @param sXMLParentTag
	 *          The immediate parent tag that sXMLTag is within.
	 * @param oAttributes
	 *          Attributes of the object. Ignored, but may be needed by overriding
	 *          objects.
	 * @param oData
	 *          Data value appropriate to the data type
	 * @return true if the value was set successfully; false if the value could
	 *         not be found. (This would not be an error, because I need a way to
	 *         cycle through the objects until one of the objects comes up with a
	 *         match.)
	 * @throws ModelException
	 *           if the value could not be assigned to the data object, or if the
	 *           cut type or cut type amount values are unrecognized.
	 */
	public boolean SetSingleValueByXMLTag(String sXMLTag, String sXMLParentTag,
			Attributes oAttributes, Object oData) throws ModelException {
		if (sXMLTag != null) {
			if (sXMLTag.equals("ds_timestep")) {

				// Get the timestep value
				int iTimestep = new Integer(String.valueOf(oData)).intValue();

				// Get the last harvest event and set this as the timestep
				HarvestData oMortEpisode = (HarvestData) mp_oMortEpisodes
						.elementAt(mp_oMortEpisodes.size() - 1);
				oMortEpisode.SetTimestep(iTimestep);

				return true;
			} else if (sXMLTag.equals("ds_cutAmountType")) {

				// Get the cut amount type string
				String sCut = String.valueOf(oData);

				// Get the last harvest event
				HarvestData oMortEpisode = (HarvestData) mp_oMortEpisodes
						.elementAt(mp_oMortEpisodes.size() - 1);

				if (sCut.equalsIgnoreCase("percent of basal area")) {
					oMortEpisode.SetCutAmountType(PERCENTAGE_BASAL_AREA);
				} else if (sCut.equalsIgnoreCase("percent of density")) {
					oMortEpisode.SetCutAmountType(PERCENTAGE_DENSITY);
				} else if (sCut.equalsIgnoreCase("absolute basal area")) {
					oMortEpisode.SetCutAmountType(ABSOLUTE_BASAL_AREA);
				} else if (sCut.equalsIgnoreCase("absolute density")) {
					oMortEpisode.SetCutAmountType(ABSOLUTE_DENSITY);
				} else {
					// Unrecognized - error
					throw (new ModelException(ErrorGUI.BAD_DATA, "JAVA",
							"Episodic mortality event in file has an "
									+ "unrecognized cut amount type - " + sCut + "."));
				}

				return true;
			} else if (sXMLTag.equals("ds_low")) {

				// Extract the value
				float fVal = new Float(String.valueOf(oData)).floatValue();

				// Check to see whether the high and amount values have already been
				// read
				if (m_iCutRangeAmount > -1 && m_iCutRangeUpperBound > -1) {

					// Assign them
					HarvestData oMortEpisode = (HarvestData) mp_oMortEpisodes
							.elementAt(mp_oMortEpisodes.size() - 1);

					oMortEpisode.AddCutRange(fVal, m_iCutRangeUpperBound,
							m_iCutRangeAmount);
					m_iCutRangeAmount = -1;
					m_iCutRangeUpperBound = -1;
					m_iCutRangeLowerBound = -1;
				}

				// Not time to assign - stash this value
				m_iCutRangeLowerBound = fVal;

				return true;
			} else if (sXMLTag.equals("ds_high")) {
				// Extract the value
				float fVal = new Float(String.valueOf(oData)).floatValue();

				// Check to see whether the low and amount values have already been
				// read
				if (m_iCutRangeAmount > -1 && m_iCutRangeLowerBound > -1) {

					// Assign them
					HarvestData oMortEpisode = (HarvestData) mp_oMortEpisodes
							.elementAt(mp_oMortEpisodes.size() - 1);

					oMortEpisode.AddCutRange(m_iCutRangeLowerBound, fVal,
							m_iCutRangeAmount);
					m_iCutRangeAmount = -1;
					m_iCutRangeUpperBound = -1;
					m_iCutRangeLowerBound = -1;
				}

				// Not time to assign - stash this value
				m_iCutRangeUpperBound = fVal;

				return true;
			} else if (sXMLTag.equals("ds_amountToCut")) {
				// Extract the value
				float fVal = new Float(String.valueOf(oData)).floatValue();

				// Check to see whether the low and high bounds have already been
				// read
				if (m_iCutRangeUpperBound > -1 && m_iCutRangeLowerBound > -1) {

					// Assign them
					HarvestData oMortEpisode = (HarvestData) mp_oMortEpisodes
							.elementAt(mp_oMortEpisodes.size() - 1);

					oMortEpisode.AddCutRange(m_iCutRangeLowerBound,
							m_iCutRangeUpperBound, fVal);
					m_iCutRangeAmount = -1;
					m_iCutRangeUpperBound = -1;
					m_iCutRangeLowerBound = -1;
				}

				// Not time to assign - stash this value
				m_iCutRangeAmount = fVal;

				return true;
			} else if (sXMLTag.equals("ha_timestep")) {

				// Get the timestep value
				int iTimestep = new Integer(String.valueOf(oData)).intValue();

				// Get the last harvest event and set this as the timestep
				HarvestData oHarvest = (HarvestData) mp_oHarvestCuts
						.elementAt(mp_oHarvestCuts.size() - 1);
				oHarvest.SetTimestep(iTimestep);

				return true;
			} else if (sXMLTag.equals("ha_cutType")) {

				// Get the cut type string
				String sCut = String.valueOf(oData);

				// Get the last harvest event
				HarvestData oHarvest = (HarvestData) mp_oHarvestCuts
						.elementAt(mp_oHarvestCuts.size() - 1);

				if (sCut.equalsIgnoreCase("gap")) {
					oHarvest.SetCutType(GAP_CUT);
				} else if (sCut.equalsIgnoreCase("partial")) {
					oHarvest.SetCutType(PARTIAL_CUT);
				} else if (sCut.equalsIgnoreCase("clear")) {
					oHarvest.SetCutType(CLEAR_CUT);
				} else {
					// Unrecognized - error
					throw (new ModelException(ErrorGUI.BAD_DATA, "JAVA",
							"Harvest cut in file has an unrecognized " + "cut type - " + sCut
									+ "."));
				}

				return true;
			} else if (sXMLTag.equals("ha_cutAmountType")) {

				// Get the cut amount type string
				String sCut = String.valueOf(oData);

				// Get the last harvest event
				HarvestData oHarvest = (HarvestData) mp_oHarvestCuts
						.elementAt(mp_oHarvestCuts.size() - 1);

				if (sCut.equalsIgnoreCase("percent of basal area")) {
					oHarvest.SetCutAmountType(PERCENTAGE_BASAL_AREA);
				} else if (sCut.equalsIgnoreCase("percent of density")) {
					oHarvest.SetCutAmountType(PERCENTAGE_DENSITY);
				} else if (sCut.equalsIgnoreCase("absolute basal area")) {
					oHarvest.SetCutAmountType(ABSOLUTE_BASAL_AREA);
				} else if (sCut.equalsIgnoreCase("absolute density")) {
					oHarvest.SetCutAmountType(ABSOLUTE_DENSITY);
				} else {
					// Unrecognized - error
					throw (new ModelException(ErrorGUI.BAD_DATA, "JAVA",
							"Harvest cut in file has an unrecognized " + "cut amount type - "
									+ sCut + "."));
				}

				return true;
			} else if (sXMLTag.equals("ha_low")) {

				// Extract the value
				float fVal = new Float(String.valueOf(oData)).floatValue();

				// Check to see whether the high and amount values have already been
				// read
				if (m_iCutRangeAmount > -1 && m_iCutRangeUpperBound > -1) {

					// Assign them
					HarvestData oHarvest = (HarvestData) mp_oHarvestCuts
							.elementAt(mp_oHarvestCuts.size() - 1);

					oHarvest.AddCutRange(fVal, m_iCutRangeUpperBound, m_iCutRangeAmount);
					m_iCutRangeAmount = -1;
					m_iCutRangeUpperBound = -1;
					m_iCutRangeLowerBound = -1;
				}

				// Not time to assign - stash this value
				m_iCutRangeLowerBound = fVal;

				return true;
			} else if (sXMLTag.equals("ha_high")) {
				// Extract the value
				float fVal = new Float(String.valueOf(oData)).floatValue();

				// Check to see whether the low and amount values have already been
				// read
				if (m_iCutRangeAmount > -1 && m_iCutRangeLowerBound > -1) {

					// Assign them
					HarvestData oHarvest = (HarvestData) mp_oHarvestCuts
							.elementAt(mp_oHarvestCuts.size() - 1);

					oHarvest.AddCutRange(m_iCutRangeLowerBound, fVal, m_iCutRangeAmount);
					m_iCutRangeAmount = -1;
					m_iCutRangeUpperBound = -1;
					m_iCutRangeLowerBound = -1;
				}

				// Not time to assign - stash this value
				m_iCutRangeUpperBound = fVal;

				return true;
			} else if (sXMLTag.equals("ha_amountToCut")) {
				// Extract the value
				float fVal = new Float(String.valueOf(oData)).floatValue();

				// Check to see whether the low and high bounds have already been
				// read
				if (m_iCutRangeUpperBound > -1 && m_iCutRangeLowerBound > -1) {

					// Assign them
					HarvestData oHarvest = (HarvestData) mp_oHarvestCuts
							.elementAt(mp_oHarvestCuts.size() - 1);

					oHarvest.AddCutRange(m_iCutRangeLowerBound, m_iCutRangeUpperBound,
							fVal);
					m_iCutRangeAmount = -1;
					m_iCutRangeUpperBound = -1;
					m_iCutRangeLowerBound = -1;
				}

				// Not time to assign - stash this value
				m_iCutRangeAmount = fVal;

				return true;
			} else if (sXMLTag.equals("sha_target_BA")) {
				// Extract the value
				float fVal = new Float(String.valueOf(oData)).floatValue();

				// Assuming we have a valid cut range counter, assign this to the
				// appropriate variable
				if (m_iSelectionHarvestCutRange == 1) {
					m_fCut1BA.SetValue(fVal);
				}
				if (m_iSelectionHarvestCutRange == 2) {
					m_fCut2BA.SetValue(fVal);
				}
				if (m_iSelectionHarvestCutRange == 3) {
					m_fCut3BA.SetValue(fVal);
				}
				if (m_iSelectionHarvestCutRange == 4) {
					m_fCut4BA.SetValue(fVal);

				}
				return true;
			} else if (sXMLTag.equals("sha_loDBH")) {
				// Extract the value
				float fVal = new Float(String.valueOf(oData)).floatValue();

				// Assuming we have a valid cut range counter, assign this to the
				// appropriate variable
				if (m_iSelectionHarvestCutRange == 1) {
					m_fCut1LoDBH.SetValue(fVal);
				}
				if (m_iSelectionHarvestCutRange == 2) {
					m_fCut2LoDBH.SetValue(fVal);
				}
				if (m_iSelectionHarvestCutRange == 3) {
					m_fCut3LoDBH.SetValue(fVal);
				}
				if (m_iSelectionHarvestCutRange == 4) {
					m_fCut4LoDBH.SetValue(fVal);

				}
				return true;
			} else if (sXMLTag.equals("sha_hiDBH")) {
				// Extract the value
				float fVal = new Float(String.valueOf(oData)).floatValue();

				// Assuming we have a valid cut range counter, assign this to the
				// appropriate variable
				if (m_iSelectionHarvestCutRange == 1) {
					m_fCut1HiDBH.SetValue(fVal);
				}
				if (m_iSelectionHarvestCutRange == 2) {
					m_fCut2HiDBH.SetValue(fVal);
				}
				if (m_iSelectionHarvestCutRange == 3) {
					m_fCut3HiDBH.SetValue(fVal);
				}
				if (m_iSelectionHarvestCutRange == 4) {
					m_fCut4HiDBH.SetValue(fVal);

				}
				return true;
			}
		}
		return super.SetSingleValueByXMLTag(sXMLTag, sXMLParentTag, oAttributes,
				oData);

	}

	/**
	 * Checks episodic disturbance settings upon change of species. This will
	 * remove any deleted species.
	 * 
	 * @param iOldNumSpecies says how many species there used to be.
	 * @param p_iIndexer is an array, sized to the new number of species. For
	 * each bucket (representing the index number of a species on the new list),
	 * the value is either the index of that same species in the old species list,
	 *  or -1 if the species is new.
	 * @param p_sNewSpecies
	 *          The new species list.
	 * @throws ModelException
	 *           if anything goes wrong. Not thrown by this function.
	 */
	public void ChangeOfSpecies(int iOldNumSpecies, int[] p_iIndexer,
			String[] p_sNewSpecies) throws ModelException {

		super.ChangeOfSpecies(iOldNumSpecies, p_iIndexer, p_sNewSpecies);

		HarvestData oEpisode;
		int[] p_iOldCutSpecies;
		int i, j, k;

		// Go through all the harvests and check for species that have been removed
		for (i = 0; i < mp_oHarvestCuts.size(); i++) {
			oEpisode = (HarvestData) mp_oHarvestCuts.elementAt(i);
			// Make a list of the species for this cut
			p_iOldCutSpecies = new int[oEpisode.GetNumberOfSpecies()];
			for (j = 0; j < p_iOldCutSpecies.length; j++) {
				p_iOldCutSpecies[j] = oEpisode.GetSpecies(j);
			}
			// Now remove all the old species
			oEpisode.ClearSpecies();

			// Now add back the species from the copy we saved
			for (j = 0; j < p_iOldCutSpecies.length; j++) {

				// Look for this species in the new species list - add the new species
				// number if we find it
				for (k = 0; k < p_iIndexer.length; k++) {
					if (p_iIndexer[k] == p_iOldCutSpecies[j]) {
						// Yep, this species is still on the list - change the species
						// number
						oEpisode.AddSpecies(k);
						break;
					}
				}
			}

			// If this harvest event has no species left, delete it
			if (oEpisode.GetNumberOfSpecies() == 0) {
				mp_oHarvestCuts.remove(i);
				i--;
			}
		}

		// Repeat with episodic mortality
		for (i = 0; i < mp_oMortEpisodes.size(); i++) {
			oEpisode = (HarvestData) mp_oMortEpisodes.elementAt(i);
			// Make a list of the species for this cut
			p_iOldCutSpecies = new int[oEpisode.GetNumberOfSpecies()];
			for (j = 0; j < p_iOldCutSpecies.length; j++) {
				p_iOldCutSpecies[j] = oEpisode.GetSpecies(j);
			}
			// Now remove all the old species
			oEpisode.ClearSpecies();

			// Now add back the species from the copy we saved
			for (j = 0; j < p_iOldCutSpecies.length; j++) {

				// Look for this species in the new species list - add the new species
				// number if we find it
				for (k = 0; k < p_iIndexer.length; k++) {
					if (p_iIndexer[k] == p_iOldCutSpecies[j]) {
						// Yep, this species is still on the list - change the species
						// number
						oEpisode.AddSpecies(k);
						break;
					}
				}
			}

			// If this episodic event has no species left, delete it
			if (oEpisode.GetNumberOfSpecies() == 0) {
				mp_oMortEpisodes.remove(i);
				i--;
			}
		}
	}

	/**
	 * Overridden for backwards compatibility after changing XML tag names. This
	 * was done in version 6.06 and can be removed after one version, since it was
	 * done to a behavior not widely in production at the time of release.
	 * 
	 * @param sXMLTag
	 *          Parent XML tag of data vector whose value is to be set.
	 * @param sXMLParentTag
	 *          The immediate parent tag that sXMLTag is within.
	 * @param p_oData
	 *          Vector of data values appropriate to the data type
	 * @param p_sChildXMLTags
	 *          The XML tags of the child elements
	 * @param p_bAppliesTo
	 *          Array of booleans saying which of the vector values should be set.
	 *          This is important in the case of species-specifics - the vector
	 *          index is the species number but not all species are set.
	 * @param oParentAttributes
	 *          Attributes of parent tag. May be useful when overridding this for
	 *          unusual tags.
	 * @param p_oAttributes
	 *          Attributes passed from parser. This may be needed when overriding
	 *          this function. Basic species-specific values are already handled
	 *          by this function.
	 * @return true if the value was set successfully; false if the value could
	 *         not be found. (This would not be an error, because I need a way to
	 *         cycle through the objects until one of the objects comes up with a
	 *         match.) If a match to a data object is made via XML tag, but the
	 *         found object is not a ModelVector, this returns false.
	 * @throws ModelException
	 *           if the value could not be assigned to the data object.
	 */
	public boolean SetVectorValueByXMLTag(String sXMLTag, String sXMLParentTag,
			java.util.Vector p_oData, String[] p_sChildXMLTags,
			boolean[] p_bAppliesTo, org.xml.sax.Attributes oParentAttributes,
			org.xml.sax.Attributes[] p_oAttributes) throws ModelException {

		if (sXMLTag.equals("st_stmPropDieMedDmg")) {
			int i;
			// Old proportion of medium damaged trees that die - transform
			// into equivalent parameters for new logit function
			sXMLTag = "st_stmMedDmgSurvProbA";
			for (i = 0; i < p_sChildXMLTags.length; i++) {
				if (p_sChildXMLTags[i].equals("st_spdmdVal")) {
					p_sChildXMLTags[i] = "st_smdspaVal";
				}
			}

			// Transform proportion values into "a" values - these will be set
			// at the end of this function
			Float fA;
			for (i = 0; i < p_oData.size(); i++) {
				fA = new Float(p_oData.elementAt(i).toString());
				if (fA.floatValue() == 0)
					fA = new Float(10);
				else
					fA = new Float(java.lang.Math
							.log(1 / ((1 / (1 - fA.floatValue())) - 1)));
				p_oData.remove(i);
				p_oData.add(i, fA);
			}

			// Now set "b" values to 0
			Float[] p_fB = new Float[p_oData.size()];
			for (i = 0; i < p_fB.length; i++)
				p_fB[i] = new Float(0);
			SetVectorValues(mp_fStmMedDmgSurvProbB, p_fB, p_bAppliesTo);

			// Enable the storm killer behavior and give it the same
			// species/type combos as the storm damage applier
			Behavior oStormKiller = GetBehaviorByKey("Storm Killer"), oDamageApplier = GetBehaviorByKey("Storm Damage Applier");
			SpeciesTypeCombo oCombo;
			oStormKiller.m_bIsEnabled = true;
			oStormKiller.ClearSpeciesTypeCombos();
			for (i = 0; i < oDamageApplier.GetNumberOfCombos(); i++) {
				oCombo = (SpeciesTypeCombo) oDamageApplier.GetSpeciesTypeCombo(i)
						.clone();
				oStormKiller.AddSpeciesTypeCombo(oCombo);
			}
		} else if (sXMLTag.equals("st_stmPropDieFullDmg")) {
			int i;
			// Old proportion of heavily damaged trees that die - transform
			// into equivalent parameters for new logit function
			sXMLTag = "st_stmFullDmgSurvProbA";
			for (i = 0; i < p_sChildXMLTags.length; i++) {
				if (p_sChildXMLTags[i].equals("st_spdfdVal")) {
					p_sChildXMLTags[i] = "st_sfdspaVal";
				}
			}

			// Transform proportion values into "a" values - these will be set
			// at the end of this function
			Float fA;
			for (i = 0; i < p_oData.size(); i++) {
				fA = new Float(p_oData.elementAt(i).toString());
				if (fA.floatValue() == 0)
					fA = new Float(10);
				else
					fA = new Float(java.lang.Math
							.log(1 / ((1 / (1 - fA.floatValue())) - 1)));
				p_oData.remove(i);
				p_oData.add(i, fA);
			}

			// Now set "b" values to 0
			Float[] p_fB = new Float[p_oData.size()];
			for (i = 0; i < p_fB.length; i++)
				p_fB[i] = new Float(0);
			SetVectorValues(mp_fStmFullDmgSurvProbB, p_fB, p_bAppliesTo);

			// The storm killer behavior should be enabled when the medium damage
			// tag comes through
		}

		return super.SetVectorValueByXMLTag(sXMLTag, sXMLParentTag, p_oData,
				p_sChildXMLTags, p_bAppliesTo, oParentAttributes, p_oAttributes);
	}

	/**
	 * If the harvest interface behavior is being used, this will check to see if
	 * it needs to add new data members to the name. After changing the name and
	 * writing the behaviors, the name will be changed back.
	 * 
	 * @param out
	 *          BufferedWriter File to write to.
	 * @param oPop
	 *          TreePopulation Tree population.
	 * @throws ModelException
	 *           Passing through from called functions.
	 */
	void WriteBehaviorNodes(BufferedWriter out, TreePopulation oPop)
			throws ModelException {
		Behavior oBeh = GetBehaviorByKey("Harvest Interface");
		boolean bDoIt = oBeh.m_bIsEnabled
				&& mp_sHarvIntNewTreeDataMembers.size() > 0;
		String sXMLName = "";
		if (bDoIt) {
			sXMLName = oBeh.m_sParFileTag;
			for (int i = 0; i < mp_sHarvIntNewTreeDataMembers.size(); i++) {
				oBeh.m_sParFileTag += " ("
						+ (String) mp_sHarvIntNewTreeDataMembers.elementAt(i) + ")";
			}
		}

		super.WriteBehaviorNodes(out, oPop);

		if (bDoIt)
			oBeh.m_sParFileTag = sXMLName;
	}

	/**
	 * Overwritten to handle the potentially-funky harvest interface behavior.
	 * 
	 * @param sXMLTag
	 *          String The XML tag for which to find a behavior.
	 * @return Behavior Behavior for the XML tag, or NULL if none of the behaviors
	 *         has that tag.
	 */
	public Behavior GetBehaviorByXMLTag(String sXMLTag) {
		if (sXMLTag.startsWith("Harvest Interface")) {
			mp_sHarvIntNewTreeDataMembers.clear();
			String sDataMember;
			int iStart = sXMLTag.indexOf('('), iEnd;
			if (iStart >= 0) {
				while (iStart >= 0) {
					iEnd = sXMLTag.indexOf(')', iStart);
					if (iEnd < 0) {
						// fail quietly - we don't have a choice
						break;
					}
					iStart++;
					sDataMember = sXMLTag.substring(iStart, iEnd);
					mp_sHarvIntNewTreeDataMembers.add(sDataMember);
					iStart = sXMLTag.indexOf('(', iEnd);
				}
				sXMLTag = "Harvest Interface";
			}
		}
		return super.GetBehaviorByXMLTag(sXMLTag);
	}

	/**
	 * Updates parameters when species are copied. This updates harvest and
	 * mortality episode settings, and competition harvest lambdas.
	 * 
	 * @param iSpeciesCopyFrom
	 *          int Species to copy.
	 * @param iSpeciesCopyTo
	 *          int Species that is the copy.
	 * @throws ModelException
	 *           if there is a problem.
	 */
	public void CopySpecies(int iSpeciesCopyFrom, int iSpeciesCopyTo)
			throws ModelException {
		super.CopySpecies(iSpeciesCopyFrom, iSpeciesCopyTo);

		TreePopulation oPop = m_oManager.GetTreePopulation();
		ModelVector oCopyFrom = null, oCopyTo = null;
		ModelData oData;
		String sCopyFrom, sCopyTo;
		Float f1, f2;
		HarvestData oEpisode;
		int[] p_iOldCutSpecies;
		int i, j;
		
    //********************************
		// Harvest and mortality episode settings
		// ********************************

		// Go through all the harvests and remove the species that is becoming the
		// copy
		for (i = 0; i < mp_oHarvestCuts.size(); i++) {
			oEpisode = (HarvestData) mp_oHarvestCuts.elementAt(i);
			// Make a list of the species for this cut
			p_iOldCutSpecies = new int[oEpisode.GetNumberOfSpecies()];
			for (j = 0; j < p_iOldCutSpecies.length; j++) {
				p_iOldCutSpecies[j] = oEpisode.GetSpecies(j);
			}
			// Remove all species
			oEpisode.ClearSpecies();

			// Add back the non-copied species - and add the copied species if
			// present
			for (j = 0; j < p_iOldCutSpecies.length; j++) {
				if (iSpeciesCopyTo != p_iOldCutSpecies[j]) {
					oEpisode.AddSpecies(p_iOldCutSpecies[j]);
				}
				if (iSpeciesCopyFrom == p_iOldCutSpecies[j]) {
					oEpisode.AddSpecies(iSpeciesCopyTo);
				}
			}

			// If this harvest event has no species left, delete it
			if (oEpisode.GetNumberOfSpecies() == 0) {
				mp_oHarvestCuts.remove(i);
				i--;
			}
		}

		// Repeat with episodic mortality
		for (i = 0; i < mp_oMortEpisodes.size(); i++) {
			oEpisode = (HarvestData) mp_oMortEpisodes.elementAt(i);
			// Make a list of the species for this cut
			p_iOldCutSpecies = new int[oEpisode.GetNumberOfSpecies()];
			for (j = 0; j < p_iOldCutSpecies.length; j++) {
				p_iOldCutSpecies[j] = oEpisode.GetSpecies(j);
			}
			// Remove all species
			oEpisode.ClearSpecies();

			// Add back the non-copied species - and add the copied species if
			// present
			for (j = 0; j < p_iOldCutSpecies.length; j++) {
				if (iSpeciesCopyTo != p_iOldCutSpecies[j]) {
					oEpisode.AddSpecies(p_iOldCutSpecies[j]);
				}
				if (iSpeciesCopyFrom == p_iOldCutSpecies[j]) {
					oEpisode.AddSpecies(iSpeciesCopyTo);
				}
			}

			// If this harvest event has no species left, delete it
			if (oEpisode.GetNumberOfSpecies() == 0) {
				mp_oHarvestCuts.remove(i);
				i--;
			}
		}

		// ********************************
		// Competition Harvest lambdas
		// ********************************
		sCopyFrom = oPop.GetSpeciesNameFromCode(iSpeciesCopyFrom).replace('_', ' ');
		sCopyTo = oPop.GetSpeciesNameFromCode(iSpeciesCopyTo).replace('_', ' ');

		// Find the lambdas in our list
		for (i = 0; i < mp_oAllData.size(); i++) {
			oData = (ModelData) mp_oAllData.elementAt(i);
			String sKey = oData.GetDescriptor();
			if (sKey.indexOf(sCopyFrom) > -1) {
				oCopyFrom = (ModelVector) oData;
			}
			if (sKey.indexOf(sCopyTo) > -1) {
				oCopyTo = (ModelVector) oData;
			}
		}

		if (null == oCopyFrom || null == oCopyTo) {
			throw (new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA",
					"Disturbance could not find competition harvest lambdas for neighbor"
							+ " species when copying species."));
		}
		
		if (oCopyTo.GetValue().size() == 0) {
			for (i = 0; i < oCopyFrom.GetValue().size(); i++) {
				f1 = (Float) oCopyFrom.GetValue().elementAt(i);
				f2 = new Float(f1.floatValue());
				oCopyTo.GetValue().add(i, f2);
			}	
		} else {
		  for (i = 0; i < oCopyFrom.GetValue().size(); i++) {
			  f1 = (Float) oCopyFrom.GetValue().elementAt(i);
			  if (null == f1)
				  f2 = null;
			  else
				  f2 = new Float(f1.floatValue());
			  oCopyTo.GetValue().remove(i);
			  oCopyTo.GetValue().add(i, f2);
		  }
		}
	}

	/**
	 * Removes cells from harvest and disturbance events that are outside the
	 * plot.
	 * 
	 * @param fOldX
	 *          float Old plot X length.
	 * @param fOldY
	 *          float Old plot Y length.
	 * @param fNewX
	 *          float New plot X length.
	 * @param fNewY
	 *          float New plot Y length.
	 * @throws ModelException
	 *           if anything goes wrong.
	 */
	public void ChangeOfPlotResolution(float fOldX, float fOldY, float fNewX,
			float fNewY) throws ModelException {

		// If the plot has shrunk, check for cells now outside the plot
		if (fOldX > fNewX || fOldY > fNewY) {
			HarvestData oEpisode;
			Cell oCell;
			double fCellLength = 8.0;
			int iNewMaxX = (int) java.lang.Math.ceil(fNewX / fCellLength), iNewMaxY = (int) java.lang.Math
					.ceil(fNewY / fCellLength), i, j;

			for (i = 0; i < mp_oHarvestCuts.size(); i++) {
				oEpisode = (HarvestData) mp_oHarvestCuts.elementAt(i);
				for (j = 0; j < oEpisode.GetNumberOfCells(); j++) {
					oCell = (Cell) oEpisode.GetCell(j);
					if (oCell.GetX() >= iNewMaxX || oCell.GetY() >= iNewMaxY) {
						oEpisode.RemoveCell(j);
						j--;
					}
				}
				// If there's no cells left remove this harvest episode
				if (oEpisode.GetNumberOfCells() == 0) {
					mp_oHarvestCuts.remove(i);
					i--;
				}
			}

			// Repeat with mortality episodes
			for (i = 0; i < mp_oMortEpisodes.size(); i++) {
				oEpisode = (HarvestData) mp_oMortEpisodes.elementAt(i);
				for (j = 0; j < oEpisode.GetNumberOfCells(); j++) {
					oCell = (Cell) oEpisode.GetCell(j);
					if (oCell.GetX() >= iNewMaxX || oCell.GetY() >= iNewMaxY) {
						oEpisode.RemoveCell(j);
						j--;
					}
				}
				// If there's no cells left remove this harvest episode
				if (oEpisode.GetNumberOfCells() == 0) {
					mp_oMortEpisodes.remove(i);
					i--;
				}
			}
		}
	}

	/**
	 * Overridden to remove the competition harvest lambdas that aren't necessary,
	 * but put them back afterwards.
	 * 
	 * @param oPop
	 *          TreePopulation object.
	 * @return Vector from the base class.
	 */
	public Vector FormatDataForDisplay(TreePopulation oPop) {
		Behavior oBeh = GetBehaviorByKey("Competition Harvest");
		Vector p_oExtraLambdas = null;
		if (oBeh.m_bIsEnabled) {
			p_oExtraLambdas = new Vector(0); // lambdas not to display
			boolean[] p_bAppliesTo = oBeh.GetWhichSpeciesUsed(oPop);
			int i, j;
			// Remove all lambdas for species not being used
			for (i = 0; i < mp_oAllData.size(); i++) {
				ModelData oData = (ModelData) mp_oAllData.elementAt(i);
				String sKey = oData.GetDescriptor().toLowerCase();
				String sSpecies;
				int iCode;
				if (sKey.indexOf("target lambda") > -1) {
					sSpecies = sKey.substring(sKey.indexOf(": ") + 2,
							sKey.indexOf(" target")).replace(' ', '_');
					iCode = oPop.GetSpeciesCodeFromName(sSpecies);
					if (iCode > -1 && false == p_bAppliesTo[iCode]) {
						p_oExtraLambdas.add(oData);
						mp_oAllData.remove(i);
						i--;

						for (j = 0; j < oBeh.mp_oRequiredData.size(); j++) {
							oData = (ModelData) oBeh.mp_oRequiredData.elementAt(j);
							if (oData.GetDescriptor().toLowerCase().equals(sKey)) {
								oBeh.mp_oRequiredData.remove(j);
								break;
							}
						}
					}
				}
			}
		}
		Vector oReturn = super.FormatDataForDisplay(oPop);

		if (oBeh.m_bIsEnabled && p_oExtraLambdas != null
				&& p_oExtraLambdas.size() > 0) {
			// Put the lambdas back
			ModelVector oData;
			int i;
			for (i = 0; i < p_oExtraLambdas.size(); i++) {
				oData = (ModelVector) p_oExtraLambdas.elementAt(i);
				mp_oAllData.add(oData);
				oBeh.mp_oRequiredData.add(oData);
			}
		}

		return oReturn;
	}
}
