package javawrapper;

import javax.swing.*;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Window for planting editing.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */

public class PlantEdit
    extends EpisodicEventsEdit
    implements ActionListener {

  /**List of text fields, one for each species, for which species to apply
   * the planting to*/
  protected JTextField[] mp_jSpeciesAmt;

  /**Edit box where timestep is entered*/
  protected JTextField m_jTimestepEdit = new JTextField("       ");

  /**Radio button for gridded planting*/
  protected JRadioButton m_jGriddedPlantingButton = new JRadioButton("Gridded");

  /**Radio button for random planting*/
  protected JRadioButton m_jRandomPlantingButton = new JRadioButton("Random");

  /**Label that displays the plant amount value for the currently displayed planting*/
  protected JLabel m_jPlantAmountValueLabel = new JLabel();

  /**Edit box where amount to plant or plant spacing is entered*/
  protected JTextField m_jPlantAmtEdit = new JTextField("       ");

  /**This is where our new planting event goes.  In the case of an edited
   * event, this will make sure that it is inserted where it used to be.*/
  protected int m_iPlantingIndex = -1;

  /**Help ID string*/
  private String m_sHelpID = "windows.edit_planting_window";

  /**
   * Constructor.  Creates the window.
   * @param oWindow Object to exchange planting data with.
   * @throws ModelException If the window cannot be set up correctly.
   */
  public PlantEdit(EpisodicEventsWindow oWindow) throws ModelException {
    super(oWindow, "Edit Planting");

    //Get the cell numbers and lengths
    m_iNumXCells = m_oEpisodicEventsWindow.m_iNumPlantXCells;
    m_iNumYCells = m_oEpisodicEventsWindow.m_iNumPlantYCells;
    m_fLengthXCells = (float) m_oEpisodicEventsWindow.m_iPlotLengthX /
        (float) m_iNumXCells;
    m_fLengthYCells = (float) m_oEpisodicEventsWindow.m_iPlotLengthY /
        (float) m_iNumYCells;
    m_jColor = EpisodicEventsWindow.PLANT_COLOR;

    MakeGUI();
    SetUpCharting();
    RefreshChart();

    //Help ID
    m_oEpisodicEventsWindow.GetHelpBroker().enableHelpKey(this.getRootPane(),
        m_sHelpID, null);
  }

  /**
   * Constructor that displays a planting event to edit.
   * @param oWindow Object to exchange planting data with.
   * @param oToDisplay Planting event to display for editing.
   * @throws ModelException If the window cannot be set up correctly.
   */
  public PlantEdit(EpisodicEventsWindow oWindow, PlantingData oToDisplay) throws
      ModelException {
    this(oWindow);

    //Get the index of this planting
    m_iPlantingIndex = oWindow.mp_oPlantingData.indexOf(oToDisplay);
    int iTemp, i;

    //Set timestep
    m_jTimestepEdit.setText(String.valueOf(oToDisplay.GetTimestep()) + "     ");

    //Set the planting space type and amount
    iTemp = oToDisplay.GetPlantSpacing();
    if (PlantingBehaviors.GRIDDED == iTemp) {
      m_jGriddedPlantingButton.setSelected(true);
      m_jPlantAmountValueLabel.setText("Spacing (m):");
      m_jPlantAmtEdit.setText(String.valueOf(oToDisplay.GetSpacingDistance()) +
                              "     ");
    }
    else if (PlantingBehaviors.RANDOM == iTemp) {
      m_jRandomPlantingButton.setSelected(true);
      m_jPlantAmtEdit.setText(String.valueOf(oToDisplay.GetDensity()));
      m_jPlantAmountValueLabel.setText("# total trees/ha:" + "     ");
    }

    //Set the species
    iTemp = oToDisplay.GetNumberOfSpecies();
    for (i = 0; i < iTemp; i++) {
      mp_jSpeciesAmt[oToDisplay.GetSpecies(i)].setText(String.valueOf(
          oToDisplay.GetAbundance(oToDisplay.GetSpecies(i))));
    }

    //Set the cells
    iTemp = oToDisplay.GetNumberOfCells();
    for (i = 0; i < iTemp; i++) {
      Cell oCell = oToDisplay.GetCell(i);
      m_oDataset.mp_bData[1][oCell.GetX()][oCell.GetY()] = true;
    }

    //Refresh the display
    RefreshChart();
  }

  /**
   * Makes the GUI.  This draws all the window components and places them
   * appropriately.
   */
  protected void MakeGUI() {
    int i;
    TreePopulation oPop = m_oEpisodicEventsWindow.GetTreePopulation();

    //Set fonts
    mp_jSpeciesAmt = new JTextField[m_iNumSpecies];
    m_jTimestepEdit.setFont(new ModelFont());
    m_jGriddedPlantingButton.setFont(new ModelFont());
    m_jRandomPlantingButton.setFont(new ModelFont());
    m_jPlantAmountValueLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    m_jPlantAmtEdit.setFont(new ModelFont());

    //*************************
     // Choices panel
     //*************************
    JPanel jDataPanel = new JPanel();
    jDataPanel.setLayout(new BoxLayout(jDataPanel, BoxLayout.PAGE_AXIS));
    jDataPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

    //Timestep
    JLabel jTimestepLabel = new JLabel("Timestep:");
    //jTimestepLabel.setHorizontalAlignment(SwingConstants.LEFT);
    jTimestepLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    FlowLayout jFlowLayout = new FlowLayout();
    jFlowLayout.setAlignment(FlowLayout.LEFT);
    JPanel jTimestepPanel = new JPanel(jFlowLayout);
    jTimestepPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jTimestepPanel.setBorder(BorderFactory.createMatteBorder(0, 0, 1, 0,
        java.awt.Color.black));
    jTimestepPanel.add(jTimestepLabel);
    jTimestepPanel.add(m_jTimestepEdit);
    jDataPanel.add(jTimestepPanel);

    //Species
    JLabel jSpeciesLabel = new JLabel("Percentage of each species:");
    jSpeciesLabel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jSpeciesLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    jDataPanel.add(jSpeciesLabel);

    for (i = 0; i < m_iNumSpecies; i++) {
      JPanel jMiniPanel = new JPanel(new BorderLayout());
      jMiniPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
      JLabel jTemp = new JLabel(oPop.GetSpeciesNameFromCode(i).replace(
          '_', ' '));
      jTemp.setFont(new ModelFont());
      mp_jSpeciesAmt[i] = new JTextField("       ");
      mp_jSpeciesAmt[i].setFont(new ModelFont());
      jMiniPanel.add(jTemp, BorderLayout.CENTER);
      jMiniPanel.add(mp_jSpeciesAmt[i], BorderLayout.EAST);
      jDataPanel.add(jMiniPanel);
      if (i == m_iNumSpecies - 1) {
        jMiniPanel.setBorder(BorderFactory.createMatteBorder(0, 0, 1, 0,
            java.awt.Color.black));
      }
    }

    //Plant spacing
    JPanel jPlantSpacingPanel = new JPanel();
    jPlantSpacingPanel.setLayout(new GridLayout(0, 1));
    JLabel jSpacingLabel = new JLabel("Plant spacing:");
    jSpacingLabel.setFont(new ModelFont(java.awt.Font.BOLD));
    jPlantSpacingPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jPlantSpacingPanel.add(jSpacingLabel);
    //Group our radio buttons and add action listeners for each
    m_jGriddedPlantingButton.setActionCommand("Chose gridded");
    m_jGriddedPlantingButton.addActionListener(this);

    m_jRandomPlantingButton.setActionCommand("Chose random");
    m_jRandomPlantingButton.addActionListener(this);

    ButtonGroup jPlantGroup = new ButtonGroup();
    jPlantGroup.add(m_jGriddedPlantingButton);
    jPlantGroup.add(m_jRandomPlantingButton);
    jPlantSpacingPanel.add(m_jGriddedPlantingButton);
    jPlantSpacingPanel.add(m_jRandomPlantingButton);
    jDataPanel.add(jPlantSpacingPanel);

    //Plant spacing/density
    JPanel jSpacingDensityPanel = new JPanel();
    jFlowLayout = null;
    jFlowLayout = new FlowLayout();
    jFlowLayout.setAlignment(FlowLayout.LEFT);
    jSpacingDensityPanel.setLayout(jFlowLayout);
    jSpacingDensityPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jSpacingDensityPanel.add(m_jPlantAmountValueLabel);
    jSpacingDensityPanel.add(m_jPlantAmtEdit);
    jDataPanel.add(jSpacingDensityPanel);

    //**********************************
     //Chart panel
     //**********************************
    JLabel jInstructionLabel = new JLabel("Click on a map cell to select it for this planting.  Click on a selected cell to de-select it.");
    jInstructionLabel.setFont(new ModelFont());
    jInstructionLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
    m_jChartPanel.setLayout(new BorderLayout());
    m_jChartPanel.add(jInstructionLabel, BorderLayout.NORTH);
    m_jChartPanel.setBorder(BorderFactory.createMatteBorder(0, 1, 0, 0,
        java.awt.Color.black));

    //**********************************
     // Legend panel
     //**********************************
    JPanel jLegendPanel = MakeTreeLegendPanel(oPop);

    //Put it all together
    /*getContentPane().setLayout(new BorderLayout());
         getContentPane().add(jDataPanel, BorderLayout.WEST);
         getContentPane().add(m_jChartPanel, BorderLayout.CENTER);
         getContentPane().add(jLegendPanel, BorderLayout.EAST);*/

    JPanel jContentPanel = new JPanel(new BorderLayout());

    jContentPanel.add(jDataPanel, BorderLayout.WEST);
    jContentPanel.add(m_jChartPanel, BorderLayout.CENTER);
    jContentPanel.add(jLegendPanel, BorderLayout.EAST);

    getContentPane().setLayout(new BorderLayout());
    getContentPane().add(new JScrollPane(jContentPanel), BorderLayout.CENTER);
    getContentPane().add(new OKCancelButtonPanel(this,
                                                 m_oEpisodicEventsWindow.
                                                 GetHelpBroker(), m_sHelpID),
                         BorderLayout.SOUTH);

    m_jRandomPlantingButton.setSelected(true);
    m_jPlantAmountValueLabel.setText("# total trees/ha:");
  }

  /**
   * Puts the new planting into DisturbanceWindow.
   * @throws ModelException if any of the values cannot be translated into
   * numbers, or if the new PlantingData object throws an error during its
   * validation.
   */
  protected void AddNewPlanting() throws ModelException {

    //Create a new PlantingData object
    PlantingData oNewPlant = new PlantingData();
    float fTemp;
    int iTemp, i, j;

    //Set the timestep, making sure it's formated correctly
    String sTemp = m_jTimestepEdit.getText().trim();
    if (sTemp.length() == 0) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "Invalid timestep."));
    }
    try {
      iTemp = new Integer(sTemp).intValue();
    }
    catch (java.lang.NumberFormatException oEx) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "Invalid timestep."));
    }
    oNewPlant.SetTimestep(iTemp);

    try {
      //Set the species
      for (i = 0; i < mp_jSpeciesAmt.length; i++) {
        sTemp = mp_jSpeciesAmt[i].getText();
        if (sTemp != null) {
          sTemp = sTemp.trim();
          if (sTemp.length() > 0) {
            fTemp = new Float(sTemp).floatValue();
            if (fTemp > 0) {
              oNewPlant.AddSpecies(i);
              oNewPlant.AddAbundance(i, fTemp);
            }
          }
        }
      }
    }
    catch (java.lang.NumberFormatException oEx) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                               "Invalid species amount."));
    }

    //Set the plant spacing type and amount
    try {
      fTemp = -1;
      sTemp = m_jPlantAmtEdit.getText();
      if (sTemp != null) {
        sTemp = sTemp.trim();
        if (sTemp.length() > 0) {
          fTemp = new Float(sTemp).floatValue();
        }
      }
    }
    catch (java.lang.NumberFormatException oEx) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                               "Invalid plant spacing or density."));
    }

    if (m_jRandomPlantingButton.isSelected()) {
      oNewPlant.SetPlantSpacing(PlantingBehaviors.RANDOM);
      oNewPlant.SetSpacingDistance(fTemp);
    }
    else if (m_jGriddedPlantingButton.isSelected()) {
      oNewPlant.SetPlantSpacing(PlantingBehaviors.GRIDDED);
      oNewPlant.SetDensity(fTemp);
    }

    //Grid cells
    Plot oPlot = m_oEpisodicEventsWindow.m_oDisturbanceBehaviors.m_oManager.
        GetPlot();
    for (i = 0; i < m_oDataset.mp_bData[1].length; i++) {
      for (j = 0; j < m_oDataset.mp_bData[1][i].length; j++) {
        if (m_oDataset.mp_bData[1][i][j]) {
          oNewPlant.AddCell(i, j, oPlot);
        }
      }
    }

    oNewPlant.ValidatePlant(m_oEpisodicEventsWindow.m_oDisturbanceBehaviors.
                            m_oManager.GetTreePopulation(),
                            m_oEpisodicEventsWindow.m_oDisturbanceBehaviors.
                            m_oManager.GetPlot());

    //Add this new planting to the list in DisturbanceWindow
    if (m_iPlantingIndex == -1) {
      m_oEpisodicEventsWindow.mp_oPlantingData.add(oNewPlant);
    }
    else {
      //This was the edit of an existing planting
      m_oEpisodicEventsWindow.mp_oPlantingData.remove(m_iPlantingIndex);
      m_oEpisodicEventsWindow.mp_oPlantingData.add(m_iPlantingIndex, oNewPlant);
      //Cause this planting to be displayed
      m_oEpisodicEventsWindow.DisplayPlanting(oNewPlant);
    }

    //Update the number of planting events
    m_oEpisodicEventsWindow.m_jNumPlantingEvents.setText(String.valueOf(
        m_oEpisodicEventsWindow.mp_oPlantingData.size()));
  }

  /**
   * Responds to window events.
   * @param oEvent Event triggering this call.
   */
  public void actionPerformed(ActionEvent oEvent) {
    if (oEvent.getActionCommand().equals("OK")) {
      try {
        AddNewPlanting();
        this.setVisible(false);
        this.dispose();
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }
    else if (oEvent.getActionCommand().equals("Cancel")) {
      //Close this window
      this.setVisible(false);
      this.dispose();
    }
    else if (oEvent.getActionCommand().equals("Chose random")) {

      //Set the text of the amount button
      m_jPlantAmountValueLabel.setText("# total trees/ha:");
    }
    else if (oEvent.getActionCommand().equals("Chose gridded")) {

      //Set the text of the amount button
      m_jPlantAmountValueLabel.setText("Spacing (m):");
    }
  }

}
