package javawrapper;

import javax.swing.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.help.HelpBroker;
import java.awt.BorderLayout;

/**
 * Dialog allowing the user to set up basic tree information.  This includes
 * the species list and initial density size classes.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 * <br>June 21, 2004:  Added support to write tree maps as tab-delimited
 *                     text (LEM)
 * <br>November 3, 2005:  Added a "Rename" button for species (LEM)
 * <br>January 11, 2007: Added a "Copy" button for species (LEM)
 */

public class TreeSetup
    extends JDialog
    implements ActionListener {

  /**TreePopulation object that data changes will be communicated to*/
  protected TreePopulation m_oPop;

  protected DefaultListModel
      /**List model for species*/
      m_jSpeciesListModel,
      /**List model for size classes*/
      m_jSizeClassListModel;

  /**List displaying the species*/
  protected JList m_jSpeciesList;

  /**Vector of RenamePair objects, one for each species the user has asked
   * to rename*/
  protected java.util.Vector mp_oRenameSpecies = new java.util.Vector(0);

  /**Vector of RenamePair objects, one for each species the user has asked
   * to copy*/
  protected java.util.Vector mp_oCopySpecies = new java.util.Vector(0);

  /**Field for entering a new species name*/
  protected JTextField m_jNewSpecies;

  /**Button for clearing tree map trees*/
  protected JButton m_jClearTreeMap;

  /**Button for writing a tree map as tab-delimited text*/
  protected JButton m_jWriteTreeMap;

  /**Help ID string*/
  private String m_sHelpID = "windows.tree_setup_window";

  /**
   * Constructor.  Constructs and displays the GUI.
   * @param jParent Parent window in which to display this dialog.
   * @param oPop TreePopulation object.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public TreeSetup(JFrame jParent, TreePopulation oPop) {
    super(jParent, "Tree Setup", true);

    //Help ID
    oPop.m_oManager.GetHelpBroker().enableHelpKey(this.getRootPane(), m_sHelpID, null);

    m_oPop = oPop;
    int i;

    //Create the list model and load with any existing species
    m_jSpeciesListModel = new DefaultListModel();
    for (i = 0; i < m_oPop.GetNumberOfSpecies(); i++) {
      m_jSpeciesListModel.addElement(m_oPop.GetSpeciesNameFromCode(i).replace(
          '_', ' '));
    }

    //List of species
    m_jSpeciesList = new JList(m_jSpeciesListModel);
    m_jSpeciesList.setFont(new ModelFont());
    JScrollPane jScroller = new JScrollPane(m_jSpeciesList);

    //Panel containing controls for adding new species
    JPanel jButtonPanel1 = new JPanel();
    jButtonPanel1.setLayout(new java.awt.GridLayout(0, 1));

    JLabel jNewSpeciesLabel = new JLabel("New species:");
    jNewSpeciesLabel.setFont(new ModelFont());
    jButtonPanel1.add(jNewSpeciesLabel);

    m_jNewSpecies = new JTextField();
    m_jNewSpecies.setFont(new ModelFont());
    jButtonPanel1.add(m_jNewSpecies);

    JButton jButton = new JButton("Add");
    jButton.setFont(new ModelFont());
    jButton.setActionCommand("Add");
    jButton.addActionListener(this);
    jButtonPanel1.add(jButton);

    jButton = new JButton("Rename");
    jButton.setFont(new ModelFont());
    jButton.setActionCommand("Rename");
    jButton.addActionListener(this);
    jButtonPanel1.add(jButton);

    jButton = new JButton("Copy...");
    jButton.setFont(new ModelFont());
    jButton.setActionCommand("Copy");
    jButton.addActionListener(this);
    jButtonPanel1.add(jButton);

    //Panel containing controls for managing existing species
    JPanel jButtonPanel2 = new JPanel();
    jButtonPanel2.setLayout(new java.awt.GridLayout(0, 1));
    jButtonPanel2.setBorder(BorderFactory.createMatteBorder(1, 0, 0, 0,
        java.awt.Color.BLACK));

    jButton = new JButton("Up");
    jButton.setFont(new ModelFont());
    jButton.setActionCommand("Up");
    jButton.addActionListener(this);
    jButtonPanel2.add(jButton);

    jButton = new JButton("Down");
    jButton.setFont(new ModelFont());
    jButton.setActionCommand("Down");
    jButton.addActionListener(this);
    jButtonPanel2.add(jButton);

    jButton = new JButton("Remove");
    jButton.setFont(new ModelFont());
    jButton.setActionCommand("Remove");
    jButton.addActionListener(this);
    jButtonPanel2.add(jButton);

    //Put the two button panels together into a single panel
    JPanel jButtonPanel = new JPanel();
    jButtonPanel.setLayout(new java.awt.BorderLayout());
    jButtonPanel.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 0));
    jButtonPanel.add(jButtonPanel1, java.awt.BorderLayout.NORTH);
    jButtonPanel.add(new JPanel(), java.awt.BorderLayout.CENTER);
    jButtonPanel.add(jButtonPanel2, java.awt.BorderLayout.SOUTH);

    //Panel packaging together all species components
    JLabel jSpeciesLabel = new JLabel("Edit List of Species");
    jSpeciesLabel.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 14));

    JPanel jSpeciesCenterPanel = new JPanel();
    jSpeciesCenterPanel.setLayout(new java.awt.BorderLayout());
    jSpeciesCenterPanel.add(jScroller, java.awt.BorderLayout.CENTER);
    jSpeciesCenterPanel.add(jButtonPanel, java.awt.BorderLayout.EAST);

    JPanel jSpeciesPanel = new JPanel();
    jSpeciesPanel.setLayout(new java.awt.BorderLayout());
    jSpeciesPanel.add(jSpeciesLabel, java.awt.BorderLayout.PAGE_START);
    jSpeciesPanel.add(jSpeciesCenterPanel, java.awt.BorderLayout.CENTER);
    jSpeciesPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 5, 0));

    //Panel with buttons with additional tree tasks
    JPanel jAdditionalTasksPanel = new JPanel();
    jAdditionalTasksPanel.setLayout(new java.awt.GridLayout(0, 1));

    JButton jSizeClasses = new JButton("Edit Initial Densities Size Classes...");
    jSizeClasses.setFont(new ModelFont());
    jSizeClasses.setActionCommand("EditSizeClasses");
    jSizeClasses.addActionListener(this);

    m_jClearTreeMap = new JButton();
    m_jClearTreeMap.setFont(new ModelFont());
    m_jClearTreeMap.setActionCommand("ClearTreeMap");
    m_jClearTreeMap.addActionListener(this);

    m_jWriteTreeMap = new JButton();
    m_jWriteTreeMap.setFont(new ModelFont());
    m_jWriteTreeMap.setActionCommand("WriteTreeMap");
    m_jWriteTreeMap.addActionListener(this);

    JButton jTreeMapButton = new JButton("Add tree map");
    jTreeMapButton.setFont(new ModelFont());
    jTreeMapButton.setActionCommand("AddTreeMap");
    jTreeMapButton.addActionListener(this);

    if (m_oPop.mp_oTrees.size() == 0) {
      m_jClearTreeMap.setText("No tree map trees");
      m_jClearTreeMap.setEnabled(false);

      m_jWriteTreeMap.setText("No tree map trees");
      m_jWriteTreeMap.setEnabled(false);
    }
    else {
      m_jClearTreeMap.setText("Remove tree map trees");
      m_jWriteTreeMap.setText("Write tree map to text file");
    }

    jAdditionalTasksPanel.add(jSizeClasses);
    jAdditionalTasksPanel.add(jTreeMapButton);
    jAdditionalTasksPanel.add(m_jClearTreeMap);
    jAdditionalTasksPanel.add(m_jWriteTreeMap);
    jAdditionalTasksPanel.setBorder(BorderFactory.createMatteBorder(1, 0, 0, 0,
        java.awt.Color.BLACK));

    //Panel containing the components of the window
    JPanel jCenterPanel = new JPanel();
    jCenterPanel.setLayout(new BoxLayout(jCenterPanel, BoxLayout.PAGE_AXIS));
    jCenterPanel.add(jSpeciesPanel);
    jCenterPanel.add(jAdditionalTasksPanel);

    //Put everything together in the GUI
    JPanel jContentPanel = (JPanel) getContentPane();
    jContentPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
    getContentPane().setLayout(new java.awt.BorderLayout());
    getContentPane().add(jCenterPanel, java.awt.BorderLayout.CENTER);
    getContentPane().add(new OKCancelButtonPanel(this,
                                                 oPop.m_oManager.GetHelpBroker(),
                                                 m_sHelpID),
                         java.awt.BorderLayout.PAGE_END);

    //Create the size class list model - although it won't get used right
    //away
    m_jSizeClassListModel = new DefaultListModel();
    for (i = 0; i < m_oPop.GetNumberOfSizeClasses(); i++) {
      m_jSizeClassListModel.addElement(m_oPop.GetSizeClass(i));
    }

    m_jNewSpecies.requestFocus();
  }

  /**
   * Moves selected species up.
   */
  private void MoveUp() {
    //Move the selected species up in the list
    Object[] p_oSelected = m_jSpeciesList.getSelectedValues();
    if (null == p_oSelected || 0 == p_oSelected.length) {
      JOptionPane.showMessageDialog(this, "Please select a species.");
      return;
    }
    int[] p_iNewIndexes = new int[p_oSelected.length];
    int iSelectedIndex, iNewIndex, i;
    for (i = 0; i < p_oSelected.length; i++) {

      //Get the current index of the item, and its proposed new index
      iSelectedIndex = m_jSpeciesListModel.indexOf(p_oSelected[i]);
      iNewIndex = iSelectedIndex - 1;

      //Make sure the first item is not selected
      if (0 != iSelectedIndex) {

        //Swap the object from old position to new
        m_jSpeciesListModel.insertElementAt(p_oSelected[i],
                                            iNewIndex);
        m_jSpeciesListModel.removeElementAt(iSelectedIndex + 1);
      }
    }

    //Keep the same item(s) selected
    for (i = 0; i < p_oSelected.length; i++) {
      p_iNewIndexes[i] = m_jSpeciesListModel.indexOf(p_oSelected[i]);
    }
    m_jSpeciesList.setSelectedIndices(p_iNewIndexes);
  }

  /**
   * Moves selected species down.
   */
  private void MoveDown() {
    //Move the selected species down in the list
    Object[] p_oSelected = m_jSpeciesList.getSelectedValues();
    if (null == p_oSelected || 0 == p_oSelected.length) {
      JOptionPane.showMessageDialog(this, "Please select a species.");
      return;
    }
    int[] p_iNewIndexes = new int[p_oSelected.length];
    int iSelectedIndex, iNewIndex, i;
    for (i = p_oSelected.length - 1; i >= 0; i--) {

      //Get the current index of the item, and its proposed new index
      iSelectedIndex = m_jSpeciesListModel.indexOf(p_oSelected[i]);
      iNewIndex = iSelectedIndex + 2;

      //Make sure the last item is not selected
      if ( (m_jSpeciesListModel.getSize() - 1) != iSelectedIndex) {
        //Swap the object from old position to new
        m_jSpeciesListModel.insertElementAt(p_oSelected[i],
                                            iNewIndex);
        m_jSpeciesListModel.removeElementAt(iSelectedIndex);
      }
    }

    //Keep the same item(s) selected
    for (i = 0; i < p_oSelected.length; i++) {
      p_iNewIndexes[i] = m_jSpeciesListModel.indexOf(p_oSelected[i]);
    }
    m_jSpeciesList.setSelectedIndices(p_iNewIndexes);
  }

  /**
   * Performs a species add in response to the Add button click.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  protected void AddSpecies() {
    //Is there anything in the species editor field?
    String sNewSpecies = m_jNewSpecies.getText();
    sNewSpecies = sNewSpecies.trim();

    if (sNewSpecies.length() == 0) {
      JOptionPane.showMessageDialog(this,
                                    "Please enter the name of the new species.");
      return;
    }

    //Is the new species already present?
    int i;
    for (i = 0; i < m_jSpeciesListModel.size(); i++) {
      if (sNewSpecies.equals( (String) m_jSpeciesListModel.elementAt(i))) {
        JOptionPane.showMessageDialog(this,
                                      "There is already a species of that name.");
        return;
      }
    }

    //Add it
    m_jSpeciesListModel.addElement(sNewSpecies);

    m_jNewSpecies.setText("");

    m_jNewSpecies.requestFocus();
  }

  /** Removes a species. This verifiies the remove operation. Then, if valid,
   * removes the species. This will also make sure that any rename or copy
   * pairs featuring the removed species are also removed.
   */
  protected void RemoveSpecies() {

    //Verify the remove
    Object[] p_oSelected = m_jSpeciesList.getSelectedValues();
    String sRemoveSpecies;
    RenamePair oPair;
    int i, j;
    if (p_oSelected == null || p_oSelected.length == 0) {
      JOptionPane.showMessageDialog(this,
                                    "Please select a species to delete.");
      return; //Nothing selected
    }

    //For each species selected, make sure it is removed from any rename or
    //copy pairs
    for (i = 0; i < p_oSelected.length; i++) {
      sRemoveSpecies = (String) p_oSelected[i];

      for (j = 0; j < mp_oCopySpecies.size(); j++) {
        oPair = (RenamePair) mp_oCopySpecies.elementAt(j);
        if (oPair.m_sNewName.equals(sRemoveSpecies) ||
            oPair.m_sOldName.equals(sRemoveSpecies)) {
          mp_oCopySpecies.remove(j);
          j--;
        }
      }

      for (j = 0; j < mp_oRenameSpecies.size(); j++) {
        oPair = (RenamePair) mp_oRenameSpecies.elementAt(j);
        if (oPair.m_sNewName.equals(sRemoveSpecies) ||
            oPair.m_sOldName.equals(sRemoveSpecies)) {
          mp_oRenameSpecies.remove(j);
          j--;
        }
      }
    }

    //Do the species remove
    for (i = 0; i < p_oSelected.length; i++) {
      m_jSpeciesListModel.removeElement(p_oSelected[i]);
    }
  }

  /**
   * Puts a species to rename in the list of renamed species, and updates the
   * species list.
   */
  protected void RenameSpecies() {
    //Make sure the user selected a species to rename - and only one
    if (m_jSpeciesList.getSelectedIndices().length > 1) {
      JOptionPane.showMessageDialog(this,
                                    "You can only rename one species at a time.");
      return;
    }
    int iSelectedIndex = m_jSpeciesList.getSelectedIndex();
    if (iSelectedIndex == -1) {
      JOptionPane.showMessageDialog(this, "Please select a species to rename.");
      return; //Nothing selected
    }
    String sOldName = (String) m_jSpeciesList.getSelectedValue();

    //Get the new species name
    String sNewName = m_jNewSpecies.getText().trim();
    if (sNewName.length() == 0) {
      JOptionPane.showMessageDialog(this,
                                    "Please enter the new name of the species.");
      return;
    }

    //Is the new species already present?
    int i;
    for (i = 0; i < m_jSpeciesListModel.size(); i++) {
      if (sNewName.equals( (String) m_jSpeciesListModel.elementAt(i))) {
        JOptionPane.showMessageDialog(this,
                                      "There is already a species of that name.");
        m_jNewSpecies.setText("");
        m_jNewSpecies.requestFocus();
        return;
      }
    }

    //Has this species already been put on the list to rename?  If so, update
    //the pair and exit
    RenamePair oPair;
    boolean bUpdated = false;
    for (i = 0; i < mp_oRenameSpecies.size(); i++) {
      oPair = (RenamePair) mp_oRenameSpecies.elementAt(i);
      if (oPair.m_sNewName.equals(sOldName)) {
        oPair.m_sNewName = sNewName;
        bUpdated = true;
        break;
      }
    }

    if (false == bUpdated) {
      //Put this species in the list to rename
      oPair = new RenamePair();
      oPair.m_sNewName = sNewName;
      oPair.m_sOldName = sOldName;
      mp_oRenameSpecies.add(oPair);
    }

    //Redisplay the species list
    m_jSpeciesListModel.removeElementAt(iSelectedIndex);
    m_jSpeciesListModel.add(iSelectedIndex, sNewName);

    m_jNewSpecies.setText("");
    m_jNewSpecies.requestFocus();
  }

  /**
   * Updates the TreePopulation object with the accumulated changes.
   * @throws ModelException if there are no species present.
   * @return int; if 0, close the window. If anything else, don't.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  protected int UpdateTreePopulation() throws ModelException {

    int i, j;

    //First see if ANY change was made
    String sPopName;
    boolean bChanged = false;
    if (mp_oCopySpecies.size() == 0 &&
        mp_oRenameSpecies.size() == 0 &&
        m_jSpeciesListModel.size() == m_oPop.GetNumberOfSpecies() &&
        m_jSizeClassListModel.size() == m_oPop.GetNumberOfSizeClasses()) {

      //There may have been no changes - check
      for (i = 0; i < m_jSpeciesListModel.size(); i++) {
        sPopName = m_oPop.GetSpeciesNameFromCode(i).replace('_', ' ');
        if (false == sPopName.equals( (String) m_jSpeciesListModel.elementAt(i))) {
          bChanged = true;
          break;
        }
      }

      if (false == bChanged) {
        for (i = 0; i < m_jSizeClassListModel.size(); i++) {
          if (m_oPop.GetSizeClass(i).compareTo( (Float) m_jSizeClassListModel.
                                               elementAt(i)) != 0) {
            bChanged = true;
            break;
          }
        }
      }
    }
    else {
      bChanged = true;
    }

    if (bChanged == false) {
      return 0;
    }

    //Check to see if species were changed. If not, it's easier
    boolean bSppChange = false, bRenamed;
    RenamePair oPair;
    if (m_jSpeciesListModel.size() != m_oPop.GetNumberOfSpecies() ||
        mp_oCopySpecies.size() > 0) {
      bSppChange = true;
    } else {
      for (i = 0; i < m_jSpeciesListModel.size(); i++) {
        sPopName = m_oPop.GetSpeciesNameFromCode(i).replace('_', ' ');
        if (false == sPopName.equals( (String) m_jSpeciesListModel.elementAt(i))) {
          bRenamed = false;
          for (j = 0; j < mp_oRenameSpecies.size(); j++) {
            oPair = (RenamePair) mp_oRenameSpecies.elementAt(j);
            if (oPair.m_sOldName.equals(sPopName)) {
              bRenamed = true;
              break;
            }
          }
          if (false == bRenamed) {
            bSppChange = true;
            break;
          }
        }
      }
    }

    //Before we continue and actually make any changes, if we are making species
    //changes, warn the user that all grid and tree map data will be lost
    if (true == bSppChange) {
      int iReturn = JOptionPane.showConfirmDialog(this,
                                                  "Warning: any currently loaded tree map or grid\n" +
                                                  "map data will be lost. Continue?",
                                                  "SORTIE",
                                                  JOptionPane.YES_NO_OPTION,
                                                  JOptionPane.WARNING_MESSAGE);
      if (iReturn == JOptionPane.NO_OPTION) {
        return -1;
      }
    }

    //Perform any renamings
    for (i = 0; i < mp_oRenameSpecies.size(); i++) {
      oPair = (RenamePair) mp_oRenameSpecies.elementAt(i);
      m_oPop.ChangeSpeciesName(oPair.m_sOldName, oPair.m_sNewName);
    }

    if (bSppChange) {

      //Double-check to make sure there was a change in the species list
      //before calling an update
      if (m_jSpeciesListModel.size() == m_oPop.GetNumberOfSpecies()) {
        for (i = 0; i < m_jSpeciesListModel.size(); i++) {
          if (!m_oPop.GetSpeciesNameFromCode(i).replace('_',
              ' ').equals( (String) m_jSpeciesListModel.elementAt(i))) {
            bChanged = true;
            break;
          }
        }
      }
      else {
        bChanged = true;
      }

      if (bChanged == true) {
        //Translate the species into an array of strings
        String[] p_sSpeciesNames = new String[m_jSpeciesListModel.size()];
        for (i = 0; i < p_sSpeciesNames.length; i++) {
          p_sSpeciesNames[i] = (String) m_jSpeciesListModel.elementAt(i);
        }

        m_oPop.SetSpeciesNames(p_sSpeciesNames);
      }

      //If there were species copyings, do them now
      for (i = 0; i < mp_oCopySpecies.size(); i++) {
        oPair = (RenamePair) mp_oCopySpecies.elementAt(i);
        m_oPop.DoCopySpecies(oPair.m_sOldName.replace(' ', '_'), oPair.m_sNewName.replace(' ', '_'));
      }
    }

    //Now the initial densities - same game
    //Double-check to make sure there was a change in the size class list
    //before calling an update
    bChanged = false;
    if (m_jSizeClassListModel.size() == m_oPop.GetNumberOfSizeClasses()) {
      for (i = 0; i < m_jSizeClassListModel.size(); i++) {
        if (m_oPop.GetSizeClass(i).compareTo( (Float) m_jSizeClassListModel.
                                             elementAt(i)) != 0) {
          bChanged = true;
          break;
        }
      }
    }
    else {
      bChanged = true;
    }

    if (bChanged == true) {
      //Translate the size classes into an array of Floats
      Float[] p_fSizeClasses = new Float[m_jSizeClassListModel.size()];
      for (i = 0; i < p_fSizeClasses.length; i++) {
        p_fSizeClasses[i] = (Float) m_jSizeClassListModel.elementAt(i);
      }

      m_oPop.SetSizeClasses(p_fSizeClasses);
    }
    return 0;
  }

  /**
   * Controls actions for this window.
   * @param oEvent ActionEvent.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void actionPerformed(ActionEvent oEvent) {
    String sCommand = oEvent.getActionCommand();
    if (sCommand.equals("Cancel")) {
      this.setVisible(false);
      this.dispose();
    }
    else if (sCommand.equals("OK")) {
      try {

        if (0 == UpdateTreePopulation()) {

          //Let the GUI manager know that some updates were made and if there's
          //currently no parameter file, that one could be made now
          m_oPop.m_oManager.ProposeNewParameterFile();
          this.setVisible(false);
          this.dispose();
        }
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
    }
    else if (sCommand.equals("Up")) {
      MoveUp();
    }
    else if (sCommand.equals("Down")) {
      MoveDown();
    }
    else if (sCommand.equals("Add")) {
      //Request to add a species
      AddSpecies();
    }
    else if (sCommand.equals("Copy")) {
      CopySpeciesEditor oEditor = new CopySpeciesEditor(this);
      oEditor.pack();
      oEditor.setLocationRelativeTo(null);
      oEditor.setVisible(true);
    }
    else if (sCommand.equals("Remove")) {
      RemoveSpecies();
    }
    else if (sCommand.equals("Rename")) {
      RenameSpecies();
    }
    else if (sCommand.equals("EditSizeClasses")) {
      SizeClassEditor oEditor = new SizeClassEditor(this,
          m_oPop.m_oManager.GetHelpBroker());
      oEditor.pack();
      oEditor.setLocationRelativeTo(null);
      oEditor.setVisible(true);
    }
    else if (sCommand.equals("ClearTreeMap")) {
      m_oPop.ClearTrees();
      m_jClearTreeMap.setText("No tree map trees");
      m_jClearTreeMap.setEnabled(false);
      m_jWriteTreeMap.setText("No tree map trees");
      m_jWriteTreeMap.setEnabled(false);
    }
    else if (sCommand.equals("WriteTreeMap")) {
      try {
        WriteTreeMap();
      }
      catch (ModelException oExp) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oExp);
      }
    }
    else if (sCommand.equals("AddTreeMap")) {
      try {
        ModelFileChooser jChooser = new ModelFileChooser();
        jChooser.setFileFilter(new TextFileFilter());
        int returnVal = jChooser.showOpenDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
          //User chose a file
          java.io.File oFile = jChooser.getSelectedFile();
          String sFileName = oFile.getAbsolutePath();
          this.setCursor(java.awt.Cursor.getPredefinedCursor(java.awt.Cursor.
              WAIT_CURSOR));
          //Make sure a parameter file has been loaded
          m_oPop.m_oManager.InputTreeMap(sFileName);
          m_oPop.m_oManager.GetMainWindow().m_jMessagesField.setText(
              "Added tree map " + sFileName);
          if (m_oPop.mp_oTrees.size() == 0) {
            m_jClearTreeMap.setText("No tree map trees");
            m_jClearTreeMap.setEnabled(false);

            m_jWriteTreeMap.setText("No tree map trees");
            m_jWriteTreeMap.setEnabled(false);
          }
          else {
            m_jClearTreeMap.setText("Remove tree map trees");
            m_jClearTreeMap.setEnabled(true);

            m_jWriteTreeMap.setText("Write tree map to text file");
            m_jWriteTreeMap.setEnabled(true);
          }
        }
      }
      catch (ModelException oErr) {
        ErrorGUI oHandler = new ErrorGUI(this);
        oHandler.WriteErrorMessage(oErr);
      }
      finally {
        this.setCursor(java.awt.Cursor.getPredefinedCursor(java.awt.Cursor.
            DEFAULT_CURSOR));
      }
    }
  }

  /**
   * Writes the tree map trees to a tab-delimited text file.  It first displays
   * to the user a dialog for specifying the file name.  It then organizes
   * a 2-D array of strings for all possible tree data members that have been
   * defined.  Each tree contributes its applicable values to the table.  Then
   * the table is written out.
   * @throws ModelException if the file cannot be written.
   */
  protected void WriteTreeMap() throws ModelException {
    try {

      //***************************
       // Get the user to select their filename
       //**************************
        ModelFileChooser jChooser = new ModelFileChooser();
      jChooser.setFileFilter(new TextFileFilter());
      String sFileName;

      int returnVal = jChooser.showSaveDialog(this);
      if (returnVal != JFileChooser.APPROVE_OPTION) {
        return;
      }

      //User chose a file - does it already exist?
      java.io.File oFile = jChooser.getSelectedFile();
      if (oFile.exists()) {
        returnVal = JOptionPane.showConfirmDialog(this,
                                           "Do you wish to overwrite the existing file?",
                                           "Model",
                                           JOptionPane.YES_NO_OPTION);
        if (returnVal == JOptionPane.NO_OPTION) {
          return;
        }
      }

      sFileName = oFile.getAbsolutePath();
      if (!sFileName.endsWith(".txt")) {
        sFileName = sFileName.concat(".txt");
      }

      //***************************
       // Compile the list of tree data members into a vector - this will
       // become the table header
       //**************************
        java.util.Vector p_oColumnHeader = new java.util.Vector(2);
        //These are distances to the first column in the table of each type,
        //starting at zero.  The first two columns are tree species and type.
      int iStartFloatIndex;
      int iStartIntIndex;
      int iStartCharIndex;
      int iStartBoolIndex;
      int i, j, k, m;

      //Add species and type
      p_oColumnHeader.add("Species");
      p_oColumnHeader.add("Type");

      //Now add each data member uniquely
      String sTemp, sTemp2;
      boolean bFound;

      //Floats
      iStartFloatIndex = p_oColumnHeader.size();
      for (i = 0; i < m_oPop.mp_sTreeFloatDataMembers.length; i++) {
        for (j = 0; j < m_oPop.mp_sTreeFloatDataMembers[i].length; j++) {
          for (k = 0; k < m_oPop.mp_sTreeFloatDataMembers[i][j].size(); k++) {
            sTemp2 = (String) m_oPop.mp_sTreeFloatDataMembers[i][j].elementAt(k);
            bFound = false;
            for (m = 0; m < p_oColumnHeader.size(); m++) {
              sTemp = (String) p_oColumnHeader.elementAt(m);
              if (sTemp.equals(sTemp2)) {
                bFound = true;
                break;
              }
            }
            if (!bFound) {
              p_oColumnHeader.add(sTemp2);
            }
          }
        }
      }

      //Ints
      iStartIntIndex = p_oColumnHeader.size();
      for (i = 0; i < m_oPop.mp_sTreeIntDataMembers.length; i++) {
        for (j = 0; j < m_oPop.mp_sTreeIntDataMembers[i].length; j++) {
          for (k = 0; k < m_oPop.mp_sTreeIntDataMembers[i][j].size(); k++) {
            sTemp2 = (String) m_oPop.mp_sTreeIntDataMembers[i][j].elementAt(k);
            bFound = false;
            for (m = 0; m < p_oColumnHeader.size(); m++) {
              sTemp = (String) p_oColumnHeader.elementAt(m);
              if (sTemp.equals(sTemp2)) {
                bFound = true;
                break;
              }
            }
            if (!bFound) {
              p_oColumnHeader.add(sTemp2);
            }
          }
        }
      }

      //Bools
      iStartBoolIndex = p_oColumnHeader.size();
      for (i = 0; i < m_oPop.mp_sTreeBoolDataMembers.length; i++) {
        for (j = 0; j < m_oPop.mp_sTreeBoolDataMembers[i].length; j++) {
          for (k = 0; k < m_oPop.mp_sTreeBoolDataMembers[i][j].size(); k++) {
            sTemp2 = (String) m_oPop.mp_sTreeBoolDataMembers[i][j].elementAt(k);
            bFound = false;
            for (m = 0; m < p_oColumnHeader.size(); m++) {
              sTemp = (String) p_oColumnHeader.elementAt(m);
              if (sTemp.equals(sTemp2)) {
                bFound = true;
                break;
              }
            }
            if (!bFound) {
              p_oColumnHeader.add(sTemp2);
            }
          }
        }
      }

      //Chars
      iStartCharIndex = p_oColumnHeader.size();
      for (i = 0; i < m_oPop.mp_sTreeCharDataMembers.length; i++) {
        for (j = 0; j < m_oPop.mp_sTreeCharDataMembers[i].length; j++) {
          for (k = 0; k < m_oPop.mp_sTreeCharDataMembers[i][j].size(); k++) {
            sTemp2 = (String) m_oPop.mp_sTreeCharDataMembers[i][j].elementAt(k);
            bFound = false;
            for (m = 0; m < p_oColumnHeader.size(); m++) {
              sTemp = (String) p_oColumnHeader.elementAt(m);
              if (sTemp.equals(sTemp2)) {
                bFound = true;
                break;
              }
            }
            if (!bFound) {
              p_oColumnHeader.add(sTemp2);
            }
          }
        }
      }

      //*******************************
       //Now create the table of values to write out
       //******************************

        String[][] p_sTable = new String[m_oPop.mp_oTrees.size()][
            p_oColumnHeader.size()];
      String[] p_sColumnHeader = new String[p_oColumnHeader.size()];
      for (i = 0; i < p_sColumnHeader.length; i++) {
        p_sColumnHeader[i] = (String) p_oColumnHeader.elementAt(i);

      }
      Tree oTree;
      for (i = 0; i < m_oPop.mp_oTrees.size(); i++) {
        oTree = (Tree) m_oPop.mp_oTrees.elementAt(i);

        //Species and type
        p_sTable[i][0] = m_oPop.GetSpeciesNameFromCode(oTree.m_iSpecies).
            replace('_', ' ');
        p_sTable[i][1] = TreePopulation.GetTypeNameFromCode(oTree.m_iType);

        for (j = 0; j < oTree.mp_fFloats.length; j++) {
          if (oTree.mp_fFloats[j] != null) {
            //Get the name for this data member
            sTemp = (String) m_oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][
                oTree.m_iType].elementAt(j);
            //Find its column in the table
            for (k = iStartFloatIndex; k < p_sColumnHeader.length; k++) {
              if (p_sColumnHeader[k].equals(sTemp)) {
                p_sTable[i][k] = String.valueOf(oTree.mp_fFloats[j]);
                break;
              }
            }
          }
        }

        for (j = 0; j < oTree.mp_iInts.length; j++) {
          if (oTree.mp_iInts[j] != null) {
            //Get the name for this data member
            sTemp = (String) m_oPop.mp_sTreeIntDataMembers[oTree.m_iSpecies][
                oTree.m_iType].elementAt(j);
            //Find its column in the table
            for (k = iStartIntIndex; k < p_sColumnHeader.length; k++) {
              if (p_sColumnHeader[k].equals(sTemp)) {
                p_sTable[i][k] = String.valueOf(oTree.mp_iInts[j]);
                break;
              }
            }
          }
        }

        for (j = 0; j < oTree.mp_bBools.length; j++) {
          if (oTree.mp_bBools[j] != null) {
            //Get the name for this data member
            sTemp = (String) m_oPop.mp_sTreeBoolDataMembers[oTree.m_iSpecies][
                oTree.m_iType].elementAt(j);
            //Find its column in the table
            for (k = iStartBoolIndex; k < p_sColumnHeader.length; k++) {
              if (p_sColumnHeader[k].equals(sTemp)) {
                p_sTable[i][k] = String.valueOf(oTree.mp_bBools[j]);
                break;
              }
            }
          }
        }

        for (j = 0; j < oTree.mp_sChars.length; j++) {
          if (oTree.mp_sChars[j] != null) {
            //Get the name for this data member
            sTemp = (String) m_oPop.mp_sTreeCharDataMembers[oTree.m_iSpecies][
                oTree.m_iType].elementAt(j);
            //Find its column in the table
            for (k = iStartCharIndex; k < p_sColumnHeader.length; k++) {
              if (p_sColumnHeader[k].equals(sTemp)) {
                p_sTable[i][k] = String.valueOf(oTree.mp_sChars[j]);
                break;
              }
            }
          }
        }
      }

      //*******************************
       //Write the file
       //******************************
        java.io.FileWriter oOut = new java.io.FileWriter(sFileName);
      oOut.write("Tree map for parameter file " +
                 m_oPop.m_oManager.GetParameterFileName() + "\n");

      //Header
      if (p_oColumnHeader.size() > 0) {
        oOut.write( (String) p_oColumnHeader.elementAt(0));
      }
      for (i = 1; i < p_oColumnHeader.size(); i++) {
        oOut.write("\t" + (String) p_oColumnHeader.elementAt(i));
      }

      //Values
      for (i = 0; i < p_sTable.length; i++) {
        oOut.write("\n");
        if (p_sTable[i].length > 0) {
          if (p_sTable[i][0] == null) {
            oOut.write("--");
          }
          else {
            oOut.write(p_sTable[i][0]);
          }
        }
        for (j = 1; j < p_sTable[i].length; j++) {
          if (p_sTable[i][j] == null) {
            oOut.write("\t--");
          }
          else {
            oOut.write("\t" + p_sTable[i][j]);
          }
        }
      }

      oOut.close();

      //Tell the user
      JOptionPane.showMessageDialog(this, "File has been saved.");

    }
    catch (java.io.IOException oExp) {
      throw (new ModelException(ErrorGUI.BAD_FILE, "JAVA",
                                "There was a problem writing the file."));
    }
  }

  /**
   * Displays dialog for entering size class data.
   * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
   * <p>Company: Institute of Ecosystem Studies</p>
   * @author Lora E. Murphy
   * @version 1.0
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  protected class SizeClassEditor
      extends JDialog
      implements ActionListener {

    /**List displaying the size classes*/
    protected JList m_jSizeClassList;

    /**Copy of size class list model*/
    protected DefaultListModel m_jSizeClassListModelCopy;

    /**Field for entering new size class*/
    protected JTextField m_jNewClass;

    /**Help ID string*/
    private String m_sHelpID = "windows.edit_size_classes";

    /**
     * Constructor.  Creates GUI.
     * @param jParent Parent dialog in which to display this one.
     * @param oHelpBroker Application's help broker.
     *
     * <br>Edit history:
     * <br>------------------
     * <br>April 28, 2004: Submitted in beta version (LEM)
     */
    public SizeClassEditor(JDialog jParent, HelpBroker oHelpBroker) {
      super(jParent, "Edit Size Classes", true);

      //Help ID
      oHelpBroker.enableHelpKey(this.getRootPane(), m_sHelpID, null);

      //List of size classes
      m_jSizeClassListModelCopy = new DefaultListModel();
      Float fVal;
      int i;
      for (i = 0; i < m_jSizeClassListModel.size(); i++) {
        fVal = (Float) m_jSizeClassListModel.elementAt(i);
        m_jSizeClassListModelCopy.addElement(new Float(fVal.floatValue()));
      }

      m_jSizeClassList = new JList(m_jSizeClassListModelCopy);
      m_jSizeClassList.setFont(new ModelFont());
      JScrollPane jScroller = new JScrollPane(m_jSizeClassList);

      //Panel with add/remove buttons
      JPanel jButtonPanel = new JPanel();

      JLabel jNewSpeciesLabel = new JLabel("New size class upper limit DBH");
      jNewSpeciesLabel.setFont(new ModelFont());
      JLabel jNewSpeciesLabel2 = new JLabel("(0 for seedlings):");
      jNewSpeciesLabel2.setFont(new ModelFont());

      m_jNewClass = new JTextField();
      m_jNewClass.setFont(new ModelFont());

      JButton jAddButton = new JButton("Add");
      jAddButton.setFont(new ModelFont());
      jAddButton.setActionCommand("Add");
      jAddButton.addActionListener(this);

      JButton jRemoveButton = new JButton("Remove");
      jRemoveButton.setFont(new ModelFont());
      jRemoveButton.setActionCommand("Remove");
      jRemoveButton.addActionListener(this);

      jButtonPanel.setLayout(new java.awt.GridLayout(0, 1));
      jButtonPanel.add(jNewSpeciesLabel);
      jButtonPanel.add(jNewSpeciesLabel2);
      jButtonPanel.add(m_jNewClass);
      jButtonPanel.add(jAddButton);
      jButtonPanel.add(jRemoveButton);

      //Package components into panel
      JPanel jCenterPanel = new JPanel();
      jCenterPanel.setLayout(new java.awt.FlowLayout());
      jCenterPanel.add(jScroller);
      jCenterPanel.add(jButtonPanel);

      //Finish the GUI
      getContentPane().setLayout(new java.awt.BorderLayout());
      getContentPane().add(jCenterPanel, java.awt.BorderLayout.CENTER);
      getContentPane().add(new OKCancelButtonPanel(this, oHelpBroker, m_sHelpID),
                           java.awt.BorderLayout.PAGE_END);
    }

    /**
     * Performs a size class add in response to the Add button click.
     * @throws ModelException if the value is not a number or is negative.
     *
     * <br>Edit history:
     * <br>------------------
     * <br>April 28, 2004: Submitted in beta version (LEM)
     */
    protected void AddClass() throws ModelException {

      //Is there anything in the class editor field?
      String sNewClass = m_jNewClass.getText();
      sNewClass = sNewClass.trim();

      if (sNewClass.length() == 0) {
        return;
      }

      //Transform to a number
      Float fNewClass = null;
      try {
        fNewClass = new Float(sNewClass);
      }
      catch (java.lang.NumberFormatException oErr) {
        throw (new ModelException(ErrorGUI.BAD_ARGUMENT, "JAVA",
                                  "Size classes must be a number."));
      }

      //Is the number negative?
      if (fNewClass.floatValue() < 0) {
        throw (new ModelException(ErrorGUI.BAD_ARGUMENT, "JAVA",
                                  "Size classes cannot be less than zero."));
      }

      //Is the new class already present?
      int i;
      for (i = 0; i < m_jSizeClassListModelCopy.size(); i++) {
        if (fNewClass.equals( (Float) m_jSizeClassListModelCopy.elementAt(i))) {
          return;
        }
      }

      if (m_jSizeClassListModelCopy.size() == 0) {
        m_jSizeClassListModelCopy.addElement(fNewClass);
        return;
      }

      //Add it in order
      for (i = 0; i < m_jSizeClassListModelCopy.size(); i++) {
        if (fNewClass.compareTo( (Float) m_jSizeClassListModelCopy.elementAt(i)) <
            0) {
          //This should be added just before
          m_jSizeClassListModelCopy.add(i, fNewClass);
          return;
        }
      }

      //This belongs in the last position
      m_jSizeClassListModelCopy.addElement(fNewClass);
    }

    /**
     * Copies changes to the size class list made in this editor to
     * m_jSizeClassListModelCopy into m_jSizeClassListModel.
     *
     * <br>Edit history:
     * <br>------------------
     * <br>April 28, 2004: Submitted in beta version (LEM)
     */
    protected void UpdateSizeClasses() {
      Float fVal;
      int i;
      m_jSizeClassListModel.clear();

      for (i = 0; i < m_jSizeClassListModelCopy.size(); i++) {
        fVal = (Float) m_jSizeClassListModelCopy.elementAt(i);
        m_jSizeClassListModel.addElement(new Float(fVal.floatValue()));
      }
    }

    /**
     * Responds to button clicks
     * @param oEvent Event fired by button click
     *
     * <br>Edit history:
     * <br>------------------
     * <br>April 28, 2004: Submitted in beta version (LEM)
     */
    public void actionPerformed(ActionEvent oEvent) {
      if (oEvent.getActionCommand().equals("Cancel")) {
        this.setVisible(false);
        this.dispose();
      }
      else if (oEvent.getActionCommand().equals("Add")) {
        try {
          AddClass();
        }
        catch (ModelException oErr) {
          ErrorGUI oHandler = new ErrorGUI(this);
          oHandler.WriteErrorMessage(oErr);
        }
      }
      else if (oEvent.getActionCommand().equals("Remove")) {
        //Request to remove one or more size classes
        Object[] p_oRemove = m_jSizeClassList.getSelectedValues();
        if (p_oRemove == null || p_oRemove.length == 0) {
          return; //Nothing selected
        }
        int i;
        for (i = 0; i < p_oRemove.length; i++) {
          m_jSizeClassListModelCopy.removeElement(p_oRemove[i]);
        }

      }
      else if (oEvent.getActionCommand().equals("OK")) {
        UpdateSizeClasses();
        this.setVisible(false);
        this.dispose();
      }
    }
  }

  /**
   * This class creates an interface for copying species.
   * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
   * <p>Company: Institute of Ecosystem Studies</p>
   * @author Lora E. Murphy
   * @version 1.0
   *  <br>Edit history:
   * <br>------------------
   * <br>January 30, 2007: Created (LEM)
   */
  protected class CopySpeciesEditor
      extends JDialog
      implements ActionListener {

    /** List displaying the species to create a copy of*/
    private JList m_jSpeciesCopyFromList;

    /** List displaying the species to become the copy*/
    private JList m_jSpeciesCopyToList;

    /** List model for species to create a copy of*/
    private DefaultListModel m_jSpeciesCopyFromListModel = new DefaultListModel();

    /** List model for species to become a copy*/
    private DefaultListModel m_jSpeciesCopyToListModel = new DefaultListModel();
    ;

    /** Parent calling window */
    TreeSetup m_oSetup;

    public CopySpeciesEditor(TreeSetup oSetup) {
      super(oSetup, "Copy Species", true);

      m_oSetup = oSetup;

      //Get the list of species
      JPanel jPanel = new JPanel();
      jPanel.setLayout(new BoxLayout(jPanel, BoxLayout.PAGE_AXIS));
      int i;
      for (i = 0; i < m_jSpeciesListModel.size(); i++) {
        m_jSpeciesCopyFromListModel.addElement(m_jSpeciesListModel.elementAt(i));
        m_jSpeciesCopyToListModel.addElement(m_jSpeciesListModel.elementAt(i));
      }

      //Create the GUI
      m_jSpeciesCopyFromList = new JList(m_jSpeciesCopyFromListModel);
      m_jSpeciesCopyToList = new JList(m_jSpeciesCopyToListModel);
      m_jSpeciesCopyFromList.setFont(new ModelFont());
      m_jSpeciesCopyToList.setFont(new ModelFont());

      JLabel jLabel = new JLabel("Make this species...");
      jLabel.setFont(new ModelFont());
      jPanel.add(jLabel);

      jPanel.add(new JScrollPane(m_jSpeciesCopyToList));

      jLabel = new JLabel("a copy of this species...");
      jLabel.setFont(new ModelFont());
      jPanel.add(jLabel);

      jPanel.add(new JScrollPane(m_jSpeciesCopyFromList));

      JPanel jContentPane = new JPanel(new BorderLayout());
      jContentPane.add(jPanel, BorderLayout.CENTER);
      jContentPane.add(new OKCancelButtonPanel(this, null, "", true),
                       BorderLayout.SOUTH);
      jContentPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
      setContentPane(jContentPane);

    }

    /**
     * Controls actions for this window.
     * @param oEvent ActionEvent
     */
    public void actionPerformed(ActionEvent oEvent) {
      String sCommand = oEvent.getActionCommand();
      if (sCommand.equals("Cancel")) {
        this.setVisible(false);
        this.dispose();
      }
      else if (sCommand.equals("OK")) {
        //Make sure the user selected a species to become a copy
        int[] p_iSelectedCopyTo = m_jSpeciesCopyToList.getSelectedIndices();
        if (null == p_iSelectedCopyTo || p_iSelectedCopyTo.length == 0) {
          JOptionPane.showMessageDialog(this,
                                        "Please select a species to become a copy.");
          return;
        }

        //Get the species that the other species will become a copy of - but
        //first verify that only one was chosen
        if (m_jSpeciesCopyFromList.getSelectedIndices().length > 1) {
          JOptionPane.showMessageDialog(this,
                                        "You can select only one species to be copied.");
          return;
        }
        int iSelectedCopyFrom = m_jSpeciesCopyFromList.getSelectedIndex();
        if (iSelectedCopyFrom == -1) {
          JOptionPane.showMessageDialog(this,
                                        "Please select a species to be copied.");
          return;
        }

        //Has the user requested that a species become a copy of itself?
        int i, j;
        for (i = 0; i < p_iSelectedCopyTo.length; i++) {
          if (p_iSelectedCopyTo[i] == iSelectedCopyFrom) {
            JOptionPane.showMessageDialog(this,
                                        "A species cannot be a copy of itself.");
            return;
          }
        }

        String sCopyFrom = (String)m_jSpeciesCopyFromListModel.elementAt(iSelectedCopyFrom);
        for (i = 0; i < p_iSelectedCopyTo.length; i++) {
          //Put this species in the list to rename
          RenamePair oPair = new RenamePair();
          oPair.m_sNewName = (String)m_jSpeciesCopyToListModel.elementAt(p_iSelectedCopyTo[i]);
          oPair.m_sOldName = sCopyFrom;

          //Make sure this pair doesn't already exist
          boolean bExists = false;
          for (j = 0; j < mp_oRenameSpecies.size(); j++) {
            if (oPair.equals((RenamePair)mp_oCopySpecies.elementAt(j))) {
              bExists = true;
              break;
            }
          }
          if (false == bExists) {
            mp_oCopySpecies.add(oPair);
          }
        }

        this.setVisible(false);
        this.dispose();
      }
    }
  }
}

/**
* This class holds information about species renaming and copying.  It pairs
* the old name of the species with the new name.
*/
class RenamePair {

  /**Old name of the species - with spaces instead of underscores, if
  * applicable*/
  public String m_sOldName;

  /**New name of the species - with spaces instead of underscores, if
  * applicable*/
  public String m_sNewName;

  /**
  * Tests for equality.
  * @param oObject Object Object to test.
  * @return boolean True if equal.
  */
  public boolean equals(Object oObject) {
    if (false == oObject instanceof RenamePair) {
      return false;
    }

    RenamePair oPair = (RenamePair) oObject;
    if (oPair.m_sNewName.equals(m_sNewName) &&
        oPair.m_sOldName.equals(m_sOldName)) {
      return true;
    }

    return false;
  }
}
